/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.ui.picker.BasePicker'); //$NON-NLS-1$

dojo.require('jazz.app.auth'); //$NON-NLS-1$
dojo.require('jazz.ui.Dialog'); //$NON-NLS-1$
dojo.require('jazz.ui.SearchBox'); //$NON-NLS-1$
dojo.require('dojo.NodeList-traverse'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.common.web.ui.picker", "BasePickerMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('com.ibm.relm.common.web.ui.EasyMessageBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.picker.StyledList'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.DialogStatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.picker.OslcPreview'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.picker.LiveSearchBox'); //$NON-NLS-1$
dojo.require('jazz.ui.PaginationControl'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.PaginationControl_Fuzzy'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.picker.ColumnBrowser'); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.ConfigurationUIUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.ConfigurationContext"); //$NON-NLS-1$

(function ()
{
	var splitterHeight_ = 8;

	var PAGE_SIZE_INDEX_KEY = 'relm.searchResults.pageSizeIndex'; //$NON-NLS-1$

	/**
	 * Base class for ProductPicker and LqeArtifact picker.
	 * Both show a search box, with a list of results. Only the format
	 * of the results and the query they run is different.
	 *
	 * This class must be extended with doSearch and createListItem overridden.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.ui.picker.BasePicker', //$NON-NLS-1$
		[dijit._Widget, dijit._Templated],
		{
			widgetsInTemplate: true,
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web', 'ui/picker/templates/BasePicker.html'), //$NON-NLS-1$ //$NON-NLS-2$
			messages: dojo.i18n.getLocalization("com.ibm.relm.common.web.ui.picker", "BasePickerMessages"), //$NON-NLS-1$ //$NON-NLS-2$

			/**
			 * The OK callback called when an item in the list
			 * is selected. The passed argument will be the 'item' from
			 * the search results.
			 */
			onOk: null,

			dialogTitle: '', //$NON-NLS-1$
			dialogIcon: '', //$NON-NLS-1$

			/**
			 * String over the search input field describing either syntax or what to search for.
			 */
			searchLabel: '', //$NON-NLS-1$
			searchTitle: '', //$NON-NLS-1$

			pageNumber: 1,

			dialogWidth: "80%", //$NON-NLS-1$
			subtractHeight: 250,
			minResultHeight: 150,

			constructor: function (args)
			{
				this.submitButtonLabel = args.submitButtonLabel ||this.messages.ok;

				// Use to know if the search changes (after keydown event) and we need to search again.
				this._lastSearchString = ''; //$NON-NLS-1$

				this._searchId = dijit.getUniqueId('ple-BasePicker'); //$NON-NLS-1$
				this._pageSizeId = dijit.getUniqueId('ple-BasePicker'); //$NON-NLS-1$
			},

			_getPageNumberForIndex: function(startIndex) {
				var page = Math.floor(startIndex / this._pager._getPageSize()) + 1;
				return page;
			},

			changePage: function(pager, start, end) {
				this.pageNumber = this._getPageNumberForIndex(start);
				this._doPageChangeSearch();
			},

			_onChangePageSize: function(e) {
				var pageSize = this.getSelectedPageSize();
				this._pager.set('increment', pageSize); //$NON-NLS-1$
				window.localStorage[PAGE_SIZE_INDEX_KEY] = this._pageSizeSelect.selectedIndex;
				this._doFreshSearch();
			},

			/**
			 * Moves to a new page of the results (matching the current page widget).
			 * This behaves differently in that it won't change the list of total or filtered
			 * types since it's logically the same search, just viewing a different part of
			 * the results (in practice, and new search may run).
			 *
			 * This minorly-different search has its own method here since it is overriden
			 * by LqeArttifact picker to do different things for Query vs. full-text searches.
			 */
			_doPageChangeSearch: function() {
				this.willDoSearch();
				var isPageChange = true;
				this.doSearch(this.liveSearchBox.getSearchString(), isPageChange);
			},

			/**
			 * Reruns the current search, going back to the first page of the results if we were on
			 * a later one.
			 *
			 * This minorly-different search has its own method here since it is overriden
			 * by LqeArttifact picker to do different things for Query vs. full-text searches.
			 */
			_doFreshSearch: function() {
				this.liveSearchBox.runSearch();
			},

			getSelectedPageSize: function() {
				var index = this._pageSizeSelect.selectedIndex;
				if (index === -1) return 25;

				var size = this._pageSizeSelect.options[index].value;
				return parseInt(size, 10);
			},

			_resizeHeights: function() {
				// Make some guess to give reasonable sizes in both big and
				// small windows plus some padding around the dialog.
				var viewport = dijit.getViewport();
				var dialogHeight = viewport.h - 130;
				var otherStuff = this.subtractHeight;
				// var otherStuff = 250;
				// Preview will be twice the height of the list of results.
				this._resultListHeight = (dialogHeight - otherStuff) / 2;
				this._previewHeight = this._resultListHeight;

				// Enforce minumum height
				this._resultListHeight = Math.max(this.minResultHeight, this._resultListHeight) + 7;
				this._previewHeight = Math.max(this.minResultHeight, this._previewHeight);
				this._stackedListsHeight = this._resultListHeight + this._previewHeight + splitterHeight_;
				// Use a fixed height for the _stackedListsNode. This lets us only animate the top results
				// lists, but it appears the the bottom list of also animated since it will slide in or out
				// of view of the overflow:hidden stacked lists.
				// Even more padding to ensure borders are visisble.
				dojo.style(this._stackedListsNode, 'height', this._stackedListsHeight + 4 + 'px'); //$NON-NLS-1$ //$NON-NLS-2$
			},

			postCreate: function() {
				this.inherited(arguments);
				this.resultList = new com.ibm.relm.common.web.ui.picker.StyledList({
					placeholder: this.messages.noArtifacts
				});
				this.columnBrowser.pushWidget(this.resultList);

				com.ibm.relm.common.web.PaginationControl_Fuzzy.decorate(this._pager);
				this._pager.set('increment', this.getSelectedPageSize()); //$NON-NLS-1$
				this._pager.onNextClick = dojo.hitch(this, this.changePage);
				this._pager.onPreviousClick = dojo.hitch(this, this.changePage);
				this._pager.relm_toggleFuzzyPresentation(true);

				this.liveSearchBox.delegate = this;

				dojo.addClass(this.messageBox.domNode, 'r-dialog-body'); //$NON-NLS-1$

				this.connect(this.resultList, 'onSelectionChanged', this._selectionChanged); //$NON-NLS-1$
				this.connect(this.resultList, 'onDoubleClickItem', this._onOk); //$NON-NLS-1$
				this.connect(this.resultList, 'onSubmit', this._onOk); //$NON-NLS-1$

				// Will defer to the subclass to actually create the <LI> for the result list.
				this.resultList.setListItemFactory(dojo.hitch(this, this.createListItem));

				this._resizeHeights();

				this._previewVisible = window.localStorage['ple.artifactPicker.showPreview'] !== 'false'; //$NON-NLS-1$ //$NON-NLS-2$
				this.togglePreview(this.supportsPreview && this._previewVisible);

				if (this.supportsPreview) {
					this._splitterNode.style.display = ''; //$NON-NLS-1$
				}

				// Logically this does nothing, but webkit doesn't lay these
				// out right initially. The widths are 100% of the container,
				// but don't account for the padding and will overflow the
				// dialog.
				// Any change to them will fix it. Stright up bug!
				if (dojo.isWebKit) {
					dojo.place(this._filterWrapper, this._stackedListsNode);
				}
			},

			/**
			 * Animates setting the height of the result list.
			 * @param  {number} toHeightPx height in px to set the result list to.
			 * @param  {dojo.Deferred} deferred   a deferred to callback when the animation is done.
			 */
			_animateResultList: function(toHeightPx, deferred) {
				this._anim && this._anim.stop();

				var self = this;
				this._anim = dojo.animateProperty({
					node: this.columnBrowser.domNode,
					duration: 300,
					properties: {
						height: toHeightPx
					},
					onEnd: function() {
						delete self._anim;
						deferred.callback();
					}
				});

				this._anim.play();
			},

			/**
			 * Sets the height of the results lists.
			 * @param {number} heightPx the height to set it to.
			 * @param {boolean} animate  if true, the change in size will be
			 *     animated.
			 * @return {dojo.Deferred} called after height is adjusted. After
			 *     the animation if animated.
			 */
			_setResultListHeight: function(heightPx, animate) {
				var deferred = new dojo.Deferred();

				if (animate) {
					this._animateResultList(heightPx, deferred);
				}
				else {
					this.columnBrowser.domNode.style.height = heightPx + 'px'; //$NON-NLS-1$
					deferred.callback();
				}
				return deferred;
			},

			/**
			 * Shows or hides the preview pane.
			 * @param  {boolean} show    true to show it, false to hide.
			 * @param  {boolean} animate if true, the show or hide will animate with a wipe in/out.
			 *                           If false, shows immediately.
			 */
			togglePreview: function(show, animate) {
				if (!show) {
					this._previewVisible = false;
					window.localStorage['ple.artifactPicker.showPreview'] = 'false'; //$NON-NLS-1$ //$NON-NLS-2$
					dojo.addClass(this._splitterNode, 'ple-BasePicker-splitterClosed'); //$NON-NLS-1$
					dojo.attr(this._splitterTitleNode, 'title', this.messages.showPreview); //$NON-NLS-1$

					var deadOslcPreview = this._oslcPreview;
					delete this._oslcPreview;

					// If now preview, give all height to list
					this._setResultListHeight(this._stackedListsHeight - splitterHeight_, animate).then(function() {
						deadOslcPreview.destroyRecursive();
					});
				}
				else {
					this._previewVisible = true;
					window.localStorage['ple.artifactPicker.showPreview'] = 'true'; //$NON-NLS-1$ //$NON-NLS-2$
					dojo.removeClass(this._splitterNode, 'ple-BasePicker-splitterClosed'); //$NON-NLS-1$
					dojo.attr(this._splitterTitleNode, 'title', this.messages.hidePreview); //$NON-NLS-1$

					if (!this._oslcPreview) {
						this._oslcPreview = new com.ibm.relm.common.web.ui.picker.OslcPreview({});
						this._oslcPreview.placeAt(this._previewCell);
					}

					var self = this;
					this._setResultListHeight(this._resultListHeight, animate).then(function() {
						self.currentList().scrollSelectedIntoView();
					});

					this._oslcPreview.set('heightPx', this._previewHeight); //$NON-NLS-1$

					var item = this.currentList().getSelectedItem();
					this.setPreviewItem(item);
				}
			},

			_togglePreviewOnClick: function(e) {
				dojo.stopEvent(e);
				this.togglePreview(!this._previewVisible, true);
			},

			uninitialize: function() {
				this._oslcPreview && this._oslcPreview.destroyRecursive();
				this._toolbar && this._toolbar.destroyRecursive();
			},

			/**
			 * Subclass must override this. Must return an <LI> element
			 * formatted to show the given item.
			 *
			 * @param item
			 * 		the user selected item. Will be one of the items
			 * 		returned from the doSearch callback.
			 */
			createListItem: function(item) {
				throw new Error('subclass must override'); //$NON-NLS-1$
			},

			/**
			 * Called when the user does a search. Subclasses must
			 * override and return the results via a returned Deferred.
			 *
			 * @param searchString the string to search for. Can include
			 * Lucene search formatting.
			 *
			 * @return must be a dojo.Deferred. The callback should pass an array
			 * of "items". The item can be anything, since it will only be used
			 * to pass to createListItem and to the onOk callback, so it's up to the
			 * subclass to interpret what it means. Normally just a JSON object.
			 * The errback should be called with any error with the XHR object used to
			 * run the search.
			 */
			doSearch: function(searchString, isPageChange) {
				throw new Error('subclass must override'); //$NON-NLS-1$
			},

			_selectionChanged: function(item) {
				if (item) {
					this.submitButton.disabled = false;
					if (this._oslcPreview) {
						var self = this;

						// Delay showing the preview since rendering can slow the browser a lot. We don't
						// want to render a preview after each arrow-down in the dialog when looking through the list.
						// Wait for the user to pause a bit before doing this. The result is that the preview won't updated
						// wile navigating the list quickly, but feels much more responsive, especially in IE.
						clearTimeout(this._previewTimeout);
						this._previewTimeout = setTimeout(function() {
							self.setPreviewItem(item);
						}, 200);
					}
				}
				else {
					this.submitButton.disabled = true;
					this.clearPreview();
				}
			},

			/**
			 * Clears the OSLC preview. Used when selection changes somehow.
			 */
			clearPreview: function() {
				clearTimeout(this._previewTimeout);
				this._oslcPreview && this._oslcPreview.set('resourceUri', null); //$NON-NLS-1$
			},

			/**
			 * Sets the item that should be previewed. Normally this is an item
			 * with a uri property that supports OSLC previews. 
			 * This is overriden by LqeArtifactPicker to add special knowledge for DOORS
			 * baselines.
			 * @param {object} item item to preview.
			 */
			setPreviewItem: function(item) {
				if (!item) return this.clearPreview();
				
				var context = com.ibm.relm.common.web.ui.ConfigurationContext.getGlobalConfigurationContext();
				var itemUri = com.ibm.relm.common.web.ui.ConfigurationUIUtils.getConfigurationResourceUri(item.conceptUri || item.uri, context ? context.configuration : null);
				this._oslcPreview && this._oslcPreview.set('resourceUri', itemUri); //$NON-NLS-1$
				return undefined;
			},			

			startup: function() {
				this.inherited(arguments);

				this._dialog = new jazz.ui.Dialog({
					primaryTitle: this.dialogTitle,
					width: this.dialogWidth || '80%', //$NON-NLS-1$
					destroyContents: true,
					contentNode: this.domNode,
					iconClass: this.iconClass
				});
				
				// Just looks nicer since there is a big white result list--easier to distinguish
				// the dialog parts.
				dojo.addClass(this._dialog.domNode, 'r-gray-background'); //$NON-NLS-1$

				var self = this;
				window.setTimeout(function() {
					self.liveSearchBox.select();
				}, 1);
			},

			/**
			 * LiveSearchBox delegate.
			 */
			onEnter: function() {
				this._onSubmit();
			},

			/**
			 * LiveSearchBox delegate.
			 */
			selectNextResult: function() {
				this.currentList().selectNextRow(false);
			},

			/**
			 * LiveSearchBox delegate.
			 */
			selectPreviousResult: function() {
				this.currentList().selectNextRow(true);
			},

			/**
			 * LiveSearchBox delegate.
			 */
			deselectResults: function() {
				// Always uses primary result list since a `clear` scrolls back to it anyway.
				this.resultList.deselectAll();
			},

			/**
			 * Called if the results are cleared (when search string is deleted) and
			 * the subclass may want to clean up something besides the result list.
			 */
			didClearResults: function() {

			},

			/**
			 * LiveSearchBox delegate.
			 */
			clearResults: function() {
				this._slideToFirstColumn();
				this.resultList.clearResults();
				this._pager && this._pager.setTotal(0);
				this.didClearResults();
			},

			/**
			 * Callback after new results have been drawn.
			 */
			didShowResults: function() {
			},

			/**
			 * Callback when new results are available, but before they are drawn.
			 */
			willShowResults: function() {
			},

			/**
			 * If the column browser is used (when selecting DOORS baselines of a DOOR module)
			 * this will slide back to the first column and destroy all the widgets
			 * in the other columns. This is needed when clearing or redoing a search
			 * and we want show the main results again.
			 */
			_slideToFirstColumn: function() {
				this.setPagingControlsVisible(true);

				this.columnBrowser.popAllWidgetsExceptFirst().then(function(poppedWidgets) {
					for (var i = 0; i < poppedWidgets.length; i++) {
						poppedWidgets[i].destroyRecursive();
					}
				});
			},

			/**
			 * Show or hide the related paging controls all at once.
			 * They are hidden when showing a second column of nested results
			 * that may no support paging.
			 * @param {boolea} show true to show, false to disable.
			 */
			setPagingControlsVisible: function(show) {
				var visibility = show ? 'visible' : 'hidden'; //$NON-NLS-1$ //$NON-NLS-2$
				this._pager.domNode.style.visibility = visibility;
				this._pageSizeSelect.disabled = !show;
			},

			/**
			 * Callback for when a search begins, but results are not yet returned.
			 */
			willDoSearch: function() {
				this._slideToFirstColumn();
				this.submitButton.disabled = true;
				this.resultList.showLoading(true);
				this.clearPreview();
			},

			showResults: function(hits) {
				this.messageBox.hideMessage();
				this.willShowResults();
				this.resultList.showResults(hits);
				this.didShowResults();
			},

			/**
			 * Sorts the existing hits in place.
			 * @param  {function} compareFunction a compare function to sort with.
			 */
			sortHits: function(compareFunction) {
				this.currentList().sortResults(compareFunction);
			},

			/**
			 * Filters the display hits by text. Check the visible text of each row,
			 * plus an extra properties handed in that are directly in the data item.
			 * @param  {string} searchString    the text to search for.
			 * @param  {array} checkProperties list of property IDs to search on the underlying item.
			 *                                 In addition to visible text on the row. Used for checking
			 *                                 artifact types, even though they aren't visible (just an icon).
			 */
			filterHits: function(searchString, checkProperties) {
				this.currentList().filterResults(searchString, checkProperties);
			},

			cancel: function() {
				this._dialog.close();
			},

			close: function() {
				this._dialog.close();
			},

			/**
			 * Returns the currently visible list. Used when there are nested
			 * lists via the column browser (i.e., for DOORS baselines). Most actions
			 * act on what the user sees at that time even if there are actually more lists in the DOM.
			 * @return {StyledList} the current list.
			 */
			currentList: function() {
				var top = this.columnBrowser.topWidget();
				return top.list || top;
			},

			_onOk: function() {
				if (this.submitButton.disabled) return;
				var item = this.currentList().getSelectedItem();

				if (item) {
					// There are selected results, so OK the dialog.
					// Can get selected result still with getter until this is destroyed.

					if (this.onOk) {
						this.onOk(item);
					}
				}
			},

			_onSubmit: function(e) {
				e && dojo.stopEvent(e);
				if (this.submitButton.disabled) return;
				this._onOk();
			}
		}
	);
})();
