/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.hover.PropHoverContent"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.relm.explorer.web.ui.internal", "HoverMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function(){
	
var utils = com.ibm.relm.explorer.web.ui.internal.view.nodes.NodeUtils;

var PROP_STYLE_BASE = {
		padding: "2px", //$NON-NLS-1$
		verticalAlign: "top" //$NON-NLS-1$
	};

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.hover.PropHoverContent", //$NON-NLS-1$
	[ dijit._Widget, dijit._Templated ],
{
	templatePath: dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/hover/templates/PropHoverContent.html"), //$NON-NLS-1$ //$NON-NLS-2$
	contentInfo: null,
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("com.ibm.relm.explorer.web.ui.internal", "HoverMessages");		 //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function(){
		this.inherited(arguments);
		this.setContentInfo(this.contentInfo);
	},
	
	/*
	 * content info is form of {
	 * 		uri: // string
	 * 		title: // string
	 * 		summary: // string
	 * 		props: [{
	 * 			label,	// string used for label
	 * 			layout,	// string used for layout (horizontal or vertical)
	 * 			query,	// function used to retrieve values.
	 * 					// The function must have 2 functions as arguments.
	 * 					// If want to set values, call the first function with bindings 
	 * 					//	bindings is an array of 
	 * 					// 		{ targetTitle: { value: value } } 
	 * 					//		{ target: { value: value, type: type, } } // type should be "uri", "literal" or "typed-literal"
	 * 					// If want to show error, call the second function with error message
	 * 			variable// variable name in query result which will be used to retrieve property vaues. default is "target"
	 * 		},...]
	 * }
	 */
	setContentInfo: function(contentInfo){
		this.contentInfo = contentInfo;
		this._hideLoading();
		this._setTitleSummary();
		this._loadProps();
	},

	_hideLoading: function(){
		dojo.style(this.loadingRow, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	_setTitleSummary: function(){
		var contentInfo = this.contentInfo;
		if(contentInfo.title){
			if(utils.OPEN_LINK_TARGET == "") { //$NON-NLS-1$
				this.titleCell.innerHTML = "<a href='" + contentInfo.uri + "'>" + (contentInfo.title || this._messages.noTitle) + "</a>"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			} else {
				this.titleCell.innerHTML = "<a href='" + contentInfo.uri + "' target='" + utils.OPEN_LINK_TARGET + "'>" + (contentInfo.title || this._messages.noTitle) + "</a>"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
			}
			dojo.style(this.titleRow, "display", ""); //$NON-NLS-1$ //$NON-NLS-2$
		}else{
			dojo.style(this.titleRow, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
		}

		if(contentInfo.summary){
			this.summaryCell.innerHTML = contentInfo.summary;
			dojo.style(this.summaryRow, "display", ""); //$NON-NLS-1$ //$NON-NLS-2$
		}else{
			dojo.style(this.summaryRow, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
		}
	},

	_loadProps: function(){
		if(this.contentInfo.props && this.contentInfo.props.length > 0){
			var self = this;
			dojo.forEach(this.contentInfo.props, function(prop){
				var parentTr = dojo.create("tr", null , self.contentPane); //$NON-NLS-1$
				dojo.create("td", {innerHTML: self._messages.loading, colspan:2}, parentTr); //$NON-NLS-1$
				prop.query(
					function(bindings){
						self._showProps(bindings, prop.label, prop.layout, parentTr, false, prop.variable);
						self._adjustMaxsize();
					},
					function(message){
						self._showProps(null, prop.label, prop.layout, parentTr, true);
						console.error(message);
					}
				);
			});
		}
	},
	
	_showProps: function(arr, title, layout, parentTr, error, variableName){
		if ("vertical" == layout){ //$NON-NLS-1$
			this._showPropsVertically(arr, title, parentTr, error, variableName);
			return;
		}else if ("horizontal"){ //$NON-NLS-1$
			this._showPropsHorizontally(arr, title, parentTr, error, variableName);
			return;
		}
		
		// select layout by default
		if (arr && arr.length > 1){
			this._showPropsVertically(arr, title, parentTr, error, variableName);
		}else{
			this._showPropsHorizontally(arr, title, parentTr, error, variableName);
		}
	},
	
	_adjustMaxsize: function(){
		var mbox = dojo.marginBox(this.domNode);
		if(mbox.h > 350){
			mbox.h = 350;
			dojo.marginBox(this.domNode, mbox);
		}
	},
	
	_showPropsHorizontally: function(arr, title, parentTr, error, variableName){
		dojo.empty(parentTr);
		dojo.create("th", { //$NON-NLS-1$
			innerHTML: com.ibm.relm.explorer.web.ui.internal.util.escapeHtml(title)+':', //$NON-NLS-1$
			style: dojo.delegate(PROP_STYLE_BASE, {fontWeight: "bold", minWidth: "4em"}), //$NON-NLS-1$ //$NON-NLS-2$
			scope: 'row'//$NON-NLS-1$
			}, parentTr);
		if (arr && arr.length > 0){
			var self = this;
			var td = dojo.create("td", { style: PROP_STYLE_BASE }, parentTr); //$NON-NLS-1$
			dojo.forEach(arr, function(item){
				if (td.hasChildNodes()){
					td.appendChild(dojo.doc.createTextNode(", ")); //$NON-NLS-1$
				}
				td.appendChild(self._createPropItemContent(item, variableName));
			});
		}else{
			var td = dojo.create("td", { style: PROP_STYLE_BASE }, parentTr); //$NON-NLS-1$
			td.appendChild(this._createNoneContent(error));
		}
	},
	
	_showPropsVertically: function(arr, title, parentTr, error, variableName){
		dojo.empty(parentTr);
		var parentTd = dojo.create("td", { colspan: 2, style: PROP_STYLE_BASE }, parentTr); //$NON-NLS-1$
		
		var table = dojo.create("table", {style:"width:100%; margin-top:5px;"}, parentTd); //$NON-NLS-1$ //$NON-NLS-2$
		/*var caption =*/ dojo.create("caption", {innerHTML: com.ibm.relm.explorer.web.ui.internal.util.escapeHtml(title), style:"font-weight:bold;"}, table); //$NON-NLS-1$ //$NON-NLS-2$
		var tbody = dojo.create("tbody", null, table); //$NON-NLS-1$
		if(arr && arr.length>0){
			var self = this;
			dojo.forEach(arr, function(item){
				var iconUrl = item.iconUrl || utils.getStatusIconUrl(utils.getStatus(item)) || dojo.moduleUrl("com.ibm.relm.explorer.web", "ui/internal/view/images/task.gif"); //$NON-NLS-1$ //$NON-NLS-2$
				var tr = dojo.create("tr", null, tbody); //$NON-NLS-1$
				dojo.create("td", { //$NON-NLS-1$
					innerHTML: '<img src="'+iconUrl+'" style="width:16px; height:16px;" />', //$NON-NLS-1$ //$NON-NLS-2$
					style: {
						width: "16px" //$NON-NLS-1$
					}
				}, tr);
				var td = dojo.create("td", null, tr); //$NON-NLS-1$
				td.appendChild(self._createPropItemContent(item, variableName));
			}, this);
		}else{
			var tr = dojo.create("tr", null, tbody); //$NON-NLS-1$
			tr.appendChild(this._createNoneContent(error));
		}
	},
	
	_createPropItemContent: function(item, variableName){
		var targetVariable = "target"; // default //$NON-NLS-1$
		if(variableName && (item[variableName] || item[variableName + "Title"] || item[variableName + "Id"])){ //$NON-NLS-1$ //$NON-NLS-2$
			targetVariable = variableName;
		}
		// assume an item of query result contains at least one of "target", "targetTitle", "targetId"
		var targetTitle = item[targetVariable + "Title"] || {}; //$NON-NLS-1$
		var target = item[targetVariable] || {};
		var linkTitle = 
			targetTitle.value || 
			((target.type == "literal" || target.type == "typed-literal") && target.value) || //$NON-NLS-1$ //$NON-NLS-2$
			this._messages.noTitle;
		var linkId = (item[targetVariable + "Id"] && item[targetVariable + "Id"].value) || ""; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		if(linkId.length > 8) linkId = ""; // to-do DM's ID is not useful //$NON-NLS-1$
		// property label will be id:title
		if(linkId) linkTitle = linkId + ":" + linkTitle; //$NON-NLS-1$
		// create like or text based on type
		if (target.type == "uri"){ //$NON-NLS-1$
			var url = target.value;
			if (this.contentInfo.decodeUrls){
				url = decodeURIComponent(url);
			}
			var a = dojo.create("a", {href: url, target: utils.OPEN_LINK_TARGET}); //$NON-NLS-1$
		}else{
			var title = targetTitle.tooltip || target.tooltip || targetTitle.value || ""; //$NON-NLS-1$
			var a = dojo.create("span", {title: title}); //$NON-NLS-1$
		}
		dojo.html.set(a, jazz.util.html.removeScript(linkTitle));
		return a;
	},
	
	_createNoneContent: function(error){
		return dojo.create("span", { //$NON-NLS-1$
			innerHTML: error ? this._messages.errorContent : this._messages.noContent,
			style: {
				color: error ? "red" : "gray", //$NON-NLS-1$ //$NON-NLS-2$
				fontStyle: "italic" //$NON-NLS-1$
			}
		});
	}
});

})();
