/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: 
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.nodes.QueryBuilderUtils"); //$NON-NLS-1$

(function(){

dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.xmlutils"); //$NON-NLS-1$

var RESOURCE_SHORT_ID = "xsd:anyURI"; //$NON-NLS-1$
var RESOURCE_NAME = "resource"; //$NON-NLS-1$

var STRING = "http://www.w3.org/2001/XMLSchema#string"; //$NON-NLS-1$
var BOOLEAN = "http://www.w3.org/2001/XMLSchema#boolean"; //$NON-NLS-1$
var LONG = "http://www.w3.org/2001/XMLSchema#long"; //$NON-NLS-1$
var INTEGER = "http://www.w3.org/2001/XMLSchema#integer"; //$NON-NLS-1$
var INT = "http://www.w3.org/2001/XMLSchema#int"; //$NON-NLS-1$
var SHORT = "http://www.w3.org/2001/XMLSchema#short"; //$NON-NLS-1$
var BYTE = "http://www.w3.org/2001/XMLSchema#byte"; //$NON-NLS-1$
var DATETIME = "http://www.w3.org/2001/XMLSchema#dateTime"; //$NON-NLS-1$
var URI = "http://www.w3.org/2001/XMLSchema#anyURI"; //$NON-NLS-1$
var PERSON = "http://xmlns.com/foaf/0.1/Person"; //$NON-NLS-1$

var qbUtils = {};
var xmlutils = com.ibm.relm.explorer.web.ui.internal.xmlutils;

var DEFAULT_COMPONENT_TYPE = "baseResourceNode"; //$NON-NLS-1$ 

/*
 * Utility to construct a query builder output which will be passed to Report Builder to build a SPARQL query
 */
dojo.mixin(qbUtils, {
	_createGroup: function(groupName, operation, members) {
		var group = {operation: operation};
		group[groupName] = members;
		return {group: group};
	},

	/*
	 * Add new member to existing group
	 */
	_addMemberToGroup: function(group, groupName, operation, member) {
		if (!group || !group.operation) {
			group = {operation: operation};
		} else if (group.operation !== operation) {
			var newOperation = operation == "AND" ? "OR" : "AND"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			group = {operation: newOperation}; //$NON-NLS-1$
			group[groupName] = [group];
		}

		if (!group[groupName]) group[groupName] = [];
		group[groupName].push(member);
		return group;
	},
	
	_createPathGroup: function(domain, links) {
		var conditions = [];
		var path = {path: {conditions: conditions}};
		var domainLabel = this._getDomainLabel(domain);
		if (links && links.length > 0) {
			dojo.forEach(links, function(link) {
				conditions.push({condition: 
					{domain: domain, 
					domainLabel: domainLabel, 
					optional: true, 
					pathLabel: "dummy",  //$NON-NLS-1$
					id: link.linktype, 
					target: link.target, 
					targetLabel: this._getDomainLabel(link.target, link.index)}
				});
			}, this);
		} else {
			conditions.push({condition: {domain: domain, domainLabel: domainLabel}});
		}

		var pathsAndGroups = [];
		pathsAndGroups.push({path: {conditions: conditions}});
		return this._createGroup("pathsAndGroups", "AND", pathsAndGroups); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	_createFilter: function(domain, filter) {
		var copyFilter = dojo.clone(filter);
		dojo.mixin(copyFilter, {
			domain: domain, 
			domainLabel: this._getDomainLabel(domain),
		});
		return {filter: copyFilter};
	},
	
	_createFilters: function(domain, groupName, group) {
		return dojo.map(group && group.children, function(child) {
			if (child.operation) {
				return this._createFilterGroup(domain, child);
			} else {
				return this._createFilter(domain, child);
			}
		}, this);
	},
	
	/*
	 * Create filter group for query builder output
	 */
	_createFilterGroup: function(domain, filterGroup) {
		var operation = filterGroup ? filterGroup.operation : "AND"; //$NON-NLS-1$
		var groupName = "filtersAndGroups"; //$NON-NLS-1$
		return this._createGroup(groupName, operation, this._createFilters(domain, groupName, filterGroup));
	},
	
	/*
	 * Create result columns for query builder output
	 */
	_createResultColumns: function(domain, variables, links) {
		return dojo.map(variables, function(variable) {
			return {resultColumn: {
						id: variable.id, 
						domain: domain, 
						domainLabel: this._getDomainLabel(domain),
						variableType: variable.type || "xsd:anyURI", //$NON-NLS-1$
						label: "dummy", //$NON-NLS-1$
						name: "dummy",  //$NON-NLS-1$
						variableName: "dummy",  //$NON-NLS-1$
						customLabel: "dummy",   //$NON-NLS-1$
					}};
		}, this).concat(dojo.map(links, function(link) {
			return {resultColumn: {
				id: RESOURCE_SHORT_ID,
				domain: link.target, 
				domainLabel: this._getDomainLabel(link.target, link.index),
				variableType: "xsd:anyURI", //$NON-NLS-1$
				label: "url", //$NON-NLS-1$
				name: "dummy",  //$NON-NLS-1$
				variableName: "dummy",  //$NON-NLS-1$
				customLabel: "dummy",   //$NON-NLS-1$
			}};
		}, this));
		
		
	},
	
	/*
	 * Create projects for query builder output
	 */
	_createProjects: function(domain) {
		var projects = [];
		return projects;
	},
	
	_getDomainLabel: function(domain, index) {
		// must be unique per domain
		if (index == undefined) index = 1;
		return  domain.replace(/[\(\)\[\]\{\}\<\>]/g, "") + index.toString(); //$NON-NLS-1$
	},
	
	_getComponentInfo: function(nodeType, config, container) {
		var items = config.componentInfo.items;
		if(items) {
			for(var i = 0; i < items.length; i++) {
				if(items[i].id == nodeType)
					return items[i];
			}
		}

		items = container._viewer._componentInfo;
		if (items) {
			for (var key in items) {
				if(items[key].id == nodeType)
					return items[key];
			}
		}
		return null;
	},

	_getPropertyName: function(property) {
		if (property) {
			if (property.shortid) {
				return this.extractPropertySuffix(property.shortid);
			}	
			return property.name;
		}
	},
	
	/*
	 * Get a property for given parameter
	 */
	_getVariableFromParam: function(param, properties) {
		var value = param.value;
		if (!dojo.isString(value)) return;
		
		var matches = value.match(/\${(\w+)}/); // e.g. extract title from ${title}
		if (matches && matches.length > 1) {
			return this._getVariableFromName(matches[1], properties);
		}
	},

	/*
	 * Get a property for given links
	 */
	_getVariableFromLinks: function(name, linkProperties) {
		
	},
	
	/*
	 * Get a property for given name
	 */
	_getVariableFromName: function(name, properties) {
		// A resource is a special name that represents the resource's URI. Treat it specially.
		if (name == "resource") { //$NON-NLS-1$
			name = "anyURI"; //$NON-NLS-1$
		}
		if (!properties) return null;
		
		for (var i = 0; i < properties.length; i++) {
			var property = properties[i];
			var propName = this._getPropertyName(property);
			if (propName == name)
				return {id: property.shortid, type: property.range && property.range.value};
		}
	},
	
	_getVariablesFromAction: function(variables, properties, ids, action){
		if(!dojo.isString(action)) return;
		
		var matches = action.match(/\${(\w+)}/g); // e.g. extract type and status from reloadContainer(["container1"],{"witype":"${type}","param2":"${status}"})
		if(matches && matches.length > 0){
			dojo.forEach(matches, function(match){
				this._addVariable(variables, this._getVariableFromParam({value: match}, properties), ids);
			}, this);
		}
	},

	_getVariableFromTargetProps: function(variables, properties, ids, targetProps){
		dojo.forEach(targetProps, function(target) {
			dojo.forEach(target, function(prop) {
				this._addVariable(variables, this._getVariableFromName(prop.name, properties), ids);
			}, this);
		}, this);
	},

	/*
	 * Collect variables defined in a component for a container
	 */
	_getVariables: function(domain, config, properties, container) {
		var variables = [];
		var componentType = null;
		var ids = [];
		var nodeType = container.getNodeType();
		var targetNode = nodeType ? config.getResourceInfoById(nodeType) : config.getDefaultResourceInfo();
		if (targetNode) {
			// Conditions
			this._getVariableFromTargetProps(variables, properties, ids, targetNode.targetProps);

			// Hover
			dojo.forEach(targetNode.hoverProps, function(prop) {
				this._addVariable(variables, this._getVariableFromName(prop.name, properties), ids);
			}, this);				
			
			// Action
			dojo.forEach(targetNode.actions, function(action){
				this._getVariablesFromAction(variables, properties, ids, action.action);
			}, this);

			// Node UI Type
			if (targetNode.component) {
				componentType = targetNode.component.type;
				if (targetNode.component.params) {
					var params = targetNode.component.params;
					for (var key in params) {
						var param = params[key];
						if(dojo.isArray(param.value)){
							// Collect from conditional parameter
							dojo.forEach(param.value, function(conditionalParamValue){
								this._getVariableFromTargetProps(variables, properties, ids, conditionalParamValue.targetProps);
								this._addVariable(variables, this._getVariableFromParam(conditionalParamValue, properties), ids);
							}, this);
						}else{
							this._addVariable(variables, this._getVariableFromParam(params[key], properties), ids);
						}
					}
				}
			}
		}
		
		// System UI Type
		if (!componentType) {
			// fallback : when no componentType is specified. This must be a Default type, which is "baseResourceNode" (see Resource.js)
			componentType = DEFAULT_COMPONENT_TYPE;
		}
		var component = this._getComponentInfo(componentType, config, container);
		if (component && component.attributes) {
			for (var i = 0; i < component.attributes.length; i++) {
				this._addVariable(variables, this._getVariableFromParam(component.attributes[i], properties), ids);
			}
		}

		// Set a resource variable explicitly to get resource URI if it isn't defined anywhere. The resource URI is necessary to show artifact nodes in view
		this._addVariable(variables, {id: RESOURCE_SHORT_ID, type: URI}, ids);
		return variables;
	},
	
	_addVariable: function(variables, variable, ids) {
		if (variable && dojo.indexOf(ids, variable.id) < 0) {
			variables.push(variable);
			ids.push(variable.id);
		}
	},
	
	_getDownstreamLinks: function(container, domainId) {
		var links = container.getDownstream();
		var viewer = container._viewer;
		var indexMap = {};
		indexMap[domainId] = 1;
		var linkTypes = [];
		var self = this;
		return dojo.map(dojo.filter(links, function(link) {
			if(dojo.indexOf(linkTypes, link.linktype) < 0) {
				linkTypes.push(link.linktype);
				return true;
			}
			return false;
		}), function(link) {
			var l = dojo.clone(link); 
			var downstreamId = l.id;
			var downContainer = viewer.findContainer(downstreamId);
			if (downContainer) {
				if (downContainer.elementType == 'standard') {//$NON-NLS-1$
					l.target = downContainer.elementId;
					l.index = indexMap[l.target] = indexMap[l.target] ? indexMap[l.target] + 1 : 1;
				}	
			} else {
				// container may not have been loaded
				var doc = dojox.xml.parser.parse(viewer.config.layout);
				if (doc && doc.firstChild) {
					var targetNode = self._findTargetContainerNode(doc.firstChild, downstreamId);
					if (targetNode) {
						var targetId = targetNode.getAttribute('elementId'); //$NON-NLS-1$
						if (targetId) {
							l.target = targetId;
							l.index = indexMap[l.target] = indexMap[l.target] ? indexMap[l.target] + 1 : 1;
						}
					}
				}
			}
			return l;
		});
	},
	
	_findTargetContainerNode: function(node, id) {
		var containers = xmlutils.getChildNodesByName("container", node); //$NON-NLS-1$
		if (!containers) return null;
		
		for (var i = 0; i < containers.length; i++) {
			var container = containers[i];
			if (container.getAttribute("id") == id) { //$NON-NLS-1$
				return container;
			} else {
				var target = this._findTargetContainerNode(container, id);
				if (target)
					return target;
			}
		}
		return null;
	},
	
	/*
	 * Create Report Builder Output for a given domain from a config for building a SPARQL query
	 */
	createQueryBuilderOutput: function(domain, config, container, parameters, properties) {
		var qbOutputValue = {};
		if (domain) {
			// collect filters for a container
			var filterGroup = this._getFilterGroup(domain, container, parameters);
	
			// collect variables in a view config
			var variables = this._getVariables(domain, config, properties, container);
			var links = this._getDownstreamLinks(container, domain);

			// build query builder output
			qbOutputValue.topPathGroup = this._createPathGroup(domain, links);
			qbOutputValue.topFilterGroup = this._createFilterGroup(domain, filterGroup);
			qbOutputValue.resultColumns = this._createResultColumns(domain, variables, links);
			qbOutputValue.projects = this._createProjects(domain);
		}
		return {QueryBuilder: qbOutputValue};
	},

	/*
	 * Add upstream resource filters to existing query builder output
	 */
	addFilterForResorces: function(domain, qbOutput, resources) {
		if (!domain || !qbOutput || !qbOutput.QueryBuilder || !resources || resources.length == 0) return;
		
		var topFilterGroup = qbOutput.QueryBuilder.topFilterGroup;

		/*
		 * Combine all resource filters using OR operation
		 */
		var valueFilters = {operation: "OR", type: "filter"}; //$NON-NLS-1$ //$NON-NLS-2$
		valueFilters.children = dojo.map(resources, function(resourceUrl) {
			var filter = {domain: domain, id: RESOURCE_SHORT_ID};
			return this._setParamValueToFilter(filter, {type: "string", value: resourceUrl, operator: "is"}); //$NON-NLS-1$ //$NON-NLS-2$
		}, this);

		var valuesFilterGroup = this._createFilterGroup(domain, valueFilters);
		if (!valuesFilterGroup)
			return;

		/*
		 * Value filters must be combined with the other filters using AND operation
		 */
		var groupName = "filtersAndGroups"; //$NON-NLS-1$
		var updatedGroup = this._addMemberToGroup(topFilterGroup.group, groupName, "AND", valuesFilterGroup); //$NON-NLS-1$
		if (updatedGroup)
			topFilterGroup.group = updatedGroup
	},
	
	/*
	 * Collect parameters from container and build filter group for query builder output
	 */
	_getFilterGroup: function(domain, container, parameters) {
		
		// get filterGroup in container if defined
		var filterGroup = dojo.clone(container.getFilterGroup());
		var targetGroup = filterGroup;
		if (!filterGroup) {
			filterGroup = targetGroup = {operation: "AND", type: "filter", children: []}; //$NON-NLS-1$ //$NON-NLS-2$
		} else if (filterGroup.operation == "OR") { //$NON-NLS-1$
			targetGroup = {operation: "AND", children: []}; //$NON-NLS-1$
			if (!filterGroup.children)
				filterGroup.children = [];
			filterGroup.children.push(targetGroup);
		}

		var filters = {};
		for (var i = 0; i < targetGroup.children.length; i++) {
			var filter = targetGroup.children[i];
			if (!filter.operation)
				filters[filter.id] = filter;
		}
		
		// Collect parameters
		if (!parameters)
			parameters = container.getParameters();
		for (var name in parameters) {
			var param = parameters[name];
			var filter = filters[name];
			if (!filter) {
				filter = {domain: domain, id: name};
				targetGroup.children.push(filter);
			}
			this._setParamValueToFilter(filter, param);
		}
		return filterGroup.children.length > 0 ? filterGroup : null;
	},	

	_setParamValueToFilter: function(filter, param) {
		filter.type = param.type;
		filter.locked = "true"; //$NON-NLS-1$
		filter.operator = param.operator || "is" /* exact match */  //$NON-NLS-1$

		if (param && param.value) {
			switch (param.type) {
			case "enum": //$NON-NLS-1$
				filter.type = "ENUMERATION"; //$NON-NLS-1$
				filter.value = [param.value.substitution || param.value.value];
				break;
			case "string": //$NON-NLS-1$
				filter.type = "STRING"; //$NON-NLS-1$
				filter.value = [param.value];

				// set contains as exact match would not be useful
				filter.operator = param.operator || "contains"; //$NON-NLS-1$
				break;
			case "date": //$NON-NLS-1$
				filter.type = "DATE"; //$NON-NLS-1$
				filter.relativeDateOp = param.relativeDateOp || "exactdate"; //$NON-NLS-1$
				filter.value = [param.value];
				break;
			case "number": //$NON-NLS-1$
				filter.type = "INTEGER"; //$NON-NLS-1$
				filter.value = [param.value];
				break;
			case "person": //$NON-NLS-1$
				filter.type = "PERSON"; //$NON-NLS-1$
				filter.value = [param.value];
				break;
			default:	
				filter.type = param.type;
				filter.value = [param.value];
				break;
			}
		}
		return filter;
	},
	
	/*
	 * rtc_cm:WorkItem -> rtc_cm_WorkItem1
	 */
	getResourceId: function(domainId, index) {
		// replace all the #:.- characters. 
		// This the logic that Report Builder uses to create a variable name for SPARQL query
		var resourceId = this.makeName(domainId);
		
		// magic number but it's always true as long as only one artifact type is associated with a container
		if (index == undefined) index = 1;
		var indexStr = index.toString();
		
		resourceId += indexStr;
		return resourceId;
	},

	/*
	 * rtc_cm_WorkItem_priority -> priority
	 */
	extractPropertySuffix: function(name) {
		if (name) {
			var index = name.lastIndexOf(':');
			if (index >= 0) name = name.substring(index + 1);
			return name;
		}
	},
	
	makeName: function(name) {
		return name ? name.replace(/[#:.-]/g, '_') : ''; //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	correctParamType: function(param, property) {
		if (property.possiblevalues) {
			param.type = "enum"; //$NON-NLS-1$
			param.enumtype = property.shortid;
		} else if (property.range) {
			switch (property.range.value) {
			case BOOLEAN:
				param.type = "boolean"; //$NON-NLS-1$
				break;
			case INTEGER:
			case LONG:
			case INT:
			case SHORT:
			case BYTE:
				param.type = "number"; //$NON-NLS-1$
				break;
			case DATETIME:
				param.type = "date"; //$NON-NLS-1$
				break;
			case URI:
				param.type = property.shortid == RESOURCE_SHORT_ID ? "artifact" : "uri";; //$NON-NLS-1$; //$NON-NLS-2$;
				break;
			case PERSON:
				param.type = "person"; //$NON-NLS-1$
			}
		}
		return param;
	},
	
	/*
	 * Convert properties of query result resources. (Remove prefix created based on domainId)
	 */
	convertQueryResult: function(container, resources, properties) {
		if (!resources || resources.length == 0)
			return resources;

		var domainId = container.elementId;
		var links = this._getDownstreamLinks(container, domainId);
		
		var resourceId = this.getResourceId(domainId);
		var reference = resources[0];
		
		dojo.forEach(resources, function(res) {
			res[RESOURCE_NAME] = res[resourceId];
			delete res[resourceId];
			
			// subtract a prefix from each variable names to match parameter names used in a view config
			var prefix = resourceId + '_'; //$NON-NLS-1$
			var prefixLen = prefix.length;
			for (var key in res) {
				if (key.indexOf(prefix) === 0) {
					var variableName = key.substring(prefixLen);
					res[variableName] = res[key];
					delete res[key];
				} else {
					dojo.forEach(links, function(link) {
						var target = this.getResourceId(link.target, link.index);
						if (res[target]) {
							res[this.makeName(link.linktype)] = res[target];
							delete res[target];
						}
					}, this);
				}
			}
		}, this);
		return resources;
	},
	convertQueryParamId: function(id){
		if(id === RESOURCE_SHORT_ID){
			return RESOURCE_NAME;
		}
		return id;
	},
	
	getGroupType: function(shortId, metamodel) {
		if (!metamodel || !metamodel.metamodelGroups)
			return null;

		var groupTypes = []; 
		for (var i = 0; i < metamodel.metamodelGroups.length; i++) {
			var model = metamodel.metamodelGroups[i];
			this._addParentIds(shortId, null, model.children, groupTypes);
		}
		return groupTypes.length > 0 ? groupTypes : null;
	},
	
	_addParentIds: function(id, parentid, modelgroup, parentIds) {
		if (!modelgroup)
			return;
		
		for (var i = 0; i < modelgroup.length; i++) {
			var model = modelgroup[i];
			if (parentid && model.id === id)
				parentIds.push(parentid);
			
			this._addParentIds(id, model.id, model.children, parentIds);
		}
	}
});

com.ibm.relm.explorer.web.ui.internal.view.nodes.QueryBuilderUtils = qbUtils;

})();