/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2014, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.nodes._LinksMixin"); //$NON-NLS-1$

(function(){
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.QueryBuilderUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.nodes.ArtifactTypeElementProxyUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.ConfigurationUIUtils"); //$NON-NLS-1$

var qbUtils = com.ibm.relm.explorer.web.ui.internal.view.nodes.QueryBuilderUtils;
var elementProxyUtils = com.ibm.relm.explorer.web.ui.internal.view.nodes.ArtifactTypeElementProxyUtils;
var configurationUtils = com.ibm.relm.common.web.ui.ConfigurationUIUtils;

dojo.declare(
	"com.ibm.relm.explorer.web.ui.internal.view.nodes._LinksMixin", //$NON-NLS-1$
	null,
{
		_linkProperties: null,
		_links: {},
		_linkInitialized: false,
		_setupQueryDeferred: null,

		/*
		 * Artifact Element Link Settings
		 */
		/**
		 * Initialize link information
		 */
		_initLinkInformation: function() {
			// initialize link information
			if (this.elementId) {
				this._linkInitialized = false;
				if (this._viewer) {
					this._runinitLinkProperties();
				} else {
					this._oneTimeLinkInitHandle = this.connect(this, "reload", dojo.hitch(this, function() { //$NON-NLS-1$
						this.disconnect(this._oneTimeLinkInitHandle);
						delete this._oneTimeLinkInitHandle;
						
						this._runinitLinkProperties();
					}));
				}
			} else {
				this._linkInitialized = true;
			}
		},
	
		_getShortId: function(id, possibleResources) {
			if (possibleResources) {
				for (var i = 0; i < possibleResources.length; i++) {
					if (id == possibleResources[i].uri)
						return possibleResources[i].shortid;
				}
			}
			return id;
		},
		
		_runinitLinkProperties: function() {
			var viewer = this._viewer;
			elementProxyUtils.getElementObject(viewer, this.elementId, this.elementType, dojo.hitch(this, function(result) {
				var self = this;
				this._initLinkProperties(viewer, result, this.elementType != 'standard' ? dojo.hitch(this, function(target) {
					elementProxyUtils.getElementTitle(viewer, target.id, target.type, function(result) {target.title = result;});
					return target;
				}) : function(target) {
					self._qbMetamodel = result.metamodel;
					return target;
				});
				
				this._linkInitialized = true;
				this._runInitializationDeferred();
				
			}), dojo.hitch(this, function(err) {
				if (this.setError && err) this.setError(true, err);
				
				// Need to call even if error occurs to go to the next layouting steps
				this._linkInitialized = true;
				this._runInitializationDeferred();
			}));
		},
		
		/**
		 * Initialize possible link poroperties
		 */
		_initLinkProperties: function(viewer, elementObject, _propFunc) {
			if (!elementObject || !elementObject.properties) {
				return;
			}
			
			var targets = [];
			dojo.forEach(elementObject.properties, function(property) {
				dojo.forEach(property.linkTo, function(linkTo) {
					var target = {id: linkTo.id, fullId: linkTo.fullId, type: linkTo.type, 
							title: linkTo.targetTitle, label: linkTo.label, sourceProperty: property.name};
					if (_propFunc) target = _propFunc(target);
					targets.push(target);
				});					
			});
			if (targets.length > 0) this._linkProperties = targets;
		},
		
		/**
		 * Keep deferred runnables invoked during initialization
		 */
		_createInitializationDeferred: function() {
			var deferred = new dojo.Deferred();
			if (!this._setupQueryDeferred) this._setupQueryDeferred = [];
			this._setupQueryDeferred.push(deferred);
			return deferred;
		},
		
		/**
		 * Run deferred runnables invoked during initialization
		 */
		_runInitializationDeferred: function() {
			if (!this._setupQueryDeferred) return;
			var deferredList = this._setupQueryDeferred;
			delete this._setupQueryDeferred;
			
			dojo.forEach(deferredList, function(deferred) { deferred.callback(); });
		},

		/*
		 * Current Link Relations between Nodes
		 */

		/**
		 * Check if this node has upstream node
		 */
		isDownstream: function() {
			var links = this.getUpstream();
			return links && links.length > 0;
		},
		
		/**
		 * Check if this node has downstream node
		 */
		hasDownstream: function() {
			var links = this.getDownstream();
			return links && links.length > 0;
		},
	
		/**
		 * Get all upstream nodes
		 */
		getUpstream: function() {
			if (!this._viewer || !this._viewer.getUpstream) return null;
			return this._viewer.getUpstream(this.getId());
		},
		
		/**
		 * Get all downstream nodes
		 */
		getDownstream: function() {
			if (!this._viewer || !this._viewer.getDownstream) return null;
			return this._viewer.getDownstream(this.getId());
		},
			
		/**
		 * Callback called when node id is changed
		 */
		onLinkedIdModified: function(oldId, newId) {
			if (!this._viewer || !this._viewer.updateLinkNode) return;
			this._viewer.updateLinkNode(oldId, newId);
		},
		
		/**
		 * links is static links from/to all resource nodes in this container. Usually only tree container uses this attribute. 
		 */
		setLinks: function(links){
			this._links = links;
		},
		
		getLinks: function(){
			return this._links;
		},
		
		getLinkInfo: function(viewer, widget, target, linkInfo, linkcache) {
			if (!linkInfo) linkInfo = [];
			if (!target) target = widget;
				
			dojo.forEach(widget.getUpstream(), function(uplink) {
				var container = viewer.findContainer(uplink.id);
				if (container) {
					var link = {id: uplink.id, title: uplink.id, linktype: uplink.linktype};
					if (container.elementId) {
						elementProxyUtils.getElementTitle(viewer, container.elementId, container.elementType, function(title) {
							link.title = title;
						});
					}
					
					if (container.getHidden()) {
						if (linkcache)
							linkcache.splice(0, 0, link);
						this.getLinkInfo(viewer, container, target, linkInfo, linkcache || [link]);
					}
					else {
						var linkpath = linkcache || [];
						
						// add source node
						linkpath.splice(0, 0, link);

						// add target node
						link = {id: target.getId(), title: target.getId()};
						if (target.elementId) {
							elementProxyUtils.getElementTitle(viewer, target.elementId, target.elementType, function(title) {
								link.title = title;
							});
						}
						linkpath.push(link);

						linkInfo.push({source: container, target: target, linkpath: linkpath});
					} 
				}
			}, this);
			return linkInfo;
		},

		/**
		 * Called when resources for this node are loaded. 
		 *    1. Store resources linked to outside are stored to cache 
		 *    2. Configure link information for each resource
		 */
		resourceLoaded: function(viewer, resources, linkinfo) {
			var self = this;
			var downstreamCacheMap = {};
			// store cache at downstream link information if link is already created
			dojo.forEach(this.getDownstream(), function(link) {
				var valueType;
				var linktype = link.linktype;
				if (self.elementType == 'standard') {//$NON-NLS-1$
					linktype = qbUtils.makeName(linktype);
				}
				
				var cache = {};
				dojo.forEach(resources, function(resource) {
					self._addToCache(cache, resource, linktype);
					if (!valueType && linktype && resource[linktype])
						valueType = resource[linktype].type;
				});
				link.cache = dojo.mixin(link.cache, cache);
				link.valueType = valueType;
				downstreamCacheMap[linktype] = cache;
			});

			// store cache at possible link information if link has not been created yet
			dojo.forEach(this._linkProperties, function(link) {
				var valueType;
				var linktype = link.sourceProperty;
				if (self.elementType == 'standard') //$NON-NLS-1$
					linktype = qbUtils.makeName(linktype);
				if (resources && resources.length > 0 && resources[0][linktype]) {
					valueType = resources[0][linktype].type;
				}
				if (downstreamCacheMap[linktype]) {
					link.cache = dojo.mixin(link.cache, downstreamCacheMap[linktype]);
				} else {
					var cache = [];
					var valuetype;
					dojo.forEach(resources, function(resource) {
						self._addToCache(cache, resource, linktype);
					});
					link.cache = dojo.mixin(link.cache, cache);
					link.valueType = valueType;
				}
			});
			
			// Configure link information
			this._updateLinks(viewer, resources, linkinfo);
		},
		
		/**
		 * Store map of source and target uris to cache
		 */
		_addToCache: function(cache, resource, linktype) {
			var target = resource[linktype];
			if (resource.resource && target) {
				var targets = cache[resource.resource.value];
				if (!targets) 
					targets = cache[resource.resource.value] = [];
				else if (dojo.indexOf(targets, target.value) >= 0)
					return;
				targets.push(target.value);
			}
		},
		
		resourceLoading: function() {
			this._links = null;
			dojo.forEach(this.getDownstream(), function(link) {	delete link.cache; });
			dojo.forEach(this._linkProperties, function(link) {	delete link.cache; });
		},

		_findTargetLink: function(id, linktype, links) {
			if (!id || !linktype || !links) return null;
			for (var i = 0; i < links.length; i++) {
				var link = links[i];
				if (link.id == id && link.linktype == linktype) {
					return link;
				}
			}
			return null;
		},
		
		_updateLinks: function(viewer, resources, linkinfo) {
			if (!this.isDownstream()) {
				return;
			}
			if (this.getHidden()) {
				return;
			}
			
			var resourceMap = {};
			dojo.forEach(resources, function(resource) {
				if (resource.resource.value)
					resourceMap[resource.resource.value] = {};
			});
			var linkpath = linkinfo.linkpath;
			resourceMap = this._getUpstreamResources(viewer, linkpath, linkpath.length - 2, null, linkinfo.component, true, resourceMap);
			if (resourceMap) {
				if (this._links && this._links.links) {
					this._links.links = this._links.links.concat(resourceMap);
				} else {
					this._links = {linkDefType: "multiSource", links: resourceMap}; //$NON-NLS-1$
				}
			}
		},
		
		/**
		 * Get upstream resources linked to this
		 * 
		 * [{upstreamId: upstreamId1: resources: resourceMap1}, ...] 
		 */
		_getUpstreamResources: function(viewer, linkpath, index, linkTitle, component, visibleOnly, resourceMap) {
			var resources = null;
			var thisId = this.getId();
			var linkId = (linkpath && index >= 0) ? linkpath[index].id : null;
			var elementId = this.elementId;
			var linktype = (linkpath && index >= 0) ? linkpath[index].linktype : null;
			dojo.forEach(this.getUpstream(), function(link) {
				if ((linkId && linkId != link.id) || (linktype != link.linktype)) return;
				if (visibleOnly && link.hidelines) return;
				var upContainer = viewer.findContainer(link.id);
				if (!upContainer) return;

				if (!linkTitle) {
					if (upContainer._linkProperties) {
						for (var i = 0; i < upContainer._linkProperties.length; i++) {
							var l = upContainer._linkProperties[i];
							if (l.id == elementId && l.sourceProperty == linktype) {
								linkTitle = l.label;
								break;
							}
						}
					} else {
						linkTitle = upContainer.getId();
					}
				}

				if (!resources) resources = [];
				var upResources = upContainer._getLinkedResources(viewer, link.id, thisId, link.linktype, linkpath, index, linkTitle, component, visibleOnly, resourceMap);
				if (dojo.isArray(upResources))
					resources = resources.concat(upResources);
				else if (upResources)
					resources.push(upResources);
			});
			return resources;
		},
		
		/**
		 * Create map of linked source and target nodes
		 * 
		 * {target1: [source1, ...], target2: [source2, ...]}
		 */
		_getLinkedResources: function(viewer, upstreamId, downstreamId, linktype, linkpath, index, linkTitle, component, visibleOnly, resourceMap) {

			var targetLink = this._findTargetLink(downstreamId, linktype, this.getDownstream());
			if (!targetLink || !targetLink.cache) 
				return;

			// collect source uris linked to children 
			for (var target in resourceMap) {
				var resourceInfo = resourceMap[target];
				if (resourceInfo.sources) {
					if (targetLink.cache) {
						for (var cacheSource in targetLink.cache) {
							dojo.forEach(targetLink.cache[cacheSource], function(cacheTarget) {
								resourceInfo.sources = dojo.map(resourceInfo.sources, 
										function(source) { return (source == cacheTarget) ? cacheSource : source; });
							});
						}
					}
					dojo.forEach(targetLink.cache, function(map) {
					});
				} else {
					resourceInfo.sources = [];
					if (targetLink.cache) {
						for (var cacheSource in targetLink.cache) {
							dojo.forEach(targetLink.cache[cacheSource], function(cacheTarget) {
								if (cacheTarget == target && dojo.indexOf(resourceInfo.sources, cacheSource) < 0)
									resourceInfo.sources.push(cacheSource);
							});
						}
					}
				}
			}

			// If this node is hidden, collect resources from link sources
			if (this.getHidden()) {
				return this._getUpstreamResources(viewer, linkpath, index - 1, linkTitle, component, visibleOnly, resourceMap);
			}

			// change uri map to node map
			var children = this.getChildren();
			for (var target in resourceMap) {
				var sources = resourceMap[target].sources;
				delete resourceMap[target].sources;
				
				resourceMap[target].sourceWidgets = dojo.filter(dojo.map(sources, 
									function(source) { 
										if (children != null) {							
											for (var i = 0; i < children.length; i++) {
												var node = children[i];
												if (node.uri == source) return node;
											}
										}
										return null; 
									}), 
									function(resource) { return resource != null; });
			}

			return {upstreamId : upstreamId, title: linkTitle, component: component, resources: resourceMap};
		},
		
		/**
		 * Create queries for each upstream nodes
		 */
		_getLinkedQueries: function(viewer) {
			var linkInfo = this.getLinkInfo(viewer, this);
			var upstream = this.getUpstream();
			return dojo.map(linkInfo, function(link) {
				var targetLink = null;
				var linkpath = link.linkpath;
				var target = linkpath[linkpath.length - 2];
				for (var i = 0; i < upstream.length; i++) {
					var up = upstream[i];
					if (up.id == target.id && up.linktype == target.linktype) {
						targetLink = up;
						break;
					}
				}
				
				if (!targetLink) return {linkpath: linkpath};
				
				// create query for the upstream node
				var query = {linkinfo: {linkId: targetLink.id, linkpath: linkpath}};
				if (targetLink.connectionId) {
					// set connection title and component info
					var connections = viewer.getConfig().connectionLinks;
					for (var i = 0; i < connections.length; i++) {
						var connection = connections[i];
						if (targetLink.connectionId == connection.id) {
							if (!query.linkDef) query.linkDef = {};
							query.linkinfo.title = connection.title;
							query.linkinfo.component = connection.component;
							break;
						}
					}
				} else {
					// set default title
					var upContainer = viewer.findContainer(targetLink.id);
					if (upContainer && upContainer._linkProperties) {
						var elementId = link.target.elementId;
						for (var i = 0; i < upContainer._linkProperties.length; i++) {
							var l = upContainer._linkProperties[i];
							if (l.id == elementId && l.sourceProperty == targetLink.linktype) {
								query.linkinfo.title = l.label;
								break;
							}
						}
					}
				}
				return query;
			});
		},

		/**
		 * Configure queries
		 */
		setupLinkedQuery: function(viewer, query) {
			var self = this;
			if (self._linkInitialized) {
				return self._setupLinkedQuery(viewer, query);
			} else {
				return dojo.when(this._createInitializationDeferred(), function() {
					return self._setupLinkedQuery(viewer, query);
				});
			}
		},
		
		getLinkedNodes: function(linkId, elementId, linktype) {
			var downstream = this.getDownstream();
			if (!downstream || downstream.length == 0) {
				return null;
			}

			var targetLink = null;
			for (var i = 0; i < downstream.length; i++) {
				var link = downstream[i];
				if (link.id == linkId && link.linktype == linktype) {
					targetLink = link;
					break;
				}
			}
			if (!targetLink) {
				return null;
			}

			if (!targetLink.cache) {
				if (!elementId || !this._linkProperties) {
					return null;
				}
				
				for (var i = 0; i < this._linkProperties.length; i++) {
					var link = this._linkProperties[i];
					if (link.id == elementId && link.sourceProperty == linktype) {
						if (!link.cache) return null;
						
						targetLink.cache = link.cache;
						break;
					}
				}
			}

			if (!targetLink.cache) {
				return null;
			}
			
			var targets = [];
			for (var source in targetLink.cache) {
				dojo.forEach(targetLink.cache[source], function(target) {
					if (dojo.indexOf(targets, target) < 0)
						targets.push(target);
				});
			}
			return {resources: targets, valueType: targetLink.valueType};
		},
		
		_setupLinkedQuery: function(viewer, query) {
			query.values = null;
			// add link information
			var downstream = this.getDownstream();
			if (downstream && downstream.length > 0) {
				query = dojo.clone(query);
				query.isUpstream = true;
				query.linkProperties = downstream;
				
				dojo.forEach(downstream, function(ds) {
					var linkName = "_com_ibm_relm_ae_links_" + ds.linktype + "_"; //$NON-NLS-1$ //$NON-NLS-2$
					query.parameters[linkName] = {value: "true", type: "boolean"}; //$NON-NLS-1$ //$NON-NLS-2$
				});
			}

			// add resources linked from parent containers
			var upstream = this.getUpstream();
			if (upstream && upstream.length > 0 && !dojo.some(upstream, function(link) { return link.unlimited; })) {
				query = dojo.clone(query);
				query.isDownstream = true;
				var linkinfo = query.linkinfo;
				var linkId = linkinfo ? linkinfo.linkId : null;
				var linktype = (linkinfo && linkinfo.linkpath && linkinfo.linkpath.length > 1) ? 
								linkinfo.linkpath[linkinfo.linkpath.length - 2].linktype : null;

				var sourceId = this.getId();
				var targetLink = null;
				for (var i = 0; i < upstream.length; i++) {
					var link = upstream[i];
					if (link.id == linkId && link.linktype == linktype) {
						targetLink = link;
						break;
					}
				}
				
				if (targetLink) {
					// associate connection with query if defined
					var upContainer = viewer.findContainer(targetLink.id);
					if (upContainer) {
						var target = upContainer.getLinkedNodes(sourceId, this.elementId, targetLink.linktype);
						if (target != null) {
							query.values = dojo.map(target.resources, configurationUtils.removeEmptyConfigurationContextFromUri);
							query.valueType = target.valueType;
							if (this.elementType == 'standard' && query.values.length > 0) {//$NON-NLS-1$
								qbUtils.addFilterForResorces(this.elementId, query.parameters.qbOutput, query.values);
							}
						}
					}
				}
			}
			return query;
		},
		
		prepareSetup: function() {
			this._waitingForSetup = dojo.clone(this.getUpstream());
		},
		
		readyForSetup: function(upstreamId, linktype) {
			if (this._waitingForSetup) {
				// This is called from upstream elements
				this._waitingForSetup = dojo.filter(this._waitingForSetup, function(wfs) {
					return wfs.id != upstreamId || wfs.linktype != linktype;
				});

				// go setup when all the upstream elements are loaded
				if (this._waitingForSetup.length == 0) {
					this._waitingForSetup = null;
					
					this.setup();
					this.layout();
				}
			} else {
				this.setup(true);
				this.layout();
			}
			
		}
});
})();