/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.le.ui.internal.QueryBuilderServiceUtils"); //$NON-NLS-1$

dojo.require("com.ibm.relm.common.web.ui.internal.util"); //$NON-NLS-1$
dojo.require("com.ibm.relm.common.web.ui.ConfigurationContext"); //$NON-NLS-1$

(function(){
	
// URIs
com.ibm.relm.le.ui.serviceRoot = net.jazz.ajax._contextRoot + "/"; //$NON-NLS-1$
com.ibm.relm.le.ui.qbServiceUri = com.ibm.relm.le.ui.serviceRoot + "relmle/qb/"; //$NON-NLS-1$

var baseQueryUrl = com.ibm.relm.explorer.web.ui.queryServiceUri;
com.ibm.relm.le.ui.internal.QueryBuilderServiceUtils = {};

var qbServiceUtils = com.ibm.relm.le.ui.internal.QueryBuilderServiceUtils;
var metamodelCache = {};
var _isSystemElementEnabled = null;

dojo.mixin(qbServiceUtils, 
{
	isSystemElementEnabled: function(){
		// isSystemElementEnabled won't change until restart
		if(_isSystemElementEnabled != null){
			return _isSystemElementEnabled;
		}

		var requestUri = com.ibm.relm.le.ui.qbServiceUri + "checkEnabled"; //$NON-NLS-1$
		return this._doGet(requestUri).then(function(response){
			_isSystemElementEnabled = response;
			return _isSystemElementEnabled;
		});
	},

	/*
	 * Utility method to get domain title
	 */
	getDomainTitle: function(domain, endpoint) {
		return dojo.when(this.getMetamodelItem(domain, endpoint), function(item) {
			return item ? item.label : domain;
		});
	},
	
	getMetamodelItem: function(domain, endpoint) {
		var self = this;
		return dojo.when(this.getMetamodelGroup(endpoint), function(metamodel) {
			return self._getMetamodelItem(domain, metamodel.metamodelGroups);
		});
	},
	
	_getMetamodelItem: function(domain, items) {
		if (items) {
			for (var i = 0; i < items.length; i++) {
				var item = items[i];
				if (domain == item.id) {
					return item;
				} else {
					var c = this._getMetamodelItem(domain, item.children);
					if (c) return c;
				} 
			}
		}
		return null;
	},
	
	/*
	 * Get properties for domainId from Report Builder's metamodel
	 */
	getMetamodelGroup: function(endpoint) {
		if (!endpoint){
			var globalConfigurationContext = com.ibm.relm.common.web.ui.ConfigurationContext.getGlobalConfigurationContext();
			if(globalConfigurationContext){
				endpoint = globalConfigurationContext.endpoint;
			}
		}
		if (metamodelCache[endpoint])
			return metamodelCache[endpoint];
		
		var uriParam;
		if(endpoint)
			uriParam = "?endpoint=" + encodeURIComponent(endpoint); //$NON-NLS-1$
		var requestUri = com.ibm.relm.le.ui.qbServiceUri + "metamodel"; //$NON-NLS-1$
		metamodelCache[endpoint] = this._doGet(requestUri, uriParam).then(function(response, ioArgs) {
			metamodelCache[endpoint] = response;
			return response;
		});
		return metamodelCache[endpoint];
	},

	/*
	 * Get properties for domainId from Report Builder's metamodel
	 */
	getProperties: function(viewer, shortId) {
		var params = {opType: "attributes", shortid: shortId}; //$NON-NLS-1$
		return this.getMetaModelInfo(viewer, params, true);
	},

	/*
	 * Get relationships for domainId from Report Builder's metamodel
	 */
	getRelationships: function(viewer, shortId) {
		var params = {opType: "relationships", shortid: shortId}; //$NON-NLS-1$
		return this.getMetaModelInfo(viewer, params);
	},

	/*
	 * Get possible resources from Report Builder's metamodel
	 * Call this when possible resorces corresponding given typeUris 
	 */
	getPossibleResources: function(viewer, typeUris) {
		var params = {opType: "possibleResources"}; //$NON-NLS-1$
		params.numPossible = typeUris ? typeUris.length : 0;
		dojo.forEach(typeUris, function(uri, index) {params['p' + index] = uri;}); //$NON-NLS-1$
		return this.getMetaModelInfo(viewer, params);
	},

	/*
	 * Get possible values for shortId from Report Builder's metamodel
	 */
	getPossibleValues: function(viewer, shortId, propertyId, projectUris) {
		var params = {opType: "possibleValues", type: shortId, property: propertyId}; //$NON-NLS-1$
		params.numProjects = projectUris ? projectUris.length : 0;
		dojo.forEach(projectUris, function(uri, index) {params['p' + index] = uri;}); //$NON-NLS-1$
		return this.getMetaModelInfo(viewer, params);
	},

	/*
	 * Get users from Report Builder's metamodel
	 */
	getUsers: function(viewer, showArchived, searchString) {
		var params = {opType: "users", searchString: searchString, showArchived: showArchived}; //$NON-NLS-1$
		return this.getMetaModelInfo(viewer, params);
	},

	/*
	 * Get users from Report Builder's metamodel
	 */
	getUserProjects: function(viewer, shortIds) {
		if (dojo.isArray(shortIds)) shortIds = shortIds.join(','); //$NON-NLS-1$
		var params = {opType: "userProjects", resource: shortIds}; //$NON-NLS-1$
		return this.getMetaModelInfo(viewer, params);
	},

	/*
	 * Get default columns for domainId from Report Builder's metamodel
	 */
	getDefaultColumns: function(viewer, queryJson) {
		var params = {opType: "defaultColumns", queryJson: queryJson}; //$NON-NLS-1$
		return this.getMetaModelInfo(viewer, params);
	},

	/*
	 * Get properties for domainId from Report Builder's metamodel
	 */
	getMetaModelInfo: function(viewer, params) {
		if (!params.endpoint) params.endpoint = viewer && viewer.config.endpoint;
		var paramText = dojo.toJson(params);
		var properties = viewer && viewer.getMetamodelInfoFromCache(paramText);
		if (properties)
			return properties;
	
		var requestUri = com.ibm.relm.le.ui.qbServiceUri + "metamodel"; //$NON-NLS-1$
		return this._doPost(requestUri, null, {postData: {metainfo: paramText}}).then(function(response, ioArgs) {
			viewer && viewer.addToMetamodelInfoCache(paramText, response);
			return response;
		});
	},
	
	/*
	 * Give qbOutput to Report Builder and build a SPARQL query
	 */
	buildQuery: function(qbOutput, callback, errorback, endpoint) {
		var uriParam;
		if(endpoint)
			uriParam = "?endpoint=" + encodeURIComponent(endpoint); //$NON-NLS-1$

		var arguments = {qbOutput: dojo.toJson(qbOutput)};
		var requestUri = com.ibm.relm.le.ui.qbServiceUri + "query"; //$NON-NLS-1$
		return this._doPost(requestUri, uriParam, {postData: arguments}, callback, errorback);
	},
	
	/*
	 * Build a SPARQL query using a query builder output and get the query
	 */
	getQuery: function(qbOutput, callback, errorback, queryEndpoint, container) {
		return this._buildAndRun(qbOutput, callback, errorback, queryEndpoint, container, function(result) {
			return result.queryString;
		});
	},

	/*
	 * Build a SPARQL query using a query builder output and execute the query
	 * If there is a SPARQL query that match given qbOutput in a cache, execute the query directly
	 */
	executeQuery: function(qbOutput, callback, errorback, endpoint, container) {
		return this._buildAndRun(qbOutput, callback, errorback, endpoint, container, function(result) {
			var content ={
					category: "children", //$NON-NLS-1$
					endpoint: endpoint,
					sparql: result.queryString,
					format: false,
					nosparqninresponse: true
				};

				return jazz.client.xhrPost({
					url: baseQueryUrl,
					headers: {"Accept":"application/json"}, //$NON-NLS-1$ //$NON-NLS-2$
					content: content, 
					load: callback,
					error: errorback,
					handleAs: "json" //$NON-NLS-1$
				});
		});
	},
	
	/*
	 * Build a SPARQL query using a query builder output and execute the query
	 * If there is a SPARQL query that match given qbOutput in a cache, execute the query directly
	 */
	_buildAndRun: function(qbOutput, callback, errorback, queryEndpoint, container, _run) {
		var qbOutputText = dojo.toJson(qbOutput);
		var qbOutputObj = container._viewer.getQbOutputFromCache(container, qbOutputText);

		var fromCache = false;
		var xhr = null;
		if (qbOutputObj && qbOutputObj.queryString) {
			xhr = qbOutputObj;
			fromCache = true;
		} else {
			xhr = this.buildQuery(qbOutput, null, null, queryEndpoint);
		}
		return dojo.when(xhr, function(result) {
			if (!fromCache)
				container._viewer.addToQbOutputCache(container, qbOutputText, result.queryString);
			return _run && _run(result);
		}, errorback);
	},

	_doGet: function(uri, uriParam, callback, errorback) {
		return this._run(jazz.client.xhrGet, uri, uriParam, {preventCache: true}, callback, errorback);
	},
	_doPost: function(uri, uriParam, arguments, callback, errorback) {
		return this._run(jazz.client.xhrPost, uri, uriParam, arguments, callback, errorback);
	},
	_doPut: function(uri, uriParam, arguments, callback, errorback){
		return this._run(jazz.client.xhrPut, uri, uriParam, arguments, callback, errorback);
	},
	_doDelete: function(uri, uriParam, arguments, callback, errorback) {
		return this._run(jazz.client.xhrDelete, uri, uriParam, arguments, callback, errorback);
	},

	_run: function(method, requestUri, uriParam, arguments, callback, errorback) {
		var uri = requestUri ? requestUri : com.ibm.relm.le.ui.qbServiceUri;
		if (uriParam) uri += uriParam;
		var param = {
				url: uri,
				headers: {
					"Accept": "application/json", //$NON-NLS-1$ //$NON-NLS-2$
					"Accept-Language": dojo.locale, //$NON-NLS-1$
				},
				handleAs: "json" //$NON-NLS-1$
		};
		if (arguments) dojo.mixin(param, arguments);
		var hasBody = arguments && (arguments.postData || arguments.putData) ? true : false;
		
		// use callback is specified when specified, otherwise return deferred object.
		if (callback || errorback) {
			param.load = callback;
			param.error = errorback;
			return method.call(this, param, hasBody);
		} else {
			var deferred = new dojo.Deferred();
			param.load = function(response, ioargs) {
				deferred.callback(response, ioargs);
			};
			param.error = function(response, ioargs) {
				var error = com.ibm.relm.le.ui.internal.util.getXhrErrorMessage(response, ioargs);
				deferred.errback(error);
			};
			method.call(this, param, hasBody);
			return deferred;
		}
	}

});
})();
