/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2015. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.util.OSLCServicesDocUtil");
dojo.require("dojox.xml.parser");
dojo.require("com.ibm.rqm.defects.web.util.XMLUtil");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.StringUtil");

(function(){
    var XMLUtil = com.ibm.rqm.defects.web.util.XMLUtil;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
    var StringUtil = com.ibm.rqm.planning.web.ui.internal.util.StringUtil;
    
    var PlanItemDomain = "http://open-services.net/ns/cm-x";
    var WorkItemDomain = "http://open-services.net/ns/cm";
    
	var ResourceTypeMap = {
		"http://open-services.net/ns/cm#ChangeRequest": "changeRequest",
		"http://open-services.net/ns/cm-x#Plan": "plans"
	};
	var RMResourceTypeList = [
		"http://open-services.net/ns/rm#Requirement",
		"http://open-services.net/ns/rm#RequirementCollection"
	];
	var AutoResourceTypeList = [
		"http://open-services.net/ns/auto#AutomationPlan",
		"http://open-services.net/ns/auto#AutomationResult",
		"http://open-services.net/ns/auto#AutomationRequest"
	];
	var AutoResourceUsageList = [
		"http://open-services.net/ns/core#Zero-or-one",
		"http://open-services.net/ns/core#Zero-or-many",
		"http://open-services.net/ns/core#default"
	];
	
	var UsageMap = {
		"http://open-services.net/ns/cm#defect": "defect",
		"http://open-services.net/ns/cm#planItem": "planItem",
		"http://open-services.net/ns/cm#task": "task",
		"http://open-services.net/ns/cm#requirementsChangeRequest": "requirementsChangeRequest"
	};
	
    function getURI(rootUrl, serviceDescNode){
        var uriNode = XMLUtil.getNode(serviceDescNode.childNodes, "url", XMLUtil.NAMESPACE_OSLC_CM);
        if (!uriNode) {
            uriNode = XMLUtil.getNode(serviceDescNode.childNodes, "url", XMLUtil.NAMESPACE_CALM)
        }
        var relativeURI = dojox.xml.parser.textContent(uriNode);
        return normalizeURI(rootUrl, relativeURI);
    }
    
    function normalizeURI(rootUrl, uri){
        // Strip leading "./" if this is a relative path.
        var re = /^.\//;
        if (re.test(uri)) {
            return rootUrl + "/" + uri.replace(re, "");
        }
        re = /^(..\/)+/;
        if (re.test(uri)) {
            return rootUrl + "/" + uri.replace(re, "");
        }
        // The URI is absolute.
        return uri;
    }
    
    function readServiceDescription(serviceDescDoc, projectLinkType, OSLCCoreVersion){
        var serviceDesc = {};
        if (projectLinkType == ProcessConstants.LinkTypes.TESTS || projectLinkType == ProcessConstants.LinkTypes.TRACKED_BY) {
			if(OSLCCoreVersion == "1.0"){
			    serviceDesc = readCmServiceDescription(serviceDescDoc);
			} else if (OSLCCoreVersion == "2.0") {
			    serviceDesc = readCmServiceDescriptionV2(serviceDescDoc);
			}
        }
        else{ 
            if (projectLinkType == ProcessConstants.LinkTypes.VALIDATES) {
				if(OSLCCoreVersion == "1.0"){
				    serviceDesc = readRmServiceDescriptionV1(serviceDescDoc);
				} else if (OSLCCoreVersion == "2.0") {
				    serviceDesc = readRmServiceDescriptionV2(serviceDescDoc);
				}
            }
            else if(projectLinkType == "executed-on"){
                serviceDesc = readAutoBuildServiceDescription(serviceDescDoc);
            }
            else if(projectLinkType == "automated-by")
                serviceDesc = readAutoServiceDescription(serviceDescDoc);
        }
        if (OSLCCoreVersion == "1.0") {
            return serviceDesc;
        }
        // read description for PLE for OSLC 2.0
        var pleDesc = readPLEServiceDescription(serviceDescDoc);
        return dojo.mixin(serviceDesc, pleDesc);
    }
    
    function readPLEServiceDescription(serviceDescDoc){
        var config = {};
        var xmlContent = serviceDescDoc;
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "RDF", XMLUtil.NAMESPACE_RDF);
        if (rootNode) {
            rootNode = XMLUtil.getNode(rootNode.childNodes, "ServiceProvider", XMLUtil.NAMESPACE_OSLC_CORE);
            var configNode = rootNode ? XMLUtil.getNode(rootNode.childNodes, "globalConfigurationAware", XMLUtil.NAMESPACE_JP) : null;
            if (configNode && configNode.childNodes && configNode.childNodes[0]) {
                config.globalConfigurationAware = configNode.childNodes[0].nodeValue;
            }
        }
        return config;
    }
    
    function readAutoBuildServiceDescription(serviceDescDoc){
        var xmlContent = serviceDescDoc;
        var creationDialogs = [], selectionDialogs = [];
        var buildPlanSelectDialog = null;
        var buildPlanCreateDialog = null;
        var buildResultSelectDialog = null;
        var buildResultCreateDialog = null;
        
        
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "RDF", XMLUtil.NAMESPACE_RDF);
		rootNode = XMLUtil.getNode(rootNode.childNodes, "ServiceProvider", XMLUtil.NAMESPACE_OSLC_CORE);
		var rootUrlNode = XMLUtil.getAttributeNode(rootNode, "about", XMLUtil.NAMESPACE_RDF);
		var rootUrl = XMLUtil.getAttributeNodeValue(rootUrlNode);
		var serviceNodes = XMLUtil.getNodes(rootNode.childNodes, "service", XMLUtil.NAMESPACE_OSLC_CORE);
        
        //here should check for oslc:usage to make sure it is automation for build
        
		dojo.forEach(serviceNodes, function(node){
			var serviceNode = XMLUtil.getNode(node.childNodes, "Service", XMLUtil.NAMESPACE_OSLC_CORE);

			var selectionDialogNodes = XMLUtil.getNodes(serviceNode.childNodes, "selectionDialog", XMLUtil.NAMESPACE_OSLC_CORE);
			dojo.forEach(selectionDialogNodes, function(node){
				var selectionDialog = _readDialogNode(node);
				selectionDialog = {
					title: selectionDialog.title,
					label1: selectionDialog.label1,
	                uri: selectionDialog.dialogURI,
	                width: selectionDialog.hintWidth,
	                height: selectionDialog.hintHeight,
					resourceType: selectionDialog.resourceType
	            };
				selectionDialogs.push(selectionDialog);
				if(selectionDialog.resourceType == AutoResourceTypeList[0]){
					buildPlanSelectDialog = selectionDialog;
				} else if(selectionDialog.resourceType == AutoResourceTypeList[1]){
					buildResultSelectDialog = selectionDialog;
				}				
			});
			var creationDialogNodes = XMLUtil.getNodes(serviceNode.childNodes, "creationDialog", XMLUtil.NAMESPACE_OSLC_CORE);
			dojo.forEach(creationDialogNodes, function(node){
				var creationDialog = _readDialogNode(node);
				creationDialog = {
					title: creationDialog.title,
					label1: creationDialog.label1,
	                uri: creationDialog.dialogURI,
	                width: creationDialog.hintWidth,
	                height: creationDialog.hintHeight,
					resourceType: creationDialog.resourceType
	            };
				creationDialogs.push(creationDialog);
				if(creationDialog.resourceType == AutoResourceTypeList[0]){
					buildPlanCreateDialog = creationDialog;
				} else if(creationDialog.resourceType == AutoResourceTypeList[1]){
					buildResultCreateDialog = creationDialog;
				}
				
			});

		});
		
		return {
			creationDialogs: creationDialogs,
			selectionDialogs: selectionDialogs,
            buildResultCreateDialog: buildResultCreateDialog,
            buildResultSelectDialog: buildResultSelectDialog,
            buildPlanCreateDialog: buildPlanCreateDialog,
            buildPlanSelectDialog: buildPlanSelectDialog            
        };    	    	
    }
    
    function readAutoServiceDescription(serviceDescDoc){
        var xmlContent = serviceDescDoc;
        var creationDialogs = [], selectionDialogs = [], queryCapabilities = [];
        var autoRequestSelectDialog = null;
        var autoRequestListDialog = null;
        var autoRequestCreateDialog = null;
        var autoResultSelectDialog = null;
        var autoResultListDialog = null;
        var autoResultCreateDialog = null;
        var autoPlanSelectDialog = null;
        var autoPlanListDialog = null;
        var autoPlanCreateDialog = null;
        var autoRequestQueryCapability = null;
        var autoResultQueryCapability = null;
        var autoPlanQueryCapability = null;
        
        
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "RDF", XMLUtil.NAMESPACE_RDF);
		rootNode = XMLUtil.getNode(rootNode.childNodes, "ServiceProvider", XMLUtil.NAMESPACE_OSLC_CORE);
		var rootUrlNode = XMLUtil.getAttributeNode(rootNode, "about", XMLUtil.NAMESPACE_RDF);
		var rootUrl = XMLUtil.getAttributeNodeValue(rootUrlNode);
		var serviceNodes = XMLUtil.getNodes(rootNode.childNodes, "service", XMLUtil.NAMESPACE_OSLC_CORE);
        
		dojo.forEach(serviceNodes, function(node){
			var serviceNode = XMLUtil.getNode(node.childNodes, "Service", XMLUtil.NAMESPACE_OSLC_CORE);

			var selectionDialogNodes = XMLUtil.getNodes(serviceNode.childNodes, "selectionDialog", XMLUtil.NAMESPACE_OSLC_CORE);
			dojo.forEach(selectionDialogNodes, function(node){
				var selectionDialog = _readDialogNode(node);
				selectionDialog = {
					title: selectionDialog.title,
					label1: selectionDialog.label1,
	                uri: selectionDialog.dialogURI,
	                width: selectionDialog.hintWidth,
	                height: selectionDialog.hintHeight,
					resourceType: selectionDialog.resourceType,
					usage: selectionDialog.usage
	            };
				selectionDialogs.push(selectionDialog);
				if(StringUtil.testStrInArray(selectionDialog.usage, AutoResourceUsageList[0])) {
					if(selectionDialog.resourceType == AutoResourceTypeList[0]){
						autoPlanSelectDialog = selectionDialog;
					} else if(selectionDialog.resourceType == AutoResourceTypeList[1]){
						autoResultSelectDialog = selectionDialog;
					} else if(selectionDialog.resourceType == AutoResourceTypeList[2]){
						autoRequestSelectDialog = selectionDialog;
					}
				}
				else if(StringUtil.testStrInArray(selectionDialog.usage, AutoResourceUsageList[1])){
					if(selectionDialog.resourceType == AutoResourceTypeList[0]){
						autoPlanListDialog = selectionDialog;
					} else if(selectionDialog.resourceType == AutoResourceTypeList[1]){
						autoResultListDialog = selectionDialog;
					} else if(selectionDialog.resourceType == AutoResourceTypeList[2]){
						autoRequestListDialog = selectionDialog;
					}
				} 				
				else if(StringUtil.testStrInArray(selectionDialog.usage, AutoResourceUsageList[2])){
					if(selectionDialog.resourceType == AutoResourceTypeList[0]){
						autoPlanSelectDialog = selectionDialog;
						autoPlanListDialog = selectionDialog;
					} else if(selectionDialog.resourceType == AutoResourceTypeList[1]){
						autoResultSelectDialog = selectionDialog;
						autoResultListDialog = selectionDialog;
					} else if(selectionDialog.resourceType == AutoResourceTypeList[2]){
						autoRequestSelectDialog = selectionDialog;
						autoRequestListDialog = selectionDialog;
					}
				} 				
			});
			var creationDialogNodes = XMLUtil.getNodes(serviceNode.childNodes, "creationDialog", XMLUtil.NAMESPACE_OSLC_CORE);
			dojo.forEach(creationDialogNodes, function(node){
				var creationDialog = _readDialogNode(node);
				creationDialog = {
					title: creationDialog.title,
					label1: creationDialog.label1,
	                uri: creationDialog.dialogURI,
	                width: creationDialog.hintWidth,
	                height: creationDialog.hintHeight,
					resourceType: creationDialog.resourceType
	            };
				creationDialogs.push(creationDialog);
				if(creationDialog.resourceType == AutoResourceTypeList[0]){
					autoPlanCreateDialog = creationDialog;
				} else if(creationDialog.resourceType == AutoResourceTypeList[1]){
					autoResultCreateDialog = creationDialog;
				} else if(creationDialog.resourceType == AutoResourceTypeList[2]){
					autoRequestCreateDialog = creationDialog;
				}
				
			});
			
			var queryCapabilityNodes = XMLUtil.getNodes(serviceNode.childNodes, "queryCapability", XMLUtil.NAMESPACE_OSLC_CORE);
			dojo.forEach(queryCapabilityNodes, function(node){
				var queryCapability = _readQueryCapabilityNode(node);
				queryCapability = {
					title: queryCapability.title,
					label1: queryCapability.label1,
	                resourceShape: queryCapability.resourceShape,
	                queryBase: queryCapability.queryBase,
	                usage: queryCapability.usage,
					resourceType: queryCapability.resourceType
	            };
				queryCapabilities.push(queryCapability);
				if(queryCapability.resourceType == AutoResourceTypeList[0]){
					autoPlanQueryCapability = queryCapability;
				} else if(queryCapability.resourceType == AutoResourceTypeList[1]){
					autoResultQueryCapability = queryCapability;
				} else if(queryCapability.resourceType == AutoResourceTypeList[2]){
					autoRequestQueryCapability = queryCapability;
				}
				
			});
			
		});
		
		return {
			creationDialogs: creationDialogs,
			selectionDialogs: selectionDialogs,
			queryCapabilities: queryCapabilities,
        	autoRequestSelectDialog: autoRequestSelectDialog,
        	autoRequestListDialog: autoRequestListDialog,
        	autoRequestCreateDialog: autoRequestCreateDialog,
        	autoResultSelectDialog: autoResultSelectDialog,
        	autoResultListDialog: autoResultListDialog,
        	autoResultCreateDialog: autoResultCreateDialog,
        	autoPlanSelectDialog: autoPlanSelectDialog,
        	autoPlanListDialog: autoPlanListDialog,
        	autoPlanCreateDialog: autoPlanCreateDialog,
        	autoRequestQueryCapability: autoRequestQueryCapability,
       		autoResultQueryCapability: autoResultQueryCapability,
        	autoPlanQueryCapability: autoPlanQueryCapability       
        };    	    	
    }
    
    /*For RM Spec Pre V1*/
    //As of Rational Quality Manager 5.0, the OSLC Requirements Management V1.0 consumer is deprecated and not supported or tested.  Please use the OSLC Requirements Management V2.0 consumer.
    function readRmServiceDescription(serviceDescDoc){
        var xmlContent = serviceDescDoc;
        var selectionDialogNode;
        
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "ServiceDescriptor", XMLUtil.NAMESPACE_OSLC_RM);
        var requirementsNode = XMLUtil.getNode(rootNode.childNodes, "requirements", XMLUtil.NAMESPACE_OSLC_RM);
        
        selectionDialogNode = XMLUtil.getNode(requirementsNode.childNodes, "SelectionDialog", XMLUtil.NAMESPACE_OSLC_RM);
        var reqSelectDialog = null;
        if (selectionDialogNode) {
            reqSelectDialog = _readRmDialog(selectionDialogNode);
        };
        
        var collectionsNode = XMLUtil.getNode(rootNode.childNodes, "collections", XMLUtil.NAMESPACE_OSLC_RM);
        selectionDialogNode = XMLUtil.getNode(collectionsNode.childNodes, "CollectionSelectionDialog", XMLUtil.NAMESPACE_OSLC_RM);
        var collectionSelectDialog = null;
        if (selectionDialogNode) {
            collectionSelectDialog = _readRmDialog(selectionDialogNode);
        }
        
        var linksNode = XMLUtil.getNode(rootNode.childNodes, "links", XMLUtil.NAMESPACE_OSLC_RM);
        var linkCreationServicesNode = XMLUtil.getNode(linksNode.childNodes, "LinkCreationServices", XMLUtil.NAMESPACE_OSLC_RM);
        var validatedByLinkNode = XMLUtil.getNode(linkCreationServicesNode.childNodes, "validatedByLinkFactory", XMLUtil.NAMESPACE_OSLC_RM);
        var linkCreationUri = XMLUtil.getAttributeNodeValue(XMLUtil.getAttributeNode(validatedByLinkNode, "resource", XMLUtil.NAMESPACE_RDF));
        
        return {
            reqSelectDialog: reqSelectDialog,
            collectionSelectDialog: collectionSelectDialog,
            linkCreationUri: linkCreationUri
        };
    }
    
     /*For RM Spec V1*/
     //As of Rational Quality Manager 5.0, the OSLC Requirements Management V1.0 consumer is deprecated and not supported or tested.  Please use the OSLC Requirements Management V2.0 consumer.
    function readRmServiceDescriptionV1(serviceDescDoc){
        var xmlContent = serviceDescDoc;
        var selectionDialogNode;
        var creationDialogNode;
        
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "ServiceDescriptor", XMLUtil.NAMESPACE_OSLC_RM);
        
        selectionDialogNode = XMLUtil.getNode(rootNode.childNodes, "requirementSelectionDialog", XMLUtil.NAMESPACE_OSLC_RM);
        var reqSelectDialog = null;
        if (selectionDialogNode) {
            var DialogNode = XMLUtil.getNode(selectionDialogNode.childNodes, "Dialog", XMLUtil.NAMESPACE_OSLC_RM);
            if (DialogNode) {
                reqSelectDialog = _readRmDialog(DialogNode);
            };
        }
        creationDialogNode = XMLUtil.getNode(rootNode.childNodes, "requirementCreationDialog", XMLUtil.NAMESPACE_OSLC_RM);
        var reqCreationDialog = null;
        if (creationDialogNode) {
            var DialogNode = XMLUtil.getNode(creationDialogNode.childNodes, "Dialog", XMLUtil.NAMESPACE_OSLC_RM);
            if (DialogNode) {
                reqCreationDialog = _readRmDialog(DialogNode);
            };
        }
        var collectionsselectionDialogNode = XMLUtil.getNode(rootNode.childNodes, "requirementCollectionSelectionDialog", XMLUtil.NAMESPACE_OSLC_RM);
        var collectionSelectDialog = null;
        if (collectionsselectionDialogNode) {
            var DialogNode = XMLUtil.getNode(collectionsselectionDialogNode.childNodes, "Dialog", XMLUtil.NAMESPACE_OSLC_RM);
            if (DialogNode) {
                collectionSelectDialog = _readRmDialog(DialogNode);
            }
        }
        
        var filterServiceNode = XMLUtil.getNode(rootNode.childNodes, "filter", XMLUtil.NAMESPACE_CALM);
        var filterService = null;
        if(filterServiceNode) {
            filterService = _readRmService(filterServiceNode);
        }
       /* 
        *var linksNode = XMLUtil.getNode(rootNode.childNodes, "links", XMLUtil.NAMESPACE_OSLC_RM);
        *var linkCreationServicesNode = XMLUtil.getNode(linksNode.childNodes, "LinkCreationServices", XMLUtil.NAMESPACE_OSLC_RM);
        *var validatedByLinkNode = XMLUtil.getNode(linkCreationServicesNode.childNodes, "validatedByLinkFactory", XMLUtil.NAMESPACE_OSLC_RM);
        *var linkCreationUri = XMLUtil.getAttributeNodeValue(XMLUtil.getAttributeNode(validatedByLinkNode, "resource", XMLUtil.NAMESPACE_RDF));
        */
        return {
            reqSelectDialog: reqSelectDialog,
            collectionSelectDialog: collectionSelectDialog,
           /* linkCreationUri: linkCreationUri,*/
            reqCreationDialog: reqCreationDialog,
            filterService: filterService
        };
    }
	
	/*
	 * Read the OSLC_RM v2 service description.  http://open-services.net/bin/view/Main/RmSpecificationV2Samples#Request_with_no_parameters_as_XM
	 */
	function readRmServiceDescriptionV2(serviceDescDoc){
        var xmlContent = serviceDescDoc;
        var defaultCreationDialog, defaultSelectionDialog, simpleQueryUri, workItemWebUri, homeUrl;
        var creationDialogs = [], selectionDialogs = [], queryServices = [];
        var reqSelectDialog = null;
        var reqCreationDialog = null;
        var collectionSelectDialog = null;
        var reqCollectionCreationDialog = null;
        var filterService;
        var collectionQueryService;
        var reqQueryService;
        var queryService = {};
        var suportsLinkDiscovery = false;
        
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "RDF", XMLUtil.NAMESPACE_RDF);
		rootNode = XMLUtil.getNode(rootNode.childNodes, "ServiceProvider", XMLUtil.NAMESPACE_OSLC_CORE);
		var rootUrlNode = XMLUtil.getAttributeNode(rootNode, "about", XMLUtil.NAMESPACE_RDF);
		var rootUrl = XMLUtil.getAttributeNodeValue(rootUrlNode);
		var serviceNodes = XMLUtil.getNodes(rootNode.childNodes, "service", XMLUtil.NAMESPACE_OSLC_CORE);
        var homeNode = XMLUtil.getNode(rootNode.childNodes, "home", XMLUtil.NAMESPACE_RM);
        
        //for doors requirement
        if(!homeNode){
        	homeNode = XMLUtil.getNode(rootNode.childNodes, "details", XMLUtil.NAMESPACE_OSLC_CORE);
        }
        
        var homeUrlNode = homeNode ? XMLUtil.getAttributeNode(homeNode, "resource", XMLUtil.NAMESPACE_RDF) : null;
        homeUrl = homeUrlNode ? XMLUtil.getAttributeNodeValue(homeUrlNode) : null;
        
		dojo.forEach(serviceNodes, function(node){
			var serviceNode = XMLUtil.getNode(node.childNodes, "Service", XMLUtil.NAMESPACE_OSLC_CORE);
//			var domainNode = XMLUtil.getNode(serviceNode.childNodes, "domain", XMLUtil.NAMESPACE_OSLC_CORE);
//			domainNode = XMLUtil.getAttributeNode(domainNode, "resource", XMLUtil.NAMESPACE_RDF);
//			var domain = XMLUtil.getAttributeNodeValue(domainNode);
			var selectionDialogNodes = XMLUtil.getNodes(serviceNode.childNodes, "selectionDialog", XMLUtil.NAMESPACE_OSLC_CORE);
			dojo.forEach(selectionDialogNodes, function(node){
				var selectionDialog = _readRmDialogV2(node);
				selectionDialog = {
					title: selectionDialog.title,
	                uri: selectionDialog.dialogURI,
	                width: selectionDialog.hintWidth,
	                height: selectionDialog.hintHeight,
					resourceType: selectionDialog.resourceType
	            };
				selectionDialogs.push(selectionDialog);
				if(selectionDialog.resourceType == RMResourceTypeList[0]){
					reqSelectDialog = selectionDialog;
				} else if(selectionDialog.resourceType == RMResourceTypeList[1]){
					collectionSelectDialog = selectionDialog;
				}				
			});
			var creationDialogNodes = XMLUtil.getNodes(serviceNode.childNodes, "creationDialog", XMLUtil.NAMESPACE_OSLC_CORE);
			dojo.forEach(creationDialogNodes, function(node){
				var creationDialog = _readRmDialogV2(node);
				creationDialog = {
					title: creationDialog.title,
	                uri: creationDialog.dialogURI,
	                width: creationDialog.hintWidth,
	                height: creationDialog.hintHeight,
					resourceType: creationDialog.resourceType
	            };
				creationDialogs.push(creationDialog);
				if(creationDialog.resourceType == RMResourceTypeList[0]){
					reqCreationDialog = creationDialog;
				} else if(creationDialog.resourceType == RMResourceTypeList[1]){
					reqCollectionCreationDialog = creationDialog;
				}
			});
			
			var filterServiceNode = XMLUtil.getNode(serviceNode.childNodes, "filter", XMLUtil.NAMESPACE_CALM);
          	
          	if(filterServiceNode && !filterService) {
				filterService = _readFilterService(filterServiceNode);
          	}
			
			//query capability
			var queryCapabilityNodes = XMLUtil.getNodes(serviceNode.childNodes, "queryCapability", XMLUtil.NAMESPACE_OSLC_CORE);
			dojo.forEach(queryCapabilityNodes, function(node){
				var queryCapabilityNode = node;
				queryCapabilityNode = XMLUtil.getNode(queryCapabilityNode.childNodes, "QueryCapability", XMLUtil.NAMESPACE_OSLC_CORE);
				var simpleQueryUriNode = XMLUtil.getNode(queryCapabilityNode.childNodes, "queryBase", XMLUtil.NAMESPACE_OSLC_CORE);
				simpleQueryUriNode = XMLUtil.getAttributeNode(simpleQueryUriNode, "resource", XMLUtil.NAMESPACE_RDF);
				var simpleQueryUri1 = XMLUtil.getAttributeNodeValue(simpleQueryUriNode);
				var usageNodes = XMLUtil.getNodes(queryCapabilityNode.childNodes, "usage", XMLUtil.NAMESPACE_OSLC_CORE);
				if(usageNodes){
					dojo.forEach(usageNodes, function(element){
						var usageNode = XMLUtil.getAttributeNode(element, "resource", XMLUtil.NAMESPACE_RDF);
						var usage = XMLUtil.getAttributeNodeValue(usageNode);
						if(usage == XMLUtil.NAMESPACE_DEFAULT){
							simpleQueryUri = simpleQueryUri1;
						}
					});
				}
				var resourceTypeNode = XMLUtil.getNode(queryCapabilityNode.childNodes, "resourceType", XMLUtil.NAMESPACE_OSLC_CORE);
				if (resourceTypeNode) {
					resourceTypeNode = XMLUtil.getAttributeNode(resourceTypeNode, "resource", XMLUtil.NAMESPACE_RDF);
					var resourceType = XMLUtil.getAttributeNodeValue(resourceTypeNode);
					queryService.url = simpleQueryUri1;
					queryService.resourceType = resourceType;
					queryServices.push(queryService);
				}
			});
		
			dojo.forEach(queryServices, function(queryService){
				if(queryService.resourceType == RMResourceTypeList[0]){
					reqQueryService = queryService;
				}		
				else if(queryService.resourceType == RMResourceTypeList[1])
						collectionQueryService = queryService;
			})
				
			
			
//			//query capability
//			var queryCapabilityNode = XMLUtil.getNode(serviceNode.childNodes, "queryCapability", XMLUtil.NAMESPACE_OSLC_CORE);
//			if(queryCapabilityNode){
//				queryCapabilityNode = XMLUtil.getNode(queryCapabilityNode.childNodes, "QueryCapability", XMLUtil.NAMESPACE_OSLC_CORE);
//				var simpleQueryUriNode = XMLUtil.getNode(queryCapabilityNode.childNodes, "queryBase", XMLUtil.NAMESPACE_OSLC_CORE);
//				simpleQueryUriNode = XMLUtil.getAttributeNode(simpleQueryUriNode, "resource", XMLUtil.NAMESPACE_RDF);
//				simpleQueryUri = XMLUtil.getAttributeNodeValue(simpleQueryUriNode);
//			}
//			
//			//add Embeddable Work Item Web Module uri
//			var moduleNode = XMLUtil.getNode(serviceNode.childNodes, "module", XMLUtil.NAMESPACE_CALM);
//			if(moduleNode){
//				moduleNode = XMLUtil.getNode(moduleNode.childNodes, "Module", XMLUtil.NAMESPACE_CALM);
//				moduleNode = XMLUtil.getNode(moduleNode.childNodes, "moduleDefinition", XMLUtil.NAMESPACE_CALM);
//				moduleNode = XMLUtil.getAttributeNode(moduleNode, "resource", XMLUtil.NAMESPACE_RDF);
//				workItemWebUri = XMLUtil.getAttributeNodeValue(moduleNode);
//			}
			
		});

		var supportLinkDiscoveryNode = rootNode ? XMLUtil.getNode(rootNode.childNodes, "supportLinkDiscoveryViaOSLCQueries", XMLUtil.NAMESPACE_JP) : null;
        if (supportLinkDiscoveryNode && dojox.xml.parser.textContent(supportLinkDiscoveryNode)) {        	
        	var discoversLinkTypeNode = rootNode ? XMLUtil.getNodes(rootNode.childNodes, "discoversLinkType", XMLUtil.NAMESPACE_JP) : null;
            if (discoversLinkTypeNode) {
            	_validatesReqColPropertyFlag = false;
            	_validatesRequirementFlag = false;
            	dojo.forEach(discoversLinkTypeNode, function(node){    				
    				var resourceNode = XMLUtil.getAttributeNode(node, "resource", XMLUtil.NAMESPACE_RDF);
    				var resourceNodeValue = XMLUtil.getAttributeNodeValue(resourceNode);
    				if(resourceNodeValue == "http://open-services.net/ns/qm#validatesRequirementCollection")
    					_validatesReqColPropertyFlag = true;
    				else if(resourceNodeValue == "http://open-services.net/ns/qm#validatesRequirement")
    					_validatesRequirementFlag = true;
            	});
            	if(_validatesRequirementFlag && _validatesReqColPropertyFlag)
            		suportsLinkDiscovery = true;
            }
        }
		return {
            homeUrl: homeUrl,
            reqSelectDialog: reqSelectDialog,
            collectionSelectDialog: collectionSelectDialog,
           /* linkCreationUri: linkCreationUri,*/
            reqCreationDialog: reqCreationDialog,
            collectionCreationDialog: reqCollectionCreationDialog,
            filterService: filterService,
            reqQueryService: reqQueryService,
            collectionQueryService: collectionQueryService,
            suportsLinkDiscovery: suportsLinkDiscovery
        };
    }
    
    function _readFilterService(filterServiceNode){
    	
    	var filterNode = XMLUtil.getNode(filterServiceNode.childNodes, "Filter", XMLUtil.NAMESPACE_CALM);
    	var titleNode = XMLUtil.getNode(filterNode.childNodes, "title", XMLUtil.NAMESPACE_DCTERMS);
    	var filterBaseNode = XMLUtil.getNode(filterNode.childNodes, "filerBase", XMLUtil.NAMESPACE_CALM);
    	var resourceNode = XMLUtil.getAttributeNode(filterBaseNode, "resource", XMLUtil.NAMESPACE_RDF);
    	
    	return {
            title: titleNode ? dojox.xml.parser.textContent(titleNode) : "",
            url: dojox.xml.parser.textContent(resourceNode)
        }
    	
    }
    
    function _readRmService(serviceNode){
        var serviceURI = XMLUtil.getNode(serviceNode.childNodes, "url", XMLUtil.NAMESPACE_CALM);
        var title = XMLUtil.getAttributeNode(serviceNode, "title", XMLUtil.NAMESPACE_CALM);
        return {
            title: title ? dojox.xml.parser.textContent(title) : "",
            url: dojox.xml.parser.textContent(serviceURI)
        }
    }
    
    function _readRmDialog(dialogNode){
        var selectionDialogURI;
        var selectionPreferedWidth, selectionPreferedHeight;
        var widgetNode = XMLUtil.getNode(dialogNode.childNodes, "widget", XMLUtil.NAMESPACE_OSLC_RM);
        var widgetResourceAttr = XMLUtil.getAttributeNode(widgetNode, "resource", XMLUtil.NAMESPACE_RDF);
        if (widgetResourceAttr) {
            selectionDialogURI = widgetResourceAttr.value;
        }
        var widthNode = XMLUtil.getNode(dialogNode.childNodes, "hintWidth", XMLUtil.NAMESPACE_OSLC_RM);
        if (widthNode) {
            selectionPreferedWidth = dojox.xml.parser.textContent(widthNode);
        }
        var heightNode = XMLUtil.getNode(dialogNode.childNodes, "hintHeight", XMLUtil.NAMESPACE_OSLC_RM);
        if (heightNode) {
            selectionPreferedHeight = dojox.xml.parser.textContent(heightNode);
        }
        return {
            uri: selectionDialogURI,
            width: selectionPreferedWidth,
            height: selectionPreferedHeight
        }
    }
    
	/*
	 * Read the OSLC_RM v2 delegate UI description. 
	 */
	
	function _readRmDialogV2(node){
		var dialog = _readDialogNode(node); 		
		return dialog;
	}
	
	function _readDialogNode(node){
		var dialog = {};
		
		var dialogNode = XMLUtil.getNode(node.childNodes, "Dialog", XMLUtil.NAMESPACE_OSLC_CORE);		
		
		var titleNode = XMLUtil.getNode(dialogNode.childNodes, "title", XMLUtil.NAMESPACE_DC);
		dialog.title = 	titleNode.childNodes[0].nodeValue;
		
		var labelNode = XMLUtil.getNode(dialogNode.childNodes, "label", XMLUtil.NAMESPACE_OSLC_CORE);
		dialog.label1 = labelNode.childNodes[0].nodeValue;
		
		var dialogURINode = XMLUtil.getNode(dialogNode.childNodes, "dialog", XMLUtil.NAMESPACE_OSLC_CORE);
		dialogURINode = XMLUtil.getAttributeNode(dialogURINode, "resource", XMLUtil.NAMESPACE_RDF);
		dialog.dialogURI = XMLUtil.getAttributeNodeValue(dialogURINode);
		
		dialog.hintWidth = XMLUtil.getNode(dialogNode.childNodes, "hintWidth", XMLUtil.NAMESPACE_OSLC_CORE).childNodes[0].nodeValue;
		dialog.hintHeight = XMLUtil.getNode(dialogNode.childNodes, "hintHeight", XMLUtil.NAMESPACE_OSLC_CORE).childNodes[0].nodeValue;
		
		var resourceType = XMLUtil.getNode(dialogNode.childNodes, "resourceType", XMLUtil.NAMESPACE_OSLC_CORE);
		resourceType = XMLUtil.getAttributeNode(resourceType, "resource", XMLUtil.NAMESPACE_RDF);
		dialog.resourceType = XMLUtil.getAttributeNodeValue(resourceType);
		
		var usages = XMLUtil.getNodes(dialogNode.childNodes, "usage", XMLUtil.NAMESPACE_OSLC_CORE);
		dialog.usage = [];
		if(usages && usages.length>0){
			dojo.forEach(usages, function(usage){
				usage = XMLUtil.getAttributeNode(usage, "resource", XMLUtil.NAMESPACE_RDF);
				dialog.usage.push(XMLUtil.getAttributeNodeValue(usage));
			});	
		}
		
		return dialog;		
	}
	
	function _readQueryCapabilityNode(node) {
		var query = {};
		
		var queryCapNode = XMLUtil.getNode(node.childNodes, "QueryCapability", XMLUtil.NAMESPACE_OSLC_CORE);		
		
		var usage = XMLUtil.getNode(queryCapNode.childNodes, "usage", XMLUtil.NAMESPACE_OSLC_CORE);
		usage = XMLUtil.getAttributeNode(usage, "resource", XMLUtil.NAMESPACE_RDF);
		query.usage = XMLUtil.getAttributeNodeValue(usage);
		
		var resourceType = XMLUtil.getNode(queryCapNode.childNodes, "resourceType", XMLUtil.NAMESPACE_OSLC_CORE);
		resourceType = XMLUtil.getAttributeNode(resourceType, "resource", XMLUtil.NAMESPACE_RDF);
		query.resourceType = XMLUtil.getAttributeNodeValue(resourceType);
		
		var resourceShape = XMLUtil.getNode(queryCapNode.childNodes, "resourceShape", XMLUtil.NAMESPACE_OSLC_CORE);
		resourceShape = XMLUtil.getAttributeNode(resourceShape, "resource", XMLUtil.NAMESPACE_RDF);
		query.resourceShape = XMLUtil.getAttributeNodeValue(resourceShape);
		
		var queryBase = XMLUtil.getNode(queryCapNode.childNodes, "queryBase", XMLUtil.NAMESPACE_OSLC_CORE);
		queryBase = XMLUtil.getAttributeNode(queryBase, "resource", XMLUtil.NAMESPACE_RDF);
		query.queryBase = XMLUtil.getAttributeNodeValue(queryBase);
		
		var titleNode = XMLUtil.getNode(queryCapNode.childNodes, "title", XMLUtil.NAMESPACE_DC);
		query.title = 	titleNode.childNodes[0].nodeValue;
		
		var labelNode = XMLUtil.getNode(queryCapNode.childNodes, "label", XMLUtil.NAMESPACE_OSLC_CORE);
		query.label1 = labelNode.childNodes[0].nodeValue;
		
		return query;
	}
    
    /***
     * @return {Object} {defaultCreationDialogURI, selectionDialogURI}
     * @param {XMLDoc} serviceDescDoc
     */
    function readCmServiceDescription(serviceDescDoc){
        var xmlContent = serviceDescDoc;
        var defaultCreationDialogURI, defaultSelectionDialogURI, creationDialogURI, selectionDialogURI, openItemsUri, blockedItemsUri, simpleQueryUri, workItemWebUri, providerIdentifier;
        var creationPreferedWidth, creationPreferedHeight, selectionPreferedWidth, selectionPreferedHeight;
        var creationDialogs = [], selectionDialogs = [];
        
        // get the rdf:about in case the uri is relative, it can be used as root
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "ServiceDescriptor", XMLUtil.NAMESPACE_OSLC_CM);
        var rootUrlNode = XMLUtil.getAttributeNode(rootNode, "about", XMLUtil.NAMESPACE_RDF);
        var rootUrl = XMLUtil.getAttributeNodeValue(rootUrlNode) ? XMLUtil.getAttributeNodeValue(rootUrlNode) : "./";
        var changeRequestsNode = XMLUtil.getNode(rootNode.childNodes, "changeRequests", XMLUtil.NAMESPACE_OSLC_CM);
        var plansNode = XMLUtil.getNode(rootNode.childNodes, "plans", XMLUtil.NAMESPACE_OSLC_CM);
        
		var simpleQueryNode = XMLUtil.getNodes(changeRequestsNode.childNodes, "simpleQuery", XMLUtil.NAMESPACE_OSLC_CM);
		if(simpleQueryNode.length > 0){
			simpleQueryUri = getURI(rootUrl, simpleQueryNode[0]);
		}
		
		var contributorNode = XMLUtil.getNode(rootNode.childNodes, "contributor", XMLUtil.NAMESPACE_DC);
		if(contributorNode){
			providerIdentifier = XMLUtil.getNode(contributorNode.childNodes, "identifier", XMLUtil.NAMESPACE_DC).childNodes[0].nodeValue;
		}
		
        var creationDialogNodes = XMLUtil.getNodes(changeRequestsNode.childNodes, "creationDialog", XMLUtil.NAMESPACE_OSLC_CM);
        dojo.forEach(creationDialogNodes, function(creationDialog){
            var isDefault = false;
            var id;
            var defaultAttr = XMLUtil.getAttributeNode(creationDialog, "default", XMLUtil.NAMESPACE_OSLC_CM);
            if (defaultAttr && XMLUtil.getAttributeNodeValue(defaultAttr) == "true") {
                isDefault = true;
                defaultCreationDialogURI = getURI(rootUrl, creationDialog);
            }
            var idAttr = XMLUtil.getAttributeNode(creationDialog, "id", XMLUtil.NAMESPACE_CALM);
            if (idAttr) {
                id = XMLUtil.getAttributeNodeValue(idAttr);
            }
            creationDialogURI = getURI(rootUrl, creationDialog);
            var widthAttr = XMLUtil.getAttributeNode(creationDialog, "hintWidth", XMLUtil.NAMESPACE_OSLC_CM);
            if (widthAttr) {
                creationPreferedWidth = XMLUtil.getAttributeNodeValue(widthAttr);
            }
            var heightAttr = XMLUtil.getAttributeNode(creationDialog, "hintHeight", XMLUtil.NAMESPACE_OSLC_CM);
            if (heightAttr) {
                creationPreferedHeight = XMLUtil.getAttributeNodeValue(heightAttr);
            }
            
            var dialogObject = {
                isDefault: isDefault,
                id: id,
                uri: creationDialogURI,
                width: creationPreferedWidth,
                height: creationPreferedHeight
            };
            
            var titleNode = XMLUtil.getNode(creationDialog.childNodes, "title", XMLUtil.NAMESPACE_DC)
			if(titleNode && titleNode.childNodes && titleNode.childNodes[0]){
				dialogObject.title = titleNode.childNodes[0].nodeValue;
			}
		
			var descriptionNode = XMLUtil.getNode(creationDialog.childNodes, "description", XMLUtil.NAMESPACE_DC)
			if(descriptionNode && descriptionNode.childNodes && descriptionNode.childNodes[0]){
				dialogObject.description = descriptionNode.childNodes[0].nodeValue;
			} 
            
            creationDialogs.push(dialogObject);
			
			
			// TODO: temp hard code one task creation dialog uri here. would consume changes in Support Task based creation dialog in service discovery (114993) later
			if (id == "defect") {
				creationDialogs.push({
	                isDefault: isDefault,
	                id: "task",
	                uri: creationDialogURI.replace("defect", "task"),
	                width: selectionPreferedWidth,
	                height: selectionPreferedHeight
	            });
			}
        });
        
        var selectionDialogNodes = XMLUtil.getNodes(changeRequestsNode.childNodes, "selectionDialog", XMLUtil.NAMESPACE_OSLC_CM);
        dojo.forEach(selectionDialogNodes, function(selectionDialog){
            var isDefault = false;
            var id;
            var defaultAttr = XMLUtil.getAttributeNode(selectionDialog, "default", XMLUtil.NAMESPACE_OSLC_CM);
            if (defaultAttr && (XMLUtil.getAttributeNodeValue(defaultAttr) == "true")) { // work for both IE & FF
                isDefault = true;
                defaultSelectionDialogURI = getURI(rootUrl, selectionDialog);
            }
            var idAttr = XMLUtil.getAttributeNode(selectionDialog, "id", XMLUtil.NAMESPACE_CALM);
            if (idAttr) {
                id = XMLUtil.getAttributeNodeValue(idAttr);
            }
            selectionDialogURI = getURI(rootUrl, selectionDialog);
            var widthAttr = XMLUtil.getAttributeNode(selectionDialog, "hintWidth", XMLUtil.NAMESPACE_OSLC_CM);
            if (widthAttr) {
                selectionPreferedWidth = XMLUtil.getAttributeNodeValue(widthAttr);
            }
            var heightAttr = XMLUtil.getAttributeNode(selectionDialog, "hintHeight", XMLUtil.NAMESPACE_OSLC_CM);
            if (heightAttr) {
                selectionPreferedHeight = XMLUtil.getAttributeNodeValue(heightAttr);
            }
            selectionDialogs.push({
                isDefault: isDefault,
                id: id,
                uri: selectionDialogURI,
                width: selectionPreferedWidth,
                height: selectionPreferedHeight
            });
        });
        
        // get selection dialogs for plan node
		if(plansNode){
	        var planSelectionDialogNodes = XMLUtil.getNodes(plansNode.childNodes, "selectionDialog", XMLUtil.NAMESPACE_OSLC_CM);
	        dojo.forEach(planSelectionDialogNodes, function(selectionDialog){
	            var isDefault = false;
	            var id;
	            var defaultAttr = XMLUtil.getAttributeNode(selectionDialog, "default", XMLUtil.NAMESPACE_OSLC_CM);
	            if (defaultAttr && (XMLUtil.getAttributeNodeValue(defaultAttr) == "true")) { // work for both IE & FF
	                isDefault = true;
	                //defaultSelectionDialogURI = getURI(rootUrl, selectionDialog);
	            }
	            var idAttr = XMLUtil.getAttributeNode(selectionDialog, "id", XMLUtil.NAMESPACE_CALM);
	            if (idAttr) {
	                id = XMLUtil.getAttributeNodeValue(idAttr);
	            }
	            selectionDialogURI = getURI(rootUrl, selectionDialog);
	            var widthAttr = XMLUtil.getAttributeNode(selectionDialog, "hintWidth", XMLUtil.NAMESPACE_OSLC_CM);
	            if (widthAttr) {
	                selectionPreferedWidth = XMLUtil.getAttributeNodeValue(widthAttr);
	            }
	            var heightAttr = XMLUtil.getAttributeNode(selectionDialog, "hintHeight", XMLUtil.NAMESPACE_OSLC_CM);
	            if (heightAttr) {
	                selectionPreferedHeight = XMLUtil.getAttributeNodeValue(heightAttr);
	            }
	            selectionDialogs.push({
	                type: "plans",
	                isDefault: isDefault,
	                id: id,
	                uri: selectionDialogURI,
	                width: selectionPreferedWidth,
	                height: selectionPreferedHeight
	            });
	        });
        }
        var filterNodes = XMLUtil.getNodes(changeRequestsNode.childNodes, "filter", XMLUtil.NAMESPACE_CALM);
        dojo.forEach(filterNodes, function(filterNode){
            var idAttr = XMLUtil.getAttributeNode(filterNode, "id", XMLUtil.NAMESPACE_CALM);
            if (idAttr && idAttr.value == "openItems") {
                openItemsUri = getURI(rootUrl, filterNode);
            }
            else 
                if (idAttr.value == "blockedItems") {
                    blockedItemsUri = getURI(rootUrl, filterNode);
                }
        });
		
		var moduleNodes = XMLUtil.getNodes(changeRequestsNode.childNodes, "module", XMLUtil.NAMESPACE_CALM);
        dojo.forEach(moduleNodes, function(moduleNode){
            var idAttr = XMLUtil.getAttributeNode(moduleNode, "id", XMLUtil.NAMESPACE_CALM);
            if (idAttr && idAttr.value == "embedded-cm") {
                workItemWebUri = getURI(rootUrl, moduleNode);
            }
        });
		
        // for back compatiable
        var defaultCreationDialog, defaultSelectionDialog;
        dojo.forEach(creationDialogs, function(creationDialog){
            if (creationDialog.isDefault || creationDialog.id == "defect") {
                defaultCreationDialog = creationDialog;
            }
        });
        dojo.forEach(selectionDialogs, function(selectionDialog){
            if (selectionDialog.isDefault || selectionDialog.id == "defect") {
                defaultSelectionDialog = selectionDialog;
            }
        });
        if (!defaultCreationDialogURI) {
            defaultCreationDialog = creationDialogs[0];
            defaultCreationDialogURI = creationDialogs[0].uri;
        }
        if (!defaultSelectionDialogURI) {
            defaultSelectionDialog = selectionDialogs[0];
            defaultSelectionDialogURI = selectionDialogs[0].uri;
        }
		
		// CM home that RQM can navigate to.
		var homeUrl = null;
		var cmHomeNode = XMLUtil.getNode(changeRequestsNode.childNodes, "home", XMLUtil.NAMESPACE_OSLC_CM);
		if (cmHomeNode) {
			var homeUrlNode = XMLUtil.getNode(cmHomeNode.childNodes, "url", XMLUtil.NAMESPACE_OSLC_CM);
			homeUrl = dojox.xml.parser.textContent(homeUrlNode);
		}
		//If it's self integration, replace '/web/projects/' of homeUrl with '/web/console/'. Add additional segment '/web' to match string to  
		//avoid replace by fault.
		if(homeUrl && homeUrl.indexOf(dojo.getObject(Constants.PUBLIC_URL))  == 0){
			homeUrl = homeUrl.replace('/web/projects/', '/web/console/');
		}
        
        // parse the supported RPCs.
        var rpcNodes = XMLUtil.getNodes(changeRequestsNode.childNodes, "rpc", XMLUtil.NAMESPACE_CALM);
        var rpcObjects = [];
        dojo.forEach(rpcNodes, function(rpcNode) {
            var id, title, rpcArg;
            var idAttr = XMLUtil.getAttributeNode(rpcNode, "id", XMLUtil.NAMESPACE_CALM);
            if(idAttr) {
                id = XMLUtil.getAttributeNodeValue(idAttr);
            }
            var titleNode = XMLUtil.getNode(rpcNode.childNodes, "title", XMLUtil.NAMESPACE_DC);
            if(titleNode) {
                title = dojox.xml.parser.textContent(titleNode);
            }
            var rpcargNode = XMLUtil.getNode(rpcNode.childNodes, "rpcArg", XMLUtil.NAMESPACE_CALM);
            if(rpcargNode) {
                rpcArg = dojox.xml.parser.textContent(rpcargNode);
            }
            var rpcObj = {
                'id': id,
                'title': title,
                'rpcArg': rpcArg
            };
            rpcObjects.push(rpcObj);
        });
        
        var obj = {
            defaultCreationDialogURI: defaultCreationDialogURI,
            creationDialog: defaultCreationDialog,
            creationDialogs: creationDialogs,
            selectionDialogURI: defaultSelectionDialogURI,
            selectionDialog: defaultSelectionDialog,
            selectionDialogs: selectionDialogs,
            openItemsUri: openItemsUri,
            blockedItemsUri: blockedItemsUri,
			homeUrl: homeUrl,
			simpleQueryUri: simpleQueryUri,
			rpcs: rpcObjects,
			providerIdentifier: providerIdentifier
        };
		if (workItemWebUri) {
			obj.workItemWebUri = workItemWebUri;
		}
        return obj;
    }
	
	
	/*
	 * Read the OSLC_CM v2 delegate UI description.  http://open-services.net/bin/view/Main/CmSpecificationV2Samples#Request_with_no_parameters_as_XM
	 */
	
	function _readDelegateUIDescription(node){
		var dialog = {
			usages: []
		};
		var dialogNode = XMLUtil.getNode(node.childNodes, "Dialog", XMLUtil.NAMESPACE_OSLC_CORE);
		
		var usageNodes = XMLUtil.getNodes(dialogNode.childNodes, "usage", XMLUtil.NAMESPACE_OSLC_CORE);
		if(dojo.isArray(usageNodes) && usageNodes.length > 0){
			dojo.forEach(usageNodes, function(usageNode){
				if(usageNode){
					usageNode = XMLUtil.getAttributeNode(usageNode, "resource", XMLUtil.NAMESPACE_RDF);
					if(usageNode){
						var usage = XMLUtil.getAttributeNodeValue(usageNode);
						dialog.usages.push(usage);
						if(usage == XMLUtil.NAMESPACE_DEFAULT){
							dialog.isDefault = true;
						}
					}
				}
			});
		}
		
		var dialogURINode = XMLUtil.getNode(dialogNode.childNodes, "dialog", XMLUtil.NAMESPACE_OSLC_CORE);
		dialogURINode = XMLUtil.getAttributeNode(dialogURINode, "resource", XMLUtil.NAMESPACE_RDF);
		dialog.dialogURI = XMLUtil.getAttributeNodeValue(dialogURINode);
		
		dialog.hintWidth = XMLUtil.getNode(dialogNode.childNodes, "hintWidth", XMLUtil.NAMESPACE_OSLC_CORE).childNodes[0].nodeValue;
		dialog.hintHeight = XMLUtil.getNode(dialogNode.childNodes, "hintHeight", XMLUtil.NAMESPACE_OSLC_CORE).childNodes[0].nodeValue;
		
		var resourceTypeNode = XMLUtil.getNode(dialogNode.childNodes, "resourceType", XMLUtil.NAMESPACE_OSLC_CORE);
		resourceTypeNode = XMLUtil.getAttributeNode(resourceTypeNode, "resource", XMLUtil.NAMESPACE_RDF);
		dialog.resourceType = XMLUtil.getAttributeNodeValue(resourceTypeNode);
		
		var titleNode = XMLUtil.getNode(dialogNode.childNodes, "title", XMLUtil.NAMESPACE_DC)
		if(titleNode && titleNode.childNodes && titleNode.childNodes[0]){
			dialog.title = titleNode.childNodes[0].nodeValue;
		}
		
		var descriptionNode = XMLUtil.getNode(dialogNode.childNodes, "description", XMLUtil.NAMESPACE_DC)
		if(descriptionNode && descriptionNode.childNodes && descriptionNode.childNodes[0]){
			dialog.description = descriptionNode.childNodes[0].nodeValue;
		}
		
		return dialog;
	}
	
	/*
	 * Read the OSLC_CM v2 service description.  http://open-services.net/bin/view/Main/CmSpecificationV2Samples#Request_with_no_parameters_as_XM
	 */
	function readCmServiceDescriptionV2(serviceDescDoc){
        var xmlContent = serviceDescDoc;
        var hasFoundHomeUrl = false;
        var defaultCreationDialog, defaultSelectionDialog, simpleQueryUri, workItemWebUri, homeUrl, providerIdentifier, whoamiUrl;
        var creationDialogs = [], selectionDialogs = [], rpcObjects = [], simpleQueryUris = [], resourceShapes = [];
        
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "RDF", XMLUtil.NAMESPACE_RDF);
		rootNode = XMLUtil.getNode(rootNode.childNodes, "ServiceProvider", XMLUtil.NAMESPACE_OSLC_CORE);
		var rootUrlNode = XMLUtil.getAttributeNode(rootNode, "about", XMLUtil.NAMESPACE_RDF);
		var rootUrl = XMLUtil.getAttributeNodeValue(rootUrlNode);
		
		var publisherNode = XMLUtil.getNode(rootNode.childNodes, "publisher", XMLUtil.NAMESPACE_DC);
		if(publisherNode){
			publisherNode = XMLUtil.getNode(publisherNode.childNodes, "Publisher", XMLUtil.NAMESPACE_OSLC_CORE);
			providerIdentifier = XMLUtil.getNode(publisherNode.childNodes, "identifier", XMLUtil.NAMESPACE_DC).childNodes[0].nodeValue;
		}
		
		var whoamiNode = XMLUtil.getNode(rootNode.childNodes, "whoami", XMLUtil.NAMESPACE_CMX);
		if(whoamiNode){
			whoamiNode = XMLUtil.getAttributeNode(whoamiNode, "resource", XMLUtil.NAMESPACE_RDF);
			whoamiUrl = XMLUtil.getAttributeNodeValue(whoamiNode);
		}
		
		var serviceNodes = XMLUtil.getNodes(rootNode.childNodes, "service", XMLUtil.NAMESPACE_OSLC_CORE);
		dojo.forEach(serviceNodes, function(node){
			var serviceNode = XMLUtil.getNode(node.childNodes, "Service", XMLUtil.NAMESPACE_OSLC_CORE);
			var domainNode = XMLUtil.getNode(serviceNode.childNodes, "domain", XMLUtil.NAMESPACE_OSLC_CORE);
			domainNode = XMLUtil.getAttributeNode(domainNode, "resource", XMLUtil.NAMESPACE_RDF);
			var domain = XMLUtil.getAttributeNodeValue(domainNode);
			var selectionDialogNodes = XMLUtil.getNodes(serviceNode.childNodes, "selectionDialog", XMLUtil.NAMESPACE_OSLC_CORE);
			dojo.forEach(selectionDialogNodes, function(node){
				var selectionDialog = _readDelegateUIDescription(node);
				var selectionDialog1 = {
					type: ResourceTypeMap[selectionDialog.resourceType],
	                isDefault: selectionDialog.isDefault,
	                uri: selectionDialog.dialogURI,
	                width: selectionDialog.hintWidth,
	                height: selectionDialog.hintHeight
	            };
				if(selectionDialog.usages.length > 0){
					dojo.forEach(selectionDialog.usages, function(usage){
						var id = UsageMap[usage];
						if(id){
							selectionDialog1.id = id;
						}
					});
				}
				selectionDialogs.push(selectionDialog1);
				if(selectionDialog1.isDefault){
					defaultSelectionDialog = selectionDialog1;
				}
			});
			var creationDialogNodes = XMLUtil.getNodes(serviceNode.childNodes, "creationDialog", XMLUtil.NAMESPACE_OSLC_CORE);
			dojo.forEach(creationDialogNodes, function(node){
				var creationDialog = _readDelegateUIDescription(node);
				
				var creationDialog1 = {
					type: ResourceTypeMap[creationDialog.resourceType],
	                isDefault: creationDialog.isDefault,
	                uri: creationDialog.dialogURI,
	                width: creationDialog.hintWidth,
	                height: creationDialog.hintHeight
	            };
	            
	            if(creationDialog.title){
	            	creationDialog1.title = creationDialog.title;
	            }
	            
	            if(creationDialog.description){
	            	creationDialog1.description = creationDialog.description;
	            }
	            
				if(creationDialog.usages.length > 0){
					dojo.forEach(creationDialog.usages, function(usage){
						var id = UsageMap[usage];
						if(id){
							creationDialog1.id = id;
						}
					});
				}
				creationDialogs.push(creationDialog1);
				if(creationDialog1.isDefault){
					defaultCreationDialog = creationDialog1;
				}
			});
			if(!defaultCreationDialog && creationDialogs.length){
				defaultCreationDialog = creationDialogs[0];
			}
			
			//query capability
			var queryCapabilityNodes = XMLUtil.getNodes(serviceNode.childNodes, "queryCapability", XMLUtil.NAMESPACE_OSLC_CORE);
			dojo.forEach(queryCapabilityNodes, function(node){
				var queryCapabilityNode = node;
				queryCapabilityNode = XMLUtil.getNode(queryCapabilityNode.childNodes, "QueryCapability", XMLUtil.NAMESPACE_OSLC_CORE);
				var simpleQueryUriNode = XMLUtil.getNode(queryCapabilityNode.childNodes, "queryBase", XMLUtil.NAMESPACE_OSLC_CORE);
				simpleQueryUriNode = XMLUtil.getAttributeNode(simpleQueryUriNode, "resource", XMLUtil.NAMESPACE_RDF);
				var simpleQueryUri1 = XMLUtil.getAttributeNodeValue(simpleQueryUriNode);
				var usageNodes = XMLUtil.getNodes(queryCapabilityNode.childNodes, "usage", XMLUtil.NAMESPACE_OSLC_CORE);
				if(usageNodes){
					dojo.forEach(usageNodes, function(element){
						var usageNode = XMLUtil.getAttributeNode(element, "resource", XMLUtil.NAMESPACE_RDF);
						var usage = XMLUtil.getAttributeNodeValue(usageNode);
						if(usage == XMLUtil.NAMESPACE_DEFAULT){
							simpleQueryUri = simpleQueryUri1;
						}
					});
				}
				simpleQueryUris.push(simpleQueryUri1);
			});
			if(!simpleQueryUri && simpleQueryUris.length){
				simpleQueryUri = simpleQueryUris[0];
			}
			
			//add Embeddable Work Item Web Module uri
			var moduleNode = XMLUtil.getNode(serviceNode.childNodes, "module", XMLUtil.NAMESPACE_CALM);
			if(moduleNode){
				moduleNode = XMLUtil.getNode(moduleNode.childNodes, "Module", XMLUtil.NAMESPACE_CALM);
				moduleNode = XMLUtil.getNode(moduleNode.childNodes, "moduleDefinition", XMLUtil.NAMESPACE_CALM);
				moduleNode = XMLUtil.getAttributeNode(moduleNode, "resource", XMLUtil.NAMESPACE_RDF);
				workItemWebUri = XMLUtil.getAttributeNodeValue(moduleNode);
			}
			
			//add CM home that RQM can navigate to. still not provided in RTC build services.xml now
			var cmHomeNode = XMLUtil.getNode(serviceNode.childNodes, "home", XMLUtil.NAMESPACE_CALM);
			if (cmHomeNode) {
				cmHomeNode = XMLUtil.getNode(cmHomeNode.childNodes, "Home", XMLUtil.NAMESPACE_CALM);
				if(cmHomeNode){
					cmHomeNode = XMLUtil.getNode(cmHomeNode.childNodes, "webHome", XMLUtil.NAMESPACE_CALM);
					var homeUrlNode = XMLUtil.getAttributeNode(cmHomeNode, "resource", XMLUtil.NAMESPACE_RDF);
					
					if(!hasFoundHomeUrl){
						homeUrl = XMLUtil.getAttributeNodeValue(homeUrlNode);
					}
					
					//64551, only find the home url for work items home, not plan items home
					if(domain == WorkItemDomain || domain == WorkItemDomain + "#"){
						hasFoundHomeUrl = true;
					}
					
				}
			}
			
			// parse the supported RPCs.
			var rpcNodes = XMLUtil.getNodes(serviceNode.childNodes, "rpc", XMLUtil.NAMESPACE_CALM);
			if(rpcNodes){
				dojo.forEach(rpcNodes, function(rpcNode) {
					rpcNode = XMLUtil.getNode(rpcNode.childNodes, "OpenSocialRPC", XMLUtil.NAMESPACE_CALM);
		            var id, title, rpcArg;
		            var idAttr = XMLUtil.getAttributeNode(rpcNode, "id", XMLUtil.NAMESPACE_CALM);
		            if(idAttr) {
		                id = XMLUtil.getAttributeNodeValue(idAttr);
		            }
		            var titleNode = XMLUtil.getNode(rpcNode.childNodes, "title", XMLUtil.NAMESPACE_DC);
		            if(titleNode) {
		                title = dojox.xml.parser.textContent(titleNode);
		            }
		            var rpcargNode = XMLUtil.getNode(rpcNode.childNodes, "rpcArg", XMLUtil.NAMESPACE_CALM);
		            if(rpcargNode) {
		                rpcArg = dojox.xml.parser.textContent(rpcargNode);
		            }
		            var rpcObj = {
		                'id': id,
		                'title': title,
		                'rpcArg': rpcArg
		            };
		            rpcObjects.push(rpcObj);
		        });
			}
			
			//creationFactoryNodes, extract the resourceShapes
			var creationFactoryNodes = XMLUtil.getNodes(serviceNode.childNodes, "creationFactory", XMLUtil.NAMESPACE_OSLC_CORE);
			if(creationFactoryNodes && creationFactoryNodes.length > 0){
				dojo.forEach(creationFactoryNodes, function(node){
					var creationFactoryNode = XMLUtil.getNode(node.childNodes, "CreationFactory", XMLUtil.NAMESPACE_OSLC_CORE);
					
					if(creationFactoryNode){
					
						var resourceShapeNodes = XMLUtil.getNodes(creationFactoryNode.childNodes, "resourceShape", XMLUtil.NAMESPACE_OSLC_CORE);				
						if(resourceShapeNodes && resourceShapeNodes.length > 0){
						
							for(var step = 0;step < resourceShapeNodes.length;step ++){
								var resourceShapeNode = resourceShapeNodes[step];
								var resourceShapeResourceNode = XMLUtil.getAttributeNode(resourceShapeNode, "resource", XMLUtil.NAMESPACE_RDF);
								if(resourceShapeResourceNode){
									var resourceShapeUri = XMLUtil.getAttributeNodeValue(resourceShapeResourceNode);
									if(resourceShapeUri){
										resourceShapes.push(resourceShapeUri);
									}
								}	
							}
						
						}
					}
				});
			}
		});
        
		var defSelectionDialogUri = (typeof defaultSelectionDialog != 'undefined' && defaultSelectionDialog.uri) ?
				defaultSelectionDialog.uri : null;
		var defCreationDialogUri = (typeof defaultCreationDialog != 'undefined' && defaultCreationDialog.uri) ?
				defaultCreationDialog.uri : null;
        var obj = {
            defaultCreationDialogURI: defCreationDialogUri,
            creationDialog: defaultCreationDialog,
            creationDialogs: creationDialogs,
            selectionDialogURI: defSelectionDialogUri,
            selectionDialog: defaultSelectionDialog,
            selectionDialogs: selectionDialogs,
			simpleQueryUri: simpleQueryUri,
			rpcs: rpcObjects,
			providerIdentifier: providerIdentifier,
			whoamiUrl: whoamiUrl,
			resourceShapes: resourceShapes
        };
		if (workItemWebUri) {
			obj.workItemWebUri = workItemWebUri;
		}
		if (homeUrl){
			//If it's self integration, replace '/web/projects/' of homeUrl with '/web/console/'. Add additional segment '/web' to match string to  
			//avoid replace by fault.
			if(homeUrl.indexOf(dojo.getObject(Constants.PUBLIC_URL))  == 0){
				homeUrl = homeUrl.replace('/web/projects/', '/web/console/');
			}
			obj.homeUrl = homeUrl;
		}
        return obj;
    }
    
    /***
     * @return {Object} links
     * @param {Object} projectLinkDescDoc
     */
    function readProjectLinksDescription(projectLinkDescDoc, projectLinkType){
        var links = [];
        var linksRootNode = XMLUtil.getNode(projectLinkDescDoc.childNodes, "links", XMLUtil.NAMESPACE_JP);
		if (linksRootNode) {
			var linkNodes = XMLUtil.getNodes(linksRootNode.childNodes, "link", XMLUtil.NAMESPACE_JP);
			dojo.forEach(linkNodes, function(linkNode){
				var url = null;
				var linkType = null;
				var servicesUrl = null;
				var name = null;
				dojo.forEach(linkNode.childNodes, function(childNode){
					// children of "link"
					if (childNode.tagName == "jp:url") {
						url = childNode.firstChild.data;
					}
					else 
						if (childNode.tagName == "jp:link-type") {
							linkType = childNode.firstChild.data;
						}
						else 
							if (childNode.tagName == "oslc:ServiceProvider") {
								// children of "ServiceProvider"
								dojo.forEach(childNode.childNodes, function(childNode){
									if (childNode.tagName == "dc:title") {
										name = childNode.firstChild.data;
									}
									else 
										if (childNode.tagName == "oslc:services") {
											servicesUrl = childNode.getAttribute("rdf:resource");
										}
								});
								
							}
				});

                // compose a unique key using host of services url and project area name
                var providerProjectKey = null;
                if (servicesUrl && name) {
                    var hostStart = servicesUrl.indexOf("//");
                    var hostEnd = servicesUrl.indexOf("/", hostStart + 2);
                    var providerHost = servicesUrl.substring(hostStart + 2, hostEnd);
                    providerProjectKey = providerHost + "_" + name;
                }
                
				if (name && linkType == projectLinkType) {
					links.push({
						url: url,
						linkType: linkType,
						servicesUrl: servicesUrl,
						name: name,
                        providerProjectKey: providerProjectKey
					});
				}

			});

		}
        return links;
        
    }
	
	function readWhoamiUri(serviceDescDoc){
        var xmlContent = serviceDescDoc;
        
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "RDF", XMLUtil.NAMESPACE_RDF);
		var personNode = XMLUtil.getNode(rootNode.childNodes, "Person", XMLUtil.NAMESPACE_FOAF);
		var personUrlNode = XMLUtil.getAttributeNode(personNode, "about", XMLUtil.NAMESPACE_RDF);
		var personUrl = XMLUtil.getAttributeNodeValue(personUrlNode);
        
		return personUrl;
    }
	
	function readServiceProviderProperty(doc){
        var xmlContent = doc;
        
        var rootNode = XMLUtil.getNode(xmlContent.childNodes, "RDF", XMLUtil.NAMESPACE_RDF);
		var descriptionNode = XMLUtil.getNode(rootNode.childNodes, "Description", XMLUtil.NAMESPACE_RDF);
		var serviceProviderNode = XMLUtil.getNode(descriptionNode.childNodes, "serviceProvider", XMLUtil.NAMESPACE_OSLC_CORE);
		var resourceNode = XMLUtil.getAttributeNode(serviceProviderNode, "resource", XMLUtil.NAMESPACE_RDF);
		var serviceProviderUrl = XMLUtil.getAttributeNodeValue(resourceNode);
        
		return serviceProviderUrl;
    }
   	
    
    com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readServiceDescription = readServiceDescription;
    com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readProjectLinksDescription = readProjectLinksDescription;
	com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readWhoamiUri = readWhoamiUri;
	com.ibm.rqm.defects.web.util.OSLCServicesDocUtil.readServiceProviderProperty = readServiceProviderProperty;
})();
