/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.lpa.app.ui.internal.MemberDetailsView"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$

dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
dojo.require("jazz.ui.HoverPopup"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledTable"); //$NON-NLS-1$
dojo.require("jazz.ui.StyledBox"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$

dojo.require("jazz.util.html"); //$NON-NLS-1$

dojo.require("com.ibm.team.lpa.app.client.LifecycleProjectAdminClient"); //$NON-NLS-1$

var LpaClient = com.ibm.team.lpa.app.client.LifecycleProjectAdminClient;

dojo.requireLocalization("com.ibm.team.lpa.app.ui.internal", "MemberDetailsViewMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var Button = jazz.ui.toolbar.Button;
var HoverPopup = jazz.ui.HoverPopup;
var ResourceLink = jazz.ui.ResourceLink;
var StyledBox = jazz.ui.StyledBox;
var Toolbar = jazz.ui.Toolbar;
var TrimStyles = jazz.ui.StyledTable.TrimStyles;

//
// Table Row Properties
//
var ROW_BUTTONS_PROP = "_jazzRowButtons"; //$NON-NLS-1$
var ROW_CONTAINER_PROP = "_jazzArtifactContainer"; //$NON-NLS-1$

//
// Topics
//
var TOPIC_ADD_MEMBER_TO_DELETE_REQUEST = "com.ibm.team.lpa.app.ui.internal.MembersPage#addMemberToDelete"; //$NON-NLS-1$
var TOPIC_REMOVE_MEMBER_FROM_DELETE_REQUEST = "com.ibm.team.lpa.app.ui.internal.MembersPage#removeMemberFromDelete"; //$NON-NLS-1$

var TOPIC_REFRESH_MEMBER_ROW = "com.ibm.team.lpa.app.ui.internal.MembersPage#refreshMemberRow"; //$NON-NLS-1$

//
// Static image src URIs
//
var GRAPHICS_FOLDER = "ui/graphics/"; //$NON-NLS-1$
var GRAPHICS_ICONS_FOLDER = GRAPHICS_FOLDER + "icons/"; //$NON-NLS-1$

var ADDMEMBER_IMG_SRC = dojo.moduleUrl("com.ibm.team.lpa.app",GRAPHICS_ICONS_FOLDER + "add-member_co.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var EDIT_IMG_SRC = dojo.moduleUrl("com.ibm.team.lpa.app",GRAPHICS_ICONS_FOLDER + "edit-artifact-container_co.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var EDIT_ROLES_SRC = dojo.moduleUrl("com.ibm.team.lpa.app",GRAPHICS_ICONS_FOLDER + "process-roles.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var ERROR_IMG_SRC = dojo.moduleUrl("com.ibm.team.lpa.app",GRAPHICS_ICONS_FOLDER + "error_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var RESTORE_IMG_SRC = dojo.moduleUrl("com.ibm.team.lpa.app",GRAPHICS_ICONS_FOLDER + "restore_16.png"); //$NON-NLS-1$ //$NON-NLS-2$
var REMOVE_IMG_SRC = dojo.moduleUrl("com.ibm.team.lpa.app",GRAPHICS_ICONS_FOLDER + "remove_16.png"); //$NON-NLS-1$ //$NON-NLS-2$
var GRAY_CHECKMARK_IMG_SRC = dojo.moduleUrl("com.ibm.team.lpa.app", GRAPHICS_ICONS_FOLDER + "gray-checkmark_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
var BLACK_CHECKMARK_IMG_SRC = dojo.moduleUrl("com.ibm.team.lpa.app", GRAPHICS_ICONS_FOLDER + "black-checkmark_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$

var ELLIPSIS_HTML =  '&#8230;'; //$NON-NLS-1$

//
// Helper to retrieve the row in which an event occured 
//
function getRow(e) {
	return getElement(e, "TR"); //$NON-NLS-1$
}

function getElement(e, tagName){
	var element= e.target || e.srcElement;
	// walk up the DOM tree until we find a row element
	while (element && element.tagName !== tagName) {
		element= element.parentNode;
	}
	return element || null;
}

dojo.declare("com.ibm.team.lpa.app.ui.internal.MemberDetailsView", [dijit._Widget, dijit._Templated],  { //$NON-NLS-1$
	templatePath: dojo.moduleUrl("com.ibm.team.lpa.app","ui/internal/templates/MemberDetailsView.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	/**  
	 * {Required} 
	 * 
	 * An array of project contexts
	 */
	contexts: null,
	
	/**
	 * {Required}
	 * 
	 * The member whose details we are displaying.
	 */   
	member: null,
	
	/**
	 * {Required}
	 * 
	 * The project the user is a member of.
	 */  
	project: null,
	
	/**  
	 * {Optional} 
	 * 
	 * True to display errors in the UI, False otherwise.
	 */
	displayErrors: true, 
	
	/** 
	 * {Optional}
	 * 
	 *  True if we want to the details to be displayed with Member Preview elements.
	 *  False if we should display all data about a member.
	 */ 
	isPreview: false,  
	
	/** 
	 * ===================
	 * Private Variables
	 * ===================
	 */
	_contextTable: null,
	
	constructor: function(args){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.lpa.app.ui.internal", "MemberDetailsViewMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		
		this.contexts = args.contexts;
		this.member = args.member;
		this.project = args.project;
		
		this._simpleButtonMap = {};
		
		this._childrenWidgets = [];
		this._connectHandles = [];
	},
	
	postCreate: function(){
		if(!this.isPreview){
			this.instructionNode.appendChild(document.createTextNode(this.messages.editMemberInstruction));
		}
		this._buildContextTable();
	},
	
	destroy: function(){
		this._destroyChildWidgets();
		
		if (this._simpleButtonMap){
			for (var prop in this._simpleButtonMap) {
				delete this._simpleButtonMap[prop];
			}
			delete this._simpleButtonMap;
		}
		
		this.inherited(arguments);
	},
	
	refresh: function(){
		this._destroyChildWidgets(true);
		this._buildContextTable();
	},
	
	//
	// Handle the destruction of child widgets
	//
	// params:
	// 	(optional) shouldCleanWidgetArrs:
	//					if set to TRUE, will reset the arrays that track child widgets
	//					if set to FALSE, will delete the reference to the respective arrays
	//
	_destroyChildWidgets: function(shouldCleanWidgetArrs){
		if (shouldCleanWidgetArrs) {
			// if we are just cleaning up the widget / performing a refresh
			// we have to reset the table nodes
			this.contextTableNode = dojo.create("div", {}); //$NON-NLS-1$
			this._contextTable.domNode.parentNode.replaceChild(this.contextTableNode, this._contextTable.domNode);
		}
			
		dojo.forEach(this._childrenWidgets, function(widget){
			widget.destroy();
		});
		
		dojo.forEach(this._connectHandles, dojo.disconnect);
		
		if (shouldCleanWidgetArrs){
			this._childrenWidgets = [];
			this._connectHandles = [];
		} else {
			if (this._childrenWidgets) delete this._childrenWidgets;
			if (this._connectHandles)  delete this._connectHandles;
		}
	},
	
	/*
	 * ================
	 * Context Table Builder Methods
	 * ================
	 */
	_buildContextTable: function(){
		var messages = this.messages;
		
		//
		// Column Ordering:
		// ================
		//
		// | Artifact Container | Application | Membershrip | Access Control | Process Roles | Actions |
		//
		this._contextTable = new jazz.ui.StyledTable({
			columns: [
				{
					label: messages.artifactContainerLabel,
					width: 300
				},
				{
					label: messages.applicationLabel,
					width: 300
				},
				{
					className: "membership-column", //$NON-NLS-1$
					label: messages.membershipLabel,
					width: 100
				},
				{
					className: "accessControl-column", //$NON-NLS-1$
					label: messages.accessControlLabel,
					width: 100
				},
				{
					label: messages.processRoleLabel
				},
				{
					label: messages.actionsLabel,
					className: "actionsCol" //$NON-NLS-1$
				}
			],
			trim: TrimStyles.PRIMARY_HEADLESS
		}, this.contextTableNode);
			
		this._childrenWidgets.push(this._contextTable);
		
		this._connectHandles.push(
			dojo.connect(this._contextTable.domNode, "onclick", this, this._onMouseClick) //$NON-NLS-1$
		);
		
		
		dojo.forEach(this.contexts, dojo.hitch(this, function(context){
			this._addContextRow(context);
		}));
	},
	
	_addContextRow: function(context){
		var messages = this.messages;
		
		//
		// Column Ordering:
		// ================
		//
		// | Artifact Container | Application | Membershrip | Access Control | Process Roles | Actions |
		//
		var tr = dojo.create("tr", { //$NON-NLS-1$
			className: "containerRow" //$NON-NLS-1$
		}, this._contextTable.tbody);
		
		var contextUri = context.uri;
		tr[ROW_CONTAINER_PROP] = contextUri;
		
		var isMember = false;
		if (this.member.memberUris[contextUri]) {
			isMember = true;
		}
		
		var memberContext = this.member.contextMap[contextUri];
		
		if (memberContext){
			var roleViolations = memberContext.roleViolations;
			var contextRoles = memberContext.rolesSorted;
		}
		 
		//check that a user has membership roles and/or role violations
		var hasViolations = (memberContext && (roleViolations && roleViolations.length > 0));
		var hasRoles = (memberContext && (contextRoles && contextRoles.length > 0));
		
		var errorClass = ""; //$NON-NLS-1$
		if ( this.displayErrors && isMember && hasViolations && hasRoles ) {
			errorClass = "error"; //$NON-NLS-1$
		}
		
		//create the Artifact Container column
		var td = dojo.create("td", { className: "left"}, tr); //$NON-NLS-1$ //$NON-NLS-2$
		var artifactRL = new ResourceLink({
			uri: contextUri,
			label: jazz.util.html.escape(context.label),
			lazyFetch: true
		});
		// The "edit" css class allows us to differentiate between a button click
		// and the edit link click in the onclick handler
		dojo.addClass(artifactRL.domNode, "edit"); //$NON-NLS-1$
		td.appendChild(artifactRL.domNode);
		this._childrenWidgets.push(artifactRL);
		
		//create the Application column
		var td = dojo.create("td", { className: "left " + errorClass}, tr); //$NON-NLS-1$ //$NON-NLS-2$
		if ( this.displayErrors && hasViolations && hasRoles ) {
			var domainAnchor = dojo.create("a", { //$NON-NLS-1$
				className: "j-info " + errorClass, //$NON-NLS-1$
				innerHTML: jazz.util.html.removeScript(context.domains[0].label)
			}, td);
			
			var hpContent = ""; //$NON-NLS-1$
			for(var i=0; i < roleViolations.length; i++){
				hpContent = roleViolations[i].description + "<br/>" //$NON-NLS-1$
			}
			
			var hp = new jazz.ui.HoverPopup({
				around: domainAnchor,
				html: hpContent
			});
			this._childrenWidgets.push(hp);
		} else {
			td.appendChild(document.createTextNode(
				jazz.util.html.removeScript(context.domains[0].label)
			));
		}
		
		var td = dojo.create("td", { //$NON-NLS-1$
			className: "membershipCol " + errorClass //$NON-NLS-1$
		}, tr);
		
		if (hasRoles) {
			var span = dojo.create("span", { //$NON-NLS-1$
				className: "", //$NON-NLS-1$
				innerHTML: "&nbsp;", //$NON-NLS-1$
				title: "" //$NON-NLS-1$
			}, td);
			
			if (this.isPreview && memberContext.isInferred) {
				span.className = "j-hint"; //$NON-NLS-1$
				span.innerHTML = ELLIPSIS_HTML + "&nbsp;"; //$NON-NLS-1$
				dojo.create("img", { //$NON-NLS-1$
					className: "checkmark", //$NON-NLS-1$
					src: GRAY_CHECKMARK_IMG_SRC
				}, span);
				span.title = messages.previewMembershipTitle;
			} else if (!memberContext.isInferred) {
				
				if (this.displayErrors && hasViolations) {
					//display error icon
					dojo.create("img", { //$NON-NLS-1$
						src: ERROR_IMG_SRC,
						className: "statusCol-img" //$NON-NLS-1$
					}, span);
					span.title = messages.errorTitle;
				} else {
					//membership is ok, so display a checkmark
					dojo.create("img", { //$NON-NLS-1$
						className: "checkmark", //$NON-NLS-1$
						src: BLACK_CHECKMARK_IMG_SRC
					}, span);
					span.title = messages.isMemberTitle;
				}
			}
		}
		
		
		
		//create the access control column
		var td = dojo.create("td", {className: errorClass}, tr); //$NON-NLS-1$
		
		if (!this.isPreview && memberContext) {
			if (memberContext.readAccess) {
				dojo.create("img", { //$NON-NLS-1$
					className: "checkmark", //$NON-NLS-1$
					src: BLACK_CHECKMARK_IMG_SRC
				}, td);
			} else {
				td.appendChild(document.createTextNode("-")); //$NON-NLS-1$
			}
		} else {
			td.appendChild(document.createTextNode("-")); //$NON-NLS-1$
		}
		
		//create the process roles column
		var td = dojo.create("td", {className: "left " + errorClass}, tr); //$NON-NLS-1$ //$NON-NLS-2$
		if (memberContext) {
			for(var i=0; i<contextRoles.length; i++) {
				if (contextRoles[i].label === "default") { //$NON-NLS-1$
					continue;
				}
				if (i > 1 || (i==1 && contextRoles[0].label !== "default")) { //$NON-NLS-1$
					td.appendChild(document.createTextNode(", ")); //$NON-NLS-1$
				}
				var content = jazz.util.html.removeScript(contextRoles[i].label);
				
				var roleAnchor = dojo.create("a", { //$NON-NLS-1$
					innerHTML: content,
					className: "j-info " + errorClass  //$NON-NLS-1$
				}, td);
				
				var content = messages.noDescriptionMsg;
				
				if (contextRoles[i].description.length > 0){
					content = jazz.util.html.removeScript(contextRoles[i].description);
				}
				
				var hp = new jazz.ui.HoverPopup({
					around: roleAnchor,
					html: content
				});
				this._childrenWidgets.push(hp);
			}
		} else {
			td.appendChild(document.createTextNode("-")); //$NON-NLS-1$
		}
		
		//create the Actions column
		var td = dojo.create("td", {className: "right"}, tr); //$NON-NLS-1$ //$NON-NLS-2$
		var actionToolbar = this._createActionButtons({
			href: contextUri,
			isMember: isMember,
			row: tr
		});
		tr[ROW_BUTTONS_PROP] = actionToolbar.id;
		var actionButtons = actionToolbar.getChildren();
		
		for (var i=0; i < actionButtons.length; i++) {
			actionButtons[i].connect(actionButtons[i].focusNode, "onfocus", function(event) { //$NON-NLS-1$
				clearTimeout(tr._blurTimeout);
			});
			actionButtons[i].connect(actionButtons[i].focusNode, "onblur", function(event) { //$NON-NLS-1$
				tr._blurTimeout = setTimeout(function() {
					dojo.removeClass(tr, "focused"); //$NON-NLS-1$
				}, 0);
			});
		}
		
		dojo.place(actionToolbar.domNode, td);
		actionToolbar.connect(actionToolbar.domNode, "onfocus", function(event) { //$NON-NLS-1$
			var i;
			var actionButtons = this.getChildren();
			dojo.addClass(tr, "focused"); //$NON-NLS-1$
			for(i = 0; i < actionButtons.length; i++) {
				if (actionButtons[i].isFocusable()) { //$NON-NLS-1$
					actionButtons[i].focus();
					break;
				}
			}
			if (i === actionButtons.length) {
				dojo.removeClass(tr, "focused"); //$NON-NLS-1$
			}
		});
	},
	
	_createActionButtons: function(args){
		var isMember = args.isMember;
		var link = args.href;
		var row = args.row;
		var messages = this.messages;
		
		var simpleToolbar = new Toolbar();
		
		if (isMember && !this.isPreview) {
			this._simpleButtonMap[simpleToolbar.id + "editProcessRoles"] = new Button({ //$NON-NLS-1$
				title: messages.editProcessRolesLabel,
				imgSrc: EDIT_ROLES_SRC,
				onClick: dojo.hitch(this, function(event) {
					dojo.stopEvent(event);
					this._editMemberRoles({
						uri: this._getMemberRolesEditorUri(row[ROW_CONTAINER_PROP]),
						row: row
					});
				})
			});
			dojo.addClass(this._simpleButtonMap[simpleToolbar.id + "editProcessRoles"].focusNode, "editRoles"); //$NON-NLS-1$ //$NON-NLS-2$
			simpleToolbar.addChild(this._simpleButtonMap[simpleToolbar.id + "editProcessRoles"]); //$NON-NLS-1$
		}
		
		this._simpleButtonMap[simpleToolbar.id + "editArtifactContainer"] = new Button({ //$NON-NLS-1$
			title: messages.editContainerLabel,
			href: args.href,
			imgSrc: EDIT_IMG_SRC
		});
		dojo.addClass(this._simpleButtonMap[simpleToolbar.id + "editArtifactContainer"].focusNode, "edit"); //$NON-NLS-1$ //$NON-NLS-2$
		simpleToolbar.addChild(this._simpleButtonMap[simpleToolbar.id + "editArtifactContainer"]); //$NON-NLS-1$
		
		if (isMember){
			this._simpleButtonMap[simpleToolbar.id + "undoRemoveMember"] = new Button({ //$NON-NLS-1$
				title: messages.addMemberAltLabel,
				imgSrc: RESTORE_IMG_SRC,
				onClick: dojo.hitch(this, function(event) {
					dojo.stopEvent(event);
					this._restoreMemberToContainer({
						memberUri: this._getMemberUri(row[ROW_CONTAINER_PROP]),
						row: row
					});
				})
			});
			dojo.addClass(this._simpleButtonMap[simpleToolbar.id + "undoRemoveMember"].focusNode, "undoRemoveMember action"); //$NON-NLS-1$ //$NON-NLS-2$
			this._simpleButtonMap[simpleToolbar.id + "undoRemoveMember"].set("visible", false); //$NON-NLS-1$ //$NON-NLS-2$
			simpleToolbar.addChild(this._simpleButtonMap[simpleToolbar.id + "undoRemoveMember"]); //$NON-NLS-1$
			
			this._simpleButtonMap[simpleToolbar.id + "removeMember"] = new Button({ //$NON-NLS-1$
				title: messages.removeMemberAltLabel,
				imgSrc: REMOVE_IMG_SRC,
				onClick: dojo.hitch(this, function(event) {
					dojo.stopEvent(event);
					this._removeMemberFromContainer({
						memberUri: this._getMemberUri(row[ROW_CONTAINER_PROP]),
						row: row
					});
				})
			});
			dojo.addClass(this._simpleButtonMap[simpleToolbar.id + "removeMember"].focusNode, "removeMember action"); //$NON-NLS-1$ //$NON-NLS-2$
			simpleToolbar.addChild(this._simpleButtonMap[simpleToolbar.id + "removeMember"]); //$NON-NLS-1$
		}
		
		this._childrenWidgets.push(simpleToolbar);
		return simpleToolbar;
	},
		
	_restoreMemberToContainer: function(args) {
		var member = this.member;
		
		var row = args.row;
		var memberUri = args.memberUri; 
		
		var membershipCol = dojo.query(".membershipCol", row)[0]; //$NON-NLS-1$
		var simpleTb = dijit.byId(row[ROW_BUTTONS_PROP]);
		
		var undoButton = this._simpleButtonMap[simpleTb.id+"undoRemoveMember"]; //$NON-NLS-1$
		var deleteMemberButton = this._simpleButtonMap[simpleTb.id+"removeMember"]; //$NON-NLS-1$
		
		undoButton.set("visible", false); //$NON-NLS-1$
		deleteMemberButton.set("visible", true); //$NON-NLS-1$
		deleteMemberButton.focus();
		
		dojo.empty(membershipCol);
		var span = dojo.create("span", { //$NON-NLS-1$
			className: "", //$NON-NLS-1$
			innerHTML: "&nbsp;", //$NON-NLS-1$
			title: "" //$NON-NLS-1$
		}, membershipCol);
		
		//update cur row state
		dojo.removeClass(row, "j-table-restored"); //$NON-NLS-1$
		dojo.create("img", { //$NON-NLS-1$
			className: "checkmark", //$NON-NLS-1$
			src: BLACK_CHECKMARK_IMG_SRC
		}, span);
		
		//remove member from delete request args
		dojo.publish(TOPIC_REMOVE_MEMBER_FROM_DELETE_REQUEST, [{
			member: {
				uri: member.uri,
				memberUris: [ memberUri ]
			}
		}]);
	},
	
	//
	// Manages the UI state of the row for when the member should be removed from
	// the artifact container and then updates the request object to remove the member.
	//
	_removeMemberFromContainer: function(args){
		var member = this.member;
		
		var row = args.row;
		var memberUri = args.memberUri; 
		
		var membershipCol = dojo.query(".membershipCol", row)[0]; //$NON-NLS-1$
		var simpleTb = dijit.byId(row[ROW_BUTTONS_PROP]);
		
		var undoButton = this._simpleButtonMap[simpleTb.id+"undoRemoveMember"]; //$NON-NLS-1$
		var deleteMemberButton = this._simpleButtonMap[simpleTb.id+"removeMember"]; //$NON-NLS-1$
		
		undoButton.set("visible", true); //$NON-NLS-1$
		deleteMemberButton.set("visible", false); //$NON-NLS-1$
		undoButton.focus();
		
		dojo.empty(membershipCol);
		var span = dojo.create("span", { //$NON-NLS-1$
			className: "", //$NON-NLS-1$
			innerHTML: "&nbsp;", //$NON-NLS-1$
			title: "" //$NON-NLS-1$
		}, membershipCol);

		//update cur row state
		dojo.addClass(row, "j-table-restored"); //$NON-NLS-1$
		span.className = "j-hint"; //$NON-NLS-1$
		span.innerHTML = ELLIPSIS_HTML + " -"; //$NON-NLS-1$
		span.title = this.messages.willRemoveMemberTitle;
		
		//add member to delete request args
		dojo.publish(TOPIC_ADD_MEMBER_TO_DELETE_REQUEST, [{
			member: {
				uri: member.uri,
				memberUris: [ memberUri ]
			}
		}]);		
	},
	
	_editMemberRoles: function(args) {
		var callbackUrl = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/lifecycle-project/roles-callback"; //$NON-NLS-1$
		var iframe;
		if (dojo.isIE === 7) {
			iframe = document.createElement("<iframe scrolling='auto' frameborder=0 height='460px' width='582px' name='" + callbackUrl + "'></iframe>") //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			iframe = dojo.create("iframe", { //$NON-NLS-1$
				scrolling: "auto", //$NON-NLS-1$
				frameborder: 0,
				height: "495em", //$NON-NLS-1$
				width: "617em", //$NON-NLS-1$
				name: callbackUrl
			});
		}
		iframe.src = args.uri + "#oslc-core-windowName-1.0"; //$NON-NLS-1$
		dojo.style(iframe, "border", 0); //$NON-NLS-1$
		var dialog = new jazz.ui.Dialog({
			contentNode: iframe,
			width: "52em" //$NON-NLS-1$
		});
		var onloadMethod = dojo.hitch(this, function(dialog) {
			try { // May throw an exception if the frame's location is still a different origin
				if (iframe.contentWindow.location == callbackUrl) {
					var message = iframe.contentWindow.name;
					dialog.close();
					// TODO - refresh the data without doing a full page reload
					var dfd = LpaClient.getMembers({
						project: this.project,
						user: this.member.uri
					});
					dfd.addCallback(dojo.hitch(this, function(result) {
						if (result[0]) {
							this.member = result[0];
							dojo.publish(TOPIC_REFRESH_MEMBER_ROW, [{
								member: this.member
							}]);
							this.refresh();
						}
					}));
				}
			} catch(e) {
				// ignore: access exception when trying to access window name
			};
		}, dialog);
		if (!dojo.isIE) {
			this.connect(iframe, "onload", onloadMethod); //$NON-NLS-1$
		} else {
			iframe.attachEvent("onload", onloadMethod); //$NON-NLS-1$
		}
	},
	
	/*
	 * ================
	 * Member Details Util Methods
	 * ================
	 */
	
	//
	// Get the member resource URI for a given container
	//
	_getMemberUri: function(containerUri){
		return this.member.memberUris[containerUri];
	},
	
	//
	// Get the member resource URI for a given container
	//
	_getMemberRolesEditorUri: function(containerUri){
		return this.member.roleAssignmentsEditorUris[containerUri];
	},
	
	/*
	 * ================
	 * Member Details Callback Methods
	 * ================
	 */
	
	//
	// Callback for when a user clicks on a button or link in 
	// one of the Artifact Container rows.
	//
	_onMouseClick: function(e){
	}

});

})();
