/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.lpa.app.ui.internal.MembersPage"); //$NON-NLS-1$

dojo.require("com.ibm.team.lpa.app.client.LifecycleProjectAdminClient"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$

dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.ui.PaginationControl"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$

dojo.require("com.ibm.team.lpa.common.internal.util"); //$NON-NLS-1$

dojo.require("com.ibm.team.lpa.app.ui.internal.MembersInfoWidget"); //$NON-NLS-1$
dojo.require("com.ibm.team.lpa.app.ui.internal.MembersTable"); //$NON-NLS-1$
dojo.require("com.ibm.team.lpa.app.ui.internal.UserPickerDialog"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.lpa.app.ui.internal", "MembersPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var LpaClient = com.ibm.team.lpa.app.client.LifecycleProjectAdminClient;
var MessageBox = jazz.ui.MessageBox;
var PaginationControl = jazz.ui.PaginationControl;


//
// Topics 
//
var TOPIC_DISPLAY_MSG = "com.ibm.team.lpa.app.ui.internal.MembersPage#message"; //$NON-NLS-1$
var TOPIC_ADD_MEMBER_TO_ADD_REQUEST = "com.ibm.team.lpa.app.ui.internal.MembersPage#addMember"; //$NON-NLS-1$
var TOPIC_ADD_MEMBER_TO_DELETE_REQUEST = "com.ibm.team.lpa.app.ui.internal.MembersPage#addMemberToDelete"; //$NON-NLS-1$
var TOPIC_REMOVE_MEMBER_FROM_DELETE_REQUEST = "com.ibm.team.lpa.app.ui.internal.MembersPage#removeMemberFromDelete"; //$NON-NLS-1$
var TOPIC_REMOVE_MEMBER_PREVIEW_STATE = "com.ibm.team.lpa.app.ui.internal.MembersPage#removeMemberPreviewState"; //$NON-NLS-1$
var TOPIC_REMOVE_MEMBER_FROM_ADD_REQUEST = "com.ibm.team.lpa.app.ui.internal.MembersPage#removeMemberFromAdd"; //$NON-NLS-1$
var TOPIC_SEARCH_USER = "com.ibm.team.lpa.app.ui.internal.MembersPage#searchUsers"; //$NON-NLS-1$

//
// Image Uri Paths
//
var GRAPHICS_FOLDER = "ui/graphics/"; //$NON-NLS-1$
var GRAPHICS_ICONS_FOLDER = GRAPHICS_FOLDER + "icons/"; //$NON-NLS-1$

var REFRESH_IMG_SRC = dojo.moduleUrl("com.ibm.team.lpa.app",GRAPHICS_ICONS_FOLDER + "refresh_16.png"); //$NON-NLS-1$ //$NON-NLS-2$

var PAGINATION_INCREMENTS = [10, 25, 50, 75, 100, 250, 500];

function makeRemoteCall(dfd, callback, errback) {
	dfd.addBoth(function(result) {
		if(result instanceof Error) {
			if (errback) {
				errback(result.jfsHttpException.message);
			} else {
				var msgType = result.jfsHttpException.msgType;
				
				dojo.publish(TOPIC_DISPLAY_MSG, [{
					content: result.jfsHttpException.message,
					type: (msgType && msgType === "info") ? MessageBox.STATUS_TYPES.INFO : MessageBox.STATUS_TYPES.ERROR, //$NON-NLS-1$
					subStatus: result.jfsHttpException.subStatus
				}]);
			}
		} else {
			callback(result);
		}
	});
}


dojo.declare("com.ibm.team.lpa.app.ui.internal.MembersPage", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin],  { //$NON-NLS-1$
	
	templatePath: dojo.moduleUrl("com.ibm.team.lpa.app","ui/internal/templates/MembersPage.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	defaultProjectUri: "", //$NON-NLS-1$
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.lpa.app.ui.internal", "MembersPageMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this.projectEditorMessages = dojo.i18n.getLocalization("com.ibm.team.lpa.app.ui.internal", "ProjectEditorMessages"); //$NON-NLS-1$ //$NON-NLS-2$
		this._refreshIconSrc = REFRESH_IMG_SRC;
		
		this._projectMap = {};
		
		//internal variables that references the page's MembersTable objects
		this._membersTable = null;
		this._previewTable = null;
		
		//internal variables that track the XHR arguments for interacting with members
		this._addMemberRequestArgs = {};
		this._deleteRequestArgs = {};
		
		this._isProjectAreaChanging = false; //helper variable to prevent >1 xhr request to retireve project information
		this._currentSelectedProject = ""; //$NON-NLS-1$
		
		this._loadingPreview = false; //mutex for when we are loading a member preview
		this._blockSaveAction = true;
		
		// properties used to track references that need to be destroyed on table destruction
		this._topics = []; //store references to dojo topics and connects to handle their destruction
		this._childrenWidgets = [];
		this._connectHandles = [];
		
		this._hasChanges = false;
	},
	
	postCreate: function() {
		if (net.jazz.ajax._helpRoot) {
			new jazz.app.ua.PageHoverHelp({ 
				uaKey: "com.ibm.team.lpa.app.lpaMembers" //$NON-NLS-1$
			}, this.titleUA);
		} else {
			new jazz.app.ua.PageHoverHelp({ 
				contentNode: new com.ibm.team.lpa.app.ui.internal.MembersInfoWidget().domNode 
			}, this.titleUA);
		}
		
		this._topics.push(dojo.subscribe(TOPIC_DISPLAY_MSG, this, "displayMessage")); //$NON-NLS-1$
		this._topics.push(dojo.subscribe(TOPIC_ADD_MEMBER_TO_ADD_REQUEST, this, "_buildAddMembersRequest")); //$NON-NLS-1$
		this._topics.push(dojo.subscribe(TOPIC_ADD_MEMBER_TO_DELETE_REQUEST, this, "_addToDeleteMemberRequest")); //$NON-NLS-1$
		this._topics.push(dojo.subscribe(TOPIC_REMOVE_MEMBER_FROM_DELETE_REQUEST, this, "_removeFromDeleteMemberRequest")); //$NON-NLS-1$
		this._topics.push(dojo.subscribe(TOPIC_REMOVE_MEMBER_FROM_ADD_REQUEST, this, "_removeFromAddMemberRequest")); //$NON-NLS-1$
		this._topics.push(dojo.subscribe(TOPIC_REMOVE_MEMBER_PREVIEW_STATE, this, "_removeMemberPreview")); //$NON-NLS-1$
		this._topics.push(dojo.subscribe(TOPIC_SEARCH_USER, this, "onAddUser")); //$NON-NLS-1$
		
		this._refreshTimeout = setTimeout(dojo.hitch(this, this.refresh), 0);
	},
	
	doCommand: function(command){
		this.messageBox.hideMessage();
		if (command.args && command.args.project) {
			this.defaultProjectUri = com.ibm.team.lpa._projectsService + "/" + command.args.project; //$NON-NLS-1$
			if (this._refreshTimeout) {
				clearTimeout(this._refreshTimeout);
			}
			this.refresh();
		}
	},
	
	destroy: function(){
		dojo.forEach(this._childrenWidgets, function(widget){
			widget.destroy();
		});
		
		//unsubscribe from topics
		dojo.forEach(this._topics, dojo.unsubscribe);
		
		//disconnect from events
		dojo.forEach(this._connectHandles, dojo.disconnect);
		
		if (this._membersTable) this._membersTable.destroy();
		delete this._membersTable;
		
		if (this._previewTable) this._previewTable.destroy();
		delete this._previewTable;
		
		delete this._addMemberRequestArgs;
		delete this._deleteRequestArgs;
		
		//run parent destroy() method
		this.inherited(arguments);
	},
	
	refresh: function(){
		this._refreshContent();
		com.ibm.team.lpa.common.internal.util.checkAuth(this.messageBox, dojo.hitch(this, function(response) {
			if (response === 1){
				this._blockSaveAction = false;
				if (
					(this._addMemberRequestArgs.members && this._addMemberRequestArgs.members.length > 0) || (this._deleteRequestArgs.members && this._deleteRequestArgs.members.length > 0) 
				){
					disableSave(false);
				}
				if (this.messageCache){
					this.displayMessage(this.messageCache);	
				}
			} else {
				// TODO: display a message saying the auth failed
			}
		}));
	},
	
	_refreshContent: function() {
		this._clearSaveRequests();
		this.displayEmptyProjectAreasControlMessage(this.messages.loadingProjects);
		this._makeRemoteCall(LpaClient.getProjectsList(), function(result){
			this.displayProjectAreas(result);
		});
	}, 
	
	disableSave: function(disable) {
		this.saveButton.disabled = disable;
		this._hasChanges = !disable;
	},
	
	/*
	 * ================
	 * Project Area Methods
	 * ================
	 */
	clearProjectAreas: function() {
		dojo.empty(this.projectAreas);
		this.projectAreas.disabled = true;
	},
	
	displayEmptyProjectAreasControlMessage: function(label) {
		this.clearProjectAreas();
		this.displayProjectArea("", label); //$NON-NLS-1$
	},
	
	displayMessage: function(args) {
		if(!this._blockSaveAction){
			var isClosable = (args.closable !== undefined) ? args.closable : true;
			if (this._saveContent && this._saveContent.parentNode) {
				// IE will blow away the cached _saveContent if it is in the 
				// MessageBox when we show a new message
				this._saveContent.parentNode.removeChild(this._saveContent);
			}
			var mbArgs = {
				summary: args.content,
				type: args.type,
				closable: isClosable
			};
			if (args.subStatus) {
				mbArgs.details = [{
					message: args.subStatus
				}];
			}
			this.messageBox.showMessage(mbArgs);
		} else {
			this.messageCache = args;
		}
	},
	
	//
	// Collect all of the project areas in this.projects and display them to the
	// user. Once all project areas are displayed, set the UI to display the
	// members table of the default project.
	//
	displayProjectAreas: function(projects) {
		dojo.empty(this.projectAreas);
		if(projects.length === 0) {
			this.displayEmptyProjectAreasControlMessage(this.messages.noLpaProjects);
		}
		dojo.forEach(projects, dojo.hitch(this, function(project){	
			this._projectMap[project.uri] = project;		
			this.displayProjectArea(project.uri, project.label);
		}));
		this.onProjectAreaChange();
	},
	
	//
	// Add a single project area to the projectAreas dropdown box
	//
	// params:
	//	uri: the resource URI of the project area
	//	label: the project resource's display name
	// 
	displayProjectArea: function(uri, label) {
		var option = dojo.create("option", { //$NON-NLS-1$
			value: jazz.util.html.removeScript(uri), 
			innerHTML: jazz.util.html.escape(label)
		}, this.projectAreas);
							
		if (this.defaultProjectUri === uri) {
			this._selectedProjectOption = option;
			option.selected = "true"; //$NON-NLS-1$
		}
		if(this.projectAreas.disabled) {
			this.projectAreas.disabled = false;
		}
	},
	
	/*
	 * ================
	 * Event Handlers
	 * ================
	 */
	
	onAddUser: function(e){
		if (e) {
			dojo.stopEvent(e);
		}
		
		if (!dojo.hasClass(this.addButton, "addButtonDisabled")){ //$NON-NLS-1$
			var picker = new com.ibm.team.lpa.app.ui.internal.UserPickerDialog({});
			this._connectHandles.push(
				dojo.connect(picker, "onOK", this, "_buildAddMembersRequest") //$NON-NLS-1$ //$NON-NLS-2$
			);
		}
	},
	
	onFilter: function(value) {
		if (this._membersTable) {
			var filterIndex = this._membersTable.getMemberIndex();
			for (var i = 0; i < filterIndex.length; i++) {
				var entry = filterIndex[i];
				var displayValue = "none"; //$NON-NLS-1$
				for (var j = 0; j < entry.text.length; j++) {
					if(entry.text[j].indexOf(value.toLowerCase()) >= 0) {
						displayValue = ""; //$NON-NLS-1$
						break;
					}
				}
				dojo.style(entry.uiElement, "display", displayValue); //$NON-NLS-1$
				
				if (dojo.hasClass(entry.uiElement, "double-row")){ //$NON-NLS-1$
					//if the edit box is open, we want to make sure to hide it
					//as along with its parent row
					var editRow = entry.uiElement.parentNode.rows[entry.uiElement.rowIndex - 5];
					dojo.style(editRow, "display", displayValue); //$NON-NLS-1$
				}
			}
		}
	},
	
	//
	// When a user clicks on the refresh icon, clear out the current 
	// MembersTable, get the current list of project members from the server
	// and rebuild the table.
	//
	onMemberTableRefresh: function(){
		var msg = this.beforeUnloadMsg();
		if (msg && !confirm(dojo.string.substitute(msg))) {
			return;
		}
		
		this._clearSaveRequests();
		this._clearPreviewTable();
		
		dojo.addClass(this._zeroMembersNode, "hide"); //$NON-NLS-1$
		dojo.addClass(this._zeroContainersNode, "hide"); //$NON-NLS-1$
		if (this._membersTable) dojo.addClass(this._membersTable.domNode, "hide"); //$NON-NLS-1$
		
		this.onProjectAreaChange();
	},
	
	_showZeroContainersMessage: function(){
		var currentProject = this._getSelectedProject();
		
		dojo.empty(this._configurseArtifactContainerNode);
		dojo.attr(
			this._configurseArtifactContainerNode, 
			"innerHTML",  //$NON-NLS-1$
			dojo.string.substitute(
				this.messages.noContainersMsg,
				[
					net.jazz.ajax._contextRoot + "/lpa/projects/" + encodeURIComponent(currentProject.uri), //$NON-NLS-1$
					jazz.util.html.escape(currentProject.label)
				]
			)
		);
		
		dojo.empty(this._configurseArtifactContainerTipNode);
		dojo.attr(
			this._configurseArtifactContainerTipNode, 
			"innerHTML",  //$NON-NLS-1$
			dojo.string.substitute(
				this.messages.noContainersInstruct,
				[
					net.jazz.ajax._contextRoot + "/lpa/projects/" + encodeURIComponent(currentProject.uri), //$NON-NLS-1$
					jazz.util.html.escape(currentProject.label)
				]
			)
		);
		
		dojo.addClass(this.addButton, "addButtonDisabled"); //$NON-NLS-1$
		dojo.removeClass(this._zeroContainersNode, "hide"); //$NON-NLS-1$
		this._isProjectAreaChanging = false;
	},
	
	_hideZeroContainersMessage: function(){
		dojo.removeClass(this.addButton, "addButtonDisabled"); //$NON-NLS-1$
		dojo.addClass(this._zeroContainersNode, "hide"); //$NON-NLS-1$
	},
	
	//
	// When a project is selected, this method handles the creation of the 
	// main body of content for the Members Page by creating a MembersTable
	// to display information about members of a project.
	//
	onProjectAreaChange: function(){
		var msg = this.beforeUnloadMsg();
		if (msg && !confirm(dojo.string.substitute(msg))) {
			this._selectedProjectOption.selected = "true"; //$NON-NLS-1$
			return;
		}
		
		this._selectedProjectOption = this.projectAreas.children[this.projectAreas.selectedIndex];
				
		if (this._isProjectAreaChanging) {
			// cancel the requests from the previous change
			if (this._projectChangingDfd) {
				this._projectChangingDfd.cancel();
			}
			if (this._membersDfd) {
				this._membersDfd.cancel();
			}
		}
		
		this._currentSelectedProject = this.projectAreas.value;
		this._isProjectAreaChanging = true;
		
		var messages = this.messages;
		
		this._clearSaveRequests();
		this._clearPreviewTable();
		
		if (this._membersTable) {
			this._membersTableNode = this._destroyTable({
				table: this._membersTable,
				node: this._membersTableNode
			});
			this._membersTable = null;
		}
		
        // If the current selected project is empty project, then disable Add Member button 
		// for both members page and the instruction widget. Hide no member message node
		// and show instruction widget.
        if(!this._currentSelectedProject){
			dojo.addClass(this.addButton, "addButtonDisabled"); //$NON-NLS-1$
			dojo.addClass(this._membersInfoWidget.addButton, "addButtonDisabled"); //$NON-NLS-1$
			dojo.addClass(this._noMembersMsgNode, "hide"); //$NON-NLS-1$
			dojo.removeClass(this._zeroMembersNode, "hide"); //$NON-NLS-1$
		}
		
		this._projectChangingDfd = this._getUpdatedProject(dojo.hitch(this, function(currentProject){
			if (!currentProject.label) return;
			this.paginationControl.setTotal(0);
			this._initPaginationIncrementUI();
			
			var loadingMsg = dojo.string.substitute(messages.loadingProjectMembers, [currentProject.label])
			dojo.attr(this._loadingMembersNode, "innerHTML", jazz.util.html.escape(loadingMsg)); //$NON-NLS-1$
			dojo.addClass(this.addButton, "addButtonDisabled"); //$NON-NLS-1$
			dojo.addClass(this._zeroMembersNode, "hide"); //$NON-NLS-1$
			dojo.addClass(this._zeroContainersNode, "hide"); //$NON-NLS-1$
			
			if (currentProject.contexts.length < 1){
				this._showZeroContainersMessage();
				this._isProjectAreaChanging = false;
			} else {
				dojo.addClass(this._zeroContainersNode, "hide"); //$NON-NLS-1$
				dojo.removeClass(this._loadingMembersNode, "hide"); //$NON-NLS-1$
				this._membersDfd = LpaClient.getMembers({
					project: currentProject
				});
				this._makeRemoteCall(this._membersDfd, dojo.hitch(this, function(result){
					if (!dojo.isArray(result)) return;
					dojo.addClass(this._loadingMembersNode, "hide"); //$NON-NLS-1$
					dojo.removeClass(this.addButton, "addButtonDisabled"); //$NON-NLS-1$
					this._hideZeroContainersMessage();
					
					if(result.length === 0){
						dojo.addClass(this._membersTableNode, "hide") //$NON-NLS-1$
						dojo.removeClass(this._membersInfoWidget.addButton, "addButtonDisabled"); //$NON-NLS-1$
						dojo.removeClass(this._noMembersMsgNode, "hide"); //$NON-NLS-1$
						dojo.removeClass(this._zeroMembersNode, "hide"); //$NON-NLS-1$
						this._refreshPaginationUI();
					} else {
						//create the actual table
						dojo.addClass(this._zeroMembersNode, "hide"); //$NON-NLS-1$
						dojo.removeClass(this._membersTableNode, "hide"); //$NON-NLS-1$
						
						this._membersTable = new com.ibm.team.lpa.app.ui.internal.MembersTable({
							columns: this._getTableColumns(),
							members: result,
							project: this._getSelectedProject(),
							refreshCallback: dojo.hitch(this, function() {
								this._updateMemberVisibility(null, 1, this.paginationIncrement);
								this._refreshPaginationUI();
							})
						}, this._membersTableNode);
						
						this._updateMemberVisibility(null, 1, this.paginationIncrement);
						this._refreshPaginationUI();
					}
					
					this._isProjectAreaChanging = false;
				}));
			}
		}));
	},
	
	onSave: function(){
		if (!this._blockSaveAction){
			this._performSave();	
		}
	},
	
	_performSave: function(){
		dojo.publish("com.ibm.team.lpa.app.ui.internal.MembersPage#message", [{ //$NON-NLS-1$
			content: this.messages.updatingMembersMsg,
			type: MessageBox.STATUS_TYPES.INFO
		}]);
		
		this.disableSave(true);
		
		var successCb = dojo.hitch(this, function(result){
			//display "Successfully updated the members of Project <name>"
			dojo.publish("com.ibm.team.lpa.app.ui.internal.MembersPage#message", [{ //$NON-NLS-1$
				content: dojo.string.substitute(
					this.messages.updateSuccessful,
					[this._getSelectedProject().label]
				),
				type: MessageBox.STATUS_TYPES.OK
			}]);
			this.onMemberTableRefresh();
		});
		
		var callback = successCb;
		
		var memberArr = this._getMembersToRemove();
		
		//handle add member request
		if (this._addMemberRequestArgs.members.length > 0) {
			if (memberArr.length > 0){
				//if there is also a request to remove members, chain the requests
				// then display success
				callback = dojo.hitch(this, function(result){
					this._makeRemoteCall(LpaClient.removeMembers(memberArr), successCb);
				});
			}
			
			this._makeRemoteCall(LpaClient.addMembers(this._addMemberRequestArgs), callback);
		} else if (memberArr.length > 0) {
			//handle member removal requests
			this._makeRemoteCall(LpaClient.removeMembers(memberArr), callback);
		}
	},
	
	_getMembersToRemove: function(){
		var deleteReqArgs = this._deleteRequestArgs;
		
		var memberUris = [];
		
		for (var userUri in deleteReqArgs) {
			if (deleteReqArgs[userUri].length > 0){
				memberUris = memberUris.concat(deleteReqArgs[userUri]);
			}
		}
		
		return memberUris;
	},
	
	onUndo: function(e){
		dojo.stopEvent(e);
		this._clearSaveRequests();
		this._clearPreviewTable();
		if (this._membersTable)this._membersTable.refresh();
		
		
	},
	
	
	/*
	 * ================
	 * Member XHR Request Management
	 * ================
	 * 
	 * These methods help manage adding and removing members from the 
	 * request objects that are sent when onSave() is called
	 */
	
	_buildAddMembersRequest: function(newMembers){	
		if(this._addMemberRequestArgs.contexts.length === 0){
			var contexts = this._getSelectedProject().contexts;
			dojo.forEach(contexts, dojo.hitch(this, function(context) {
				this._addMemberRequestArgs.contexts.push(context.uri);
			}));
		}

		dojo.forEach(newMembers, dojo.hitch(this, function(member){
			if (!this._membersTable || !this._membersTable.isAlreadyMember(member) || dojo.hasClass(this._membersTable.domNode, "hide")){ //$NON-NLS-1$
				if (!this._previewTable) {
					this._previewTable = new com.ibm.team.lpa.app.ui.internal.MembersTable({
						columns: this._getTableColumns(),
						displayErrors: false,
						expandDetails: false,
						isPreview: true,
						members: [],
						removeOnDelete: true,
						project: this._getSelectedProject()
					}, this._membersTablePreviewNode);
					
					dojo.addClass(this._previewTable.domNode, "hide"); //$NON-NLS-1$
				}
				
				if (!this._previewTable.isAlreadyMember(member)){
					dojo.removeClass(this.previewMemberNode, "hide"); //$NON-NLS-1$
					dojo.removeClass(this._loadingPreviewNode, "hide"); //$NON-NLS-1$
					
					if (this._membersTable) {
						this._membersTable.setLoadingState({
							userUri: member.uri,
							isLoading: true
						});
					}
					
					this._loadingPreview = true;
					this.disableSave(true);
					
					//popluate the preview data for the member
					if (!member.name) {
						member.name = member.label;
						member.label = decodeURIComponent(member.uri.substring(member.uri.lastIndexOf("/") + 1)); //$NON-NLS-1$
					}
					if (!member.roleAssignmentsEditorUris) member.roleAssignmentsEditorUris = {};
					if (!member.memberUris) member.memberUris = {};
					if (!member.contextMap) member.contextMap = {};
					var curContexts = this._getSelectedProject().contexts;
					for (var i=0; i < curContexts.length; i++) {
						if (!member.contextMap[curContexts[i].uri]) {
							member.contextMap[curContexts[i].uri] = {
								isInferred: true,
								readAccess: true,
								roleViolations: [],
								roles: [{
									id: "default", //$NON-NLS-1$
									label: "default" //$NON-NLS-1$
								}]
							};
						}
						if (!member.roleAssignmentsEditorUris[curContexts[i].uri]) {
							member.roleAssignmentsEditorUris[curContexts[i].uri] = ""; //$NON-NLS-1$
						}
						if (!member.memberUris[curContexts[i].uri]) {
							member.memberUris[curContexts[i].uri] = ""; //$NON-NLS-1$
						}
					}
					if (this._previewTable){
						this._loadingPreview = false;
						this._addMemberRequestArgs.members.push(member.uri);
						this._previewTable.addMember({
							newMember: member
						});
						
						if (this._membersTable) {
							this._membersTable.displayMemberPreview(member);
							this._membersTable.setLoadingState({
								userUri: member.uri,
								isLoading: false
							});
						}
						
						dojo.addClass(this._loadingPreviewNode, "hide"); //$NON-NLS-1$
						dojo.removeClass(this._previewTable.domNode, "hide"); //$NON-NLS-1$
						if (this._addMemberRequestArgs.members && this._addMemberRequestArgs.members.length > 0){
							this.disableSave(this._blockSaveAction);
							var saveMsg = this._getSaveMsg();
							dojo.publish("com.ibm.team.lpa.app.ui.internal.MembersPage#message", [{ //$NON-NLS-1$
								content: saveMsg,
								type: MessageBox.STATUS_TYPES.INFO,
								closable: false
							}]);
						} 
					}
				}
			}
		}));
	},
	
	//
	// Builds the request object that will remove members.  The object is in the form of:
	//
	// 		this._deleteRequestArgs = {
	//			<userUri>: [ memberUri, memberUri, ... ],
	//			<userUri>: [ memberUri, memberUri, ... ],
	//			<userUri>: [ memberUri, memberUri, ... ]
	//		}
	//
	// This method will handle adding a member to the request object.
	//
	// Params:
	//	member: {
	//		uri: {REQUIRED} {String} The resource uri of the user that the member represents
	//		memberUris: {REQUIRED} {String Array} An array of membership resource uris.  
	//					These uris represent the containers that the user is a member of 
	//					and should be removed from.
	//	}
	//
	_addToDeleteMemberRequest: function(args){
		var member = args.member;
		var memberUris = member.memberUris;
		var userUri = member.uri;
		
		var deleteRequestArgs = this._deleteRequestArgs;
		
		//init the member if they aren't already in a state to be removed from the project
		if (!this._deleteRequestArgs[userUri]) this._deleteRequestArgs[userUri] = [];
		
		var memberUrisRequestArr = this._deleteRequestArgs[userUri];
		
		//add the member to the delete request
		for (projectUri in memberUris){
			var memberUri = memberUris[projectUri];
			if (memberUri.length > 0){
				var index = dojo.indexOf(memberUrisRequestArr,memberUri); 
				if (index === -1) {
					memberUrisRequestArr.push(memberUri);
				}	
			}
		}
		
		this.disableSave(this._loadingPreview || this._blockSaveAction);			
		var saveMsg = this._getSaveMsg();
		dojo.publish("com.ibm.team.lpa.app.ui.internal.MembersPage#message", [{ //$NON-NLS-1$
			content: saveMsg,
			type: MessageBox.STATUS_TYPES.INFO,
			closable: false
		}]);
	},
	
	//
	// Handles removing a member from the delete member request object.
	//
	// Params:
	//	member: {
	//		uri: {REQUIRED} {String} The resource uri of the user that the member represents
	//		memberUris: {REQUIRED} {String Array} An array of membership resource uris.  
	//					These uris represent the containers that the user is a member of 
	//					and should be removed from.
	//	}
	//
	_removeFromDeleteMemberRequest: function(args) {
		var member = args.member;
		var memberUris = member.memberUris;
		var userUri = member.uri;
		
		var deleteRequestArgs = this._deleteRequestArgs;
		
		if (deleteRequestArgs[userUri]){
			//remove the member from the request array
			var memberUrisRequestArr = deleteRequestArgs[userUri];
			
			for (projectUri in memberUris){
				var index = dojo.indexOf(memberUrisRequestArr,memberUris[projectUri]);
				if (index !== -1){
					memberUrisRequestArr.splice(index, 1);
				}
			}
			
			if (memberUrisRequestArr.length <= 0){
				delete deleteRequestArgs[userUri];
			}
			
			this._shouldDisableSaveButton();
		}
	},
	
	//
	// Eradicates the preview state of a member by removing their preview state in the main table
	// and removing them from the preview table.
	//
	_removeMemberPreview: function(member){
		var memberObj = (member[0]) ? member[0] : member;
		 
		this._membersTable.removeMemberPreview(memberObj);
		this._previewTable.removeMember(memberObj);
	},
	
	_removeFromAddMemberRequest: function(args){
		var userUri = args.member.uri;
		var shouldRemoveFromRequest = args.shouldRemoveFromRequest;
		
		if (shouldRemoveFromRequest) {
			//remove the member from the request array
			var memberArr = this._addMemberRequestArgs.members;
			var index = dojo.indexOf(memberArr,userUri);
			if (index !== -1){
				memberArr.splice(index, 1);
			}
			
			if (this._membersTable) this._membersTable.removeMemberPreview(args.member);
			
			if (this._addMemberRequestArgs.members && this._addMemberRequestArgs.members.length < 1){
				dojo.addClass(this._previewTable.domNode, "hide"); //$NON-NLS-1$
				dojo.addClass(this.previewMemberNode, "hide"); //$NON-NLS-1$
				
				this._shouldDisableSaveButton();
			} 
		}
	},
	
	/*
	 * ================
	 * Pagination Handling Methods
	 * ================
	 */
	
	_initPaginationIncrementUI: function() {
		//the pagination control has been created already
		if (dojo.hasClass(this.paginationIncrementSelect, "_init")) return; //$NON-NLS-1$
		
		dojo.forEach(PAGINATION_INCREMENTS, dojo.hitch(this, function(inc){
			dojo.create("option", {value: inc, innerHTML: inc}, this.paginationIncrementSelect); //$NON-NLS-1$
		}));
		
		dojo.create("option", {value: 0, innerHTML: jazz.util.html.removeScript(this.messages.showAll)}, this.paginationIncrementSelect); //$NON-NLS-1$
		this.paginationIncrementSelect.value = 50;
		this.paginationIncrement = 50;
		this.connect(this.paginationIncrementSelect, "onchange", dojo.hitch(this, function() { //$NON-NLS-1$
			this.paginationIncrement = parseInt(this.paginationIncrementSelect.value);
			this._updateMemberVisibility(null, 1, this.paginationIncrement);
			this._refreshPaginationUI();
		}));
		
		dojo.addClass(this.paginationIncrementSelect, "_init"); //$NON-NLS-1$
	},
	
	_refreshPaginationUI: function() {
		var memberRowLength = 0;
		
		if (this._membersTable && !dojo.hasClass(this._membersTable.domNode, "hide")){ //$NON-NLS-1$
			var memberRowLength = dojo.query(".memberRow").length; //$NON-NLS-1$
		}
		
		var increment = (this.paginationIncrement <= 0 || memberRowLength < 1 ) ? memberRowLength : this.paginationIncrement;
		this.paginationControl.set("increment", increment); //$NON-NLS-1$
		this.paginationControl.setTotal(memberRowLength);
	},
	
	//
	// Callback method for when the user clicks on "Previous" or "Next" on the jazz.ui.PaginationNode.
	// All arguments are required for the pagination control's callback to work properly
	//
	_updateMemberVisibility: function(widget, start, end) {
		var rowList = dojo.query(".memberRow"); //$NON-NLS-1$
		var rowListLength = rowList.length;
		
		if (end <= 0) {
			end = rowListLength;
		}
		for (var i = 1; i <= rowListLength; i++) {
			if (i >= start && i <= end) {
				dojo.removeClass(rowList[i-1], "hide"); //$NON-NLS-1$
			} else {
				if (dojo.hasClass(rowList[i-1], "double-row")){ //$NON-NLS-1$
					this._membersTable._displayUserDetails({
						row: rowList[i - 1]
					});
				}
				dojo.addClass(rowList[i-1], "hide"); //$NON-NLS-1$
			}
		}
	},
	
	/*
	 * ================
	 * MembersPage Util Methods
	 * ================
	 */
	_clearSaveRequests: function(){
		if (!this._blockSaveAction){
			this.messageBox.hideMessage();	
		}
		this.disableSave(true);
		this._deleteRequestArgs = {};
		this._addMemberRequestArgs.contexts = [];
		this._addMemberRequestArgs.members = [];
	},
	
	beforeUnloadMsg: function(){
		if (this._hasChanges === true) {
			return this.projectEditorMessages.unloadMsg;
		}
	},
	
	//
	// Remove and hide the Preview members table
	//
	_clearPreviewTable: function(){
		if (this._previewTable) {
			dojo.addClass(this.previewMemberNode, "hide"); //$NON-NLS-1$
			this._membersTablePreviewNode = this._destroyTable({
				table: this._previewTable,
				node: this._membersTablePreviewNode
			});
			this._previewTable = null;
		}
	},
	
	//
	// Checks to see if the save button should be disabled and if the save message should be hidden.
	// If the UI elements should be disabled, this method also handles the process of doing so.
	// 
	_shouldDisableSaveButton: function(){
		var deleteRequestObjSize = this._getSizeOfDeleteRequestObj();
		
		if ((this._addMemberRequestArgs.members && this._addMemberRequestArgs.members.length < 1) && (deleteRequestObjSize < 1) ){
			this.disableSave(true);
			this.messageBox.hideMessage();
		}
	},
	
	//
	// Returns the number of elements contained in the map that the delete request object represents
	//
	_getSizeOfDeleteRequestObj: function(){
		return this._getSizeOfObject(this._deleteRequestArgs);
	},
	
	//
	// Returns the number of properties contained
	//
	_getSizeOfObject: function(obj){
		var size = 0;
		
		for(var key in obj){
			if (obj.hasOwnProperty(key)) size++;
		}
		
		return size;
	},
	
	//
	// Wrapper to help the destruction of Member Tables
	//
	// params:
	//	table: a reference to a Members Table widget
	//	node: the domNode that contains the widget
	//
	// return
	//	a reference to the new dom node that will contain future Members Table widgets
	//
	_destroyTable: function(args){
		var table = args.table;
		var node = args.node;
		
		node = dojo.create("div", {}); //$NON-NLS-1$
		table.domNode.parentNode.replaceChild(node, table.domNode);
		table.destroy();
		
		return node;
	},
	
	//
	// Given a string, extract all capital letters to create an abrreviation.
	// e.g. "Quality Management" becomes "QM"
	//
	_createAbbreviation: function(str){
		var abbrev = [];
		
		var length = str.length;
		for(var i=0; i<length; i++){
			//if char is a capital letter and NOT a space, extract it
			if(str.charCodeAt(i) < 91 && str.charCodeAt(i) !== 32 ){
				abbrev.push(str.charAt(i));
			}
		}
		return abbrev.join(''); //$NON-NLS-1$
	},
	
	_getTableColumns: function(){
		//
		// Column Ordering:
		// ================
		//
		// | Name | User ID | {Domain} | {Domain} | ... | {Status} | Actions |
		//
		
		var messages = this.messages;
		//dynamically build up the columns names.
		this._columns = [
			{
				className: "name-column", //$NON-NLS-1$
				label: messages.nameLabel,
				sortable: true
			},
			{
				className: "userId-column", //$NON-NLS-1$
				label: messages.userIdLabel,
				sortable: true
			}
		];
		
		var project = this._getSelectedProject();
		var abbrevArr = [];
		var containerNum = 1;
		
		dojo.forEach(project.contexts, dojo.hitch(this, function(context, index){
			var abbrev = this._createAbbreviation(
				context.domains[0].label
			);
			
			if (index > 0 && abbrevArr[index - 1] === abbrev) {
				abbrevArr[index - 1] += " (" + containerNum + ")"; //$NON-NLS-1$ //$NON-NLS-2$
				containerNum++;
				abbrev += " (" + containerNum + ")"; //$NON-NLS-1$ //$NON-NLS-2$
				containerNum++;
			} else if (index > 0 && abbrevArr[index - 1] !== abbrev) {
				containerNum = 1;
			}
			
			abbrevArr.push(abbrev);
		}));
		
		dojo.forEach(abbrevArr, dojo.hitch(this, function(abbrev){
			this._columns.push({
				className: "domain-column", //$NON-NLS-1$
				label: abbrev
			});
		}))
		
		this._columns.push({
			className: "status-column", //$NON-NLS-1$
				label: messages.statusLabel
		});
		
		this._columns.push({
			className: "action-column", //$NON-NLS-1$
			label: messages.actionsLabel
		});
		//end column building.
		
		return this._columns;
	},
	
	
	_getSelectedProject: function() {
		return this._projectMap[this._currentSelectedProject];
	},
	
	//
	// Lazily load a single project's data from the server 
	//
	_getUpdatedProject: function(cb){
		if (this._currentSelectedProject) {
			var dfd = LifecycleProjectAdminClient.getProject(this._currentSelectedProject);
			this._makeRemoteCall(dfd, dojo.hitch(this, function(result){
				this._projectMap[this._currentSelectedProject] = result;
				cb(result);
				return;
			}));
			return dfd;
		}
	},
	
	_getSaveMsg: function(){
		var saveMsg = dojo.string.substitute(
			this.messages.saveMsg, 
			[this._getSelectedProject().label]
		);
		if (!this._saveContent){
			//build up the HTML content of the message
			var div = dojo.create("div", {}); //$NON-NLS-1$
			this._saveMsgSpan = dojo.create(
				"span",  //$NON-NLS-1$
				{innerHTML: jazz.util.html.escape(saveMsg), id: "saveMsgSpan"},  //$NON-NLS-1$
				div
			);
			var undoButton = dojo.create(
				"a", {innerHTML: this.messages.undoLabel, className: "undoButton"}, div //$NON-NLS-1$ //$NON-NLS-2$
			);
			this._connectHandles.push(
				dojo.connect(undoButton, "onclick", this, this.onUndo) //$NON-NLS-1$
			);
			
			//cache the message
			this._saveContent = div;
		} else {
			this._saveMsgSpan.innerHTML = jazz.util.html.escape(saveMsg);
		}
		
		return this._saveContent;
	},
	
	_makeRemoteCall: function(dfd, callback) {
		makeRemoteCall(dfd, dojo.hitch(this, callback));
	}
});

})();